@@ -232,7 +232,6 @@ static char shellStr[30];
 
 void KstJS::showConsole() {
 #ifdef KST_HAVE_READLINE
-  _showAction->setText(i18n("Hide &JavaScript Console"));
   if (!_konsolePart) {
     strcpy(shellStr, "SHELL=kstcmd");
     putenv(shellStr);
@@ -254,7 +253,7 @@ void KstJS::showConsole() {
     _splitter->moveToLast(p->widget());
     app()->setCentralWidget(_splitter);
 
-    connect(p, SIGNAL(processExited()), this, SLOT(shellExited()));
+    connect(p, SIGNAL(destroyed()), this, SLOT(shellExited()));
     _konsolePart = p;
   }
   _splitter->show();
@@ -266,7 +265,6 @@ void KstJS::showConsole() {
 void KstJS::shellExited() {
 #ifdef KST_HAVE_READLINE
   _showAction->setChecked(false);
-  _showAction->setText(i18n("Show &JavaScript Console"));
   _konsolePart = 0L;
   QTimer::singleShot(0, this, SLOT(restoreUI())); // konsole crashes otherwise
 #endif
@@ -285,7 +283,6 @@ void KstJS::restoreUI() {
 
 void KstJS::hideConsole() {
 #ifdef KST_HAVE_READLINE
-  _showAction->setText(i18n("Show &JavaScript Console"));
   if (_konsolePart) {
     _konsolePart->widget()->hide();
   }
@@ -326,10 +323,13 @@ void KstJS::loadScript() {
 
 
 void KstJS::load(QDomElement& e) {
+  Q_UNUSED(e)
 }
 
 
 void KstJS::save(QTextStream& ts, const QString& indent) {
+  Q_UNUSED(ts)
+  Q_UNUSED(indent)
 }
 
 
