@@ -269,18 +269,17 @@ static KstRVector *GetOrCreateVector(const QString& field, KstDataSourcePtr file
     }
   }
 
-  V = new KstRVector(file, field, KST::suggestVectorName(field), in.f, in.n, in.skip, in.doskip, in.doave);
+  V = new KstRVector(file, field, KstObjectTag(KST::suggestVectorName(field), file->tag(), false), in.f, in.n, in.skip, in.doskip, in.doave);
   if (!V->isValid()) {
     if (file->fileType() == "stdin") {
       startupErrors.append(i18n("Failed to create vector '%1' from file '%2'.  Trying again later.").arg(field).arg(file->fileName()));
-      KST::addVectorToList(V);
     } else {
       startupErrors.append(i18n("Failed to create vector '%1' from file '%2'.").arg(field).arg(file->fileName()));
       V = 0L;
     }
   } else {
-    KST::addVectorToList(V);
   }
+    
 
   return V;
 }
@@ -362,6 +361,8 @@ int main(int argc, char *argv[]) {
                       "",
                       "tskisner.public@gmail.com",
                       "");
+  aboutData.setTranslator(I18N_NOOP("_: NAME OF TRANSLATORS\nYour names"), 
+                          I18N_NOOP("_: EMAIL OF TRANSLATORS\nYour emails"));
 
   KCmdLineArgs::init( argc, argv, &aboutData );
   KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
@@ -381,7 +382,6 @@ int main(int argc, char *argv[]) {
     KstApp *kst = new KstApp;
     InType in;
     QColor color;
-    KstDataSourcePtr file;
     QCStringList ycolList;
     QCStringList matrixList;
     QCStringList yEqList;
@@ -477,7 +477,7 @@ int main(int argc, char *argv[]) {
               eqS = *eq_i;
               if (NoVectorEq(eqS)) {
                 KstEquationPtr eq = new KstEquation(KST::suggestEQName(eqS), eqS, min, max, n);
-                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tagName(), true),
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tag(), true),
                                         eq->vX(), eq->vY(), 0L, 0L, 0L, 0L,
                                         KstColorSequence::next(vcurves,plot->backgroundColor()));
                 KST::dataObjectList.lock().writeLock();
@@ -513,7 +513,7 @@ int main(int argc, char *argv[]) {
         } else {
           fullPath = args->arg(i_file);
         }
-        file = KstDataSource::loadSource(fullPath);
+        KstDataSourcePtr file = KstDataSource::loadSource(fullPath);
         if (file) {
           if (!file->isValid() || file->isEmpty()) {
             kstdError() << i18n("No data in file %1.  Trying to continue...").arg(args->arg(i_file)) << endl;
@@ -541,7 +541,7 @@ int main(int argc, char *argv[]) {
                 if (yvector) {
                   // make the curves
                   color = KstColorSequence::next(vcurves,plot->backgroundColor());
-                  curve = new KstVCurve(KST::suggestCurveName(yvector->field(), false),
+                  curve = new KstVCurve(KST::suggestCurveName(yvector->tag(), false), // FIXME: this was yvector->field(), is this right?
                                       KstVectorPtr(xvector), KstVectorPtr(yvector),
                                       0L, 0L, 0L, 0L, color);
                   if (in.has_points) {
@@ -563,7 +563,7 @@ int main(int argc, char *argv[]) {
                   plot->addCurve(curve.data());
 
                   if (in.sep_plots) {
-                    plot->setTagName(curve->tagName());
+                    plot->setTagName(curve->tag());
                     i_plot++;
                     if (i_plot < in.n_plots) {
                       plot = *plist.at(i_plot);
@@ -599,7 +599,7 @@ int main(int argc, char *argv[]) {
               }
 
               if (eq) {
-                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tagName(), true),
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tag(), true),
                                       eq->vX(), eq->vY(), 0L, 0L, 0L, 0L,
                                       KstColorSequence::next(vcurves,plot->backgroundColor()));
                 KST::dataObjectList.lock().writeLock();
@@ -609,7 +609,7 @@ int main(int argc, char *argv[]) {
                 plot->addCurve(vc.data());
 
                 if (in.sep_plots) {
-                  plot->setTagName(eq->tagName());
+                  plot->setTagName(eq->tag());
                   i_plot++;
                   if (i_plot <in.n_plots) {
                     plot = *plist.at(i_plot);
@@ -627,10 +627,10 @@ int main(int argc, char *argv[]) {
               if (yvector) {
                 color = KstColorSequence::next(vcurves,plot->backgroundColor());
 
-                psd = new KstPSD( KST::suggestPSDName(yvector->field()),
+                psd = new KstPSD( KST::suggestPSDName(yvector->tag()), // FIXME: this was yvector->field(), is this right?
                     KstVectorPtr(yvector), in.rate, true, in.len,
                     true, true, in.VUnits, in.RUnits, WindowOriginal);
-                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(psd->tagName(), true),
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(psd->tag(), true),
                     psd->vX(), psd->vY(),
                     0L, 0L, 0L, 0L, color);
                 if (in.has_points) {
@@ -644,7 +644,7 @@ int main(int argc, char *argv[]) {
                 plot->addCurve(vc.data());
 
                 if (in.sep_plots) {
-                  plot->setTagName(psd->tagName());
+                  plot->setTagName(psd->tag());
                   i_plot++;
                   if (i_plot <in.n_plots) {
                     plot = *plist.at(i_plot);
@@ -666,9 +666,9 @@ int main(int argc, char *argv[]) {
 
                 KstHistogram::AutoBin(KstVectorPtr(yvector), &N, &max, &min);
 
-                hs = new KstHistogram(KST::suggestHistogramName(yvector->field()),
+                hs = new KstHistogram(KST::suggestHistogramName(yvector->tag()), // FIXME: this was yvector->field(), is this right?
                     KstVectorPtr(yvector), min, max, N, KST_HS_NUMBER);
-                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(hs->tagName(), true),
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(hs->tag(), true),
                     hs->vX(), hs->vY(),
                     0L, 0L, 0L, 0L, color);
 
@@ -684,7 +684,7 @@ int main(int argc, char *argv[]) {
                 plot->addCurve(vc.data());
 
                 if (in.sep_plots) {
-                  plot->setTagName(hs->tagName());
+                  plot->setTagName(hs->tag());
                   i_plot++;
                   if (i_plot < in.n_plots) {
                     plot = *plist.at(i_plot);
@@ -700,14 +700,14 @@ int main(int argc, char *argv[]) {
               if (!file->isValidMatrix(*mat_i)) {
                 startupErrors.append(i18n("Failed to create matrix '%1' from file '%2'.").arg(*mat_i).arg(file->fileName()));
               }
-              KstRMatrixPtr matrix = new KstRMatrix(file, *mat_i, tag_name, 
+              KstRMatrixPtr matrix = new KstRMatrix(file, *mat_i,
+                  KstObjectTag(tag_name, file->tag()),
                   0,0,-1,-1,false,false,0);
                   // xStart, yStart, xNumSteps, yNumSteps, 
                   //doAve, doSkip, skip);
-              KST::addMatrixToList(KstMatrixPtr(matrix));
               
               // Time to create the image from the matrix
-              tag_name = KST::suggestImageName(matrix->tagName());
+              tag_name = KST::suggestImageName(matrix->tag());
               QStringList palList = KPalette::getPaletteList();
               QString pal;
               if (palList.contains("IDL 13 RAINBOW")) {
@@ -726,7 +726,7 @@ int main(int argc, char *argv[]) {
               image = 0L; // drop the reference
               
               if (in.sep_plots) {
-                plot->setTagName(matrix->tagName());
+                plot->setTagName(matrix->tag());
                 i_plot++;
                 if (i_plot < in.n_plots) {
                   plot = *plist.at(i_plot);
