@@ -35,220 +35,7 @@
 #include "kstscalar.h"
 #include "stdinsource.h"
 
-
-// Eventually this will move to another file but I leave it here until then
-// to avoid confusion between the function plugins and Kst applicaton plugins.
-namespace KST {
-  class Plugin : public KstShared {
-    public:
-      Plugin(KService::Ptr svc) : KstShared(), service(svc), _lib(0L) {
-        assert(service);
-        _plugLib = service->property("X-Kst-Plugin-Library").toString();
-        //kstdDebug() << "Create plugin " << (void*)this << " " << service->property("Name").toString() << endl;
-      }
-
-      virtual ~Plugin() {
-        //kstdDebug() << "Destroy plugin " << (void*)this << " " << service->property("Name").toString() << endl;
-        if (_lib) {
-          _lib->unload();
-        }
-      }
-
-      KstDataSource *create(KConfig *cfg, const QString& filename, const QString& type = QString::null) const {
-        KstDataSource *(*sym)(KConfig*, const QString&, const QString&) = (KstDataSource*(*)(KConfig*, const QString&, const QString&))symbol("create");
-        if (sym) {
-          //kstdDebug() << "Trying to create " << filename << " type=" << type << " with " << service->property("Name").toString() << endl;
-          KstDataSource *ds = (sym)(cfg, filename, type);
-          if (ds) {
-            ds->_source = service->property("Name").toString();
-          }
-          //kstdDebug() << (ds ? "SUCCESS" : "FAILED") << endl;
-          return ds;
-        }
-
-        return 0L;
-      }
-
-      KstDataSource *create(KConfig *cfg, const QString& filename, const QString& type, const QDomElement& e) const {
-        KstDataSource *(*sym)(KConfig*, const QString&, const QString&, const QDomElement&) = (KstDataSource*(*)(KConfig*, const QString&, const QString&, const QDomElement&))symbol("load");
-        if (sym) {
-          //kstdDebug() << "Trying to create " << filename << " type=" << type << " with " << service->property("Name").toString() << endl;
-          KstDataSource *ds = (sym)(cfg, filename, type, e);
-          if (ds) {
-            ds->_source = service->property("Name").toString();
-          }
-          //kstdDebug() << (ds ? "SUCCESS" : "FAILED") << endl;
-          return ds;
-        } else {
-          KstDataSource *(*sym)(KConfig*, const QString&, const QString&) = (KstDataSource*(*)(KConfig*, const QString&, const QString&))symbol("create");
-          if (sym) {
-            KstDataSource *ds = (sym)(cfg, filename, type);
-            if (ds) {
-              ds->_source = service->property("Name").toString();
-            }
-            return ds;
-          }
-        }
-
-        return 0L;
-      }
-      
-      QStringList matrixList(KConfig *cfg, const QString& filename, const QString& type = QString::null, QString *typeSuggestion = 0L, bool *complete = 0L) const {
-        
-        QStringList (*sym)(KConfig*, const QString&, const QString&, QString*, bool*) = (QStringList(*)(KConfig*, const QString&, const QString&, QString*, bool*))symbol("matrixList");
-        if (sym) {
-          return (sym)(cfg, filename, type, typeSuggestion, complete);  
-        }
-        // fallback incase the helper isn't implemented
-        //  (note: less efficient now)
-        KstDataSourcePtr ds = create(cfg, filename, type);
-        if (ds) {
-          QStringList rc = ds->matrixList();
-          if (typeSuggestion) {
-            *typeSuggestion = ds->fileType();
-          }
-          if (complete) {
-            *complete = ds->fieldListIsComplete();
-          }
-          return rc;
-        }
-        return QStringList();
-      }
-
-      QStringList fieldList(KConfig *cfg, const QString& filename, const QString& type = QString::null, QString *typeSuggestion = 0L, bool *complete = 0L) const {
-        QStringList (*sym)(KConfig*, const QString&, const QString&, QString*, bool*) = (QStringList(*)(KConfig*, const QString&, const QString&, QString*, bool*))symbol("fieldList");
-        if (sym) {
-          return (sym)(cfg, filename, type, typeSuggestion, complete);
-        }
-
-        // fallback incase the helper isn't implemented
-        //  (note: less efficient now)
-        KstDataSourcePtr ds = create(cfg, filename, type);
-        if (ds) {
-          QStringList rc = ds->fieldList();
-          if (typeSuggestion) {
-            *typeSuggestion = ds->fileType();
-          }
-          if (complete) {
-            *complete = ds->fieldListIsComplete();
-          }
-          return rc;
-        }
-
-        return QStringList();
-      }
-
-      int understands(KConfig *cfg, const QString& filename) const {
-        int (*sym)(KConfig*, const QString&) = (int(*)(KConfig*, const QString&))symbol("understands");
-        if (sym) {
-          //kstdDebug() << "Checking if " << service->property("Name").toString() << " understands " << filename << endl;
-          int rc = (sym)(cfg, filename);
-          //kstdDebug() << "result: " << rc << endl;
-          return rc;
-        }
-
-        return 0;
-      }
-
-      bool supportsTime(KConfig *cfg, const QString& filename) const {
-        bool (*sym)(KConfig*, const QString&) = (bool(*)(KConfig*, const QString&))symbol("supportsTime");
-        if (sym) {
-          bool rc = (sym)(cfg, filename);
-          return rc;
-        }
-
-        return false;
-      }
-
-      bool provides(const QString& type) const {
-        return provides().contains(type);
-      }
-
-      QStringList provides() const {
-        QStringList (*sym)() = (QStringList(*)())symbol("provides");
-        if (sym) {
-          //kstdDebug() << "Checking if " << service->property("Name").toString() << " provides " << type << endl;
-          return (sym)();
-        }
-
-        return QStringList();
-      }
-
-      Q_UINT32 key() const {
-        Q_UINT32 (*sym)() = (Q_UINT32(*)())symbol("key");
-        if (sym) {
-          return (sym)();
-        }
-
-        return Q_UINT32();
-      }
-
-      bool hasConfigWidget() const {
-        return 0L != symbol("widget");
-      }
-
-      KstDataSourceConfigWidget *configWidget(KConfig *cfg, const QString& filename) const {
-        QWidget *(*sym)(const QString&) = (QWidget *(*)(const QString&))symbol("widget");
-        if (sym) {
-          QWidget *rc = (sym)(filename);
-          KstDataSourceConfigWidget *cw = dynamic_cast<KstDataSourceConfigWidget*>(rc);
-          if (cw) {
-            cw->setConfig(cfg);
-            return cw;
-          }
-          if (rc) {
-            KstDebug::self()->log(i18n("Error in plugin %1: Configuration widget is of the wrong type.").arg(service->property("Name").toString()), KstDebug::Error);
-            delete rc;
-          }
-        }
-
-        return 0L;
-      }
-
-      KService::Ptr service;
-
-    private:
-      void *symbol(const QString& sym) const {
-        if (!loadLibrary()) {
-          return 0L;
-        }
-
-        QCString s = QFile::encodeName(sym + "_" + _plugLib);
-        if (_lib->hasSymbol(s)) {
-          return _lib->symbol(s);
-        }
-        return 0L;
-      }
-
-      bool loadLibrary() const {
-        assert(service);
-        if (_lib) {
-          return true;
-        }
-
-        QCString libname = QFile::encodeName(QString("kstdata_") + _plugLib);
-        _lib = KLibLoader::self()->library(libname);
-        if (!_lib) {
-          KstDebug::self()->log(i18n("Error loading data-source plugin [%1]: %2").arg(libname).arg(KLibLoader::self()->lastErrorMessage()), KstDebug::Error);
-          return false;
-        }
-
-        if (key() != KST_CURRENT_DATASOURCE_KEY) {
-          KstDebug::self()->log(i18n("Error loading data-source plugin [%1]: %2").arg(libname).arg(i18n("Plugin is too old and needs to be recompiled.")), KstDebug::Error);
-          return false;
-        }
-        return true;
-      }
-
-      QString _plugLib;
-      // mutable so we can lazy load the library, but at the same time
-      // use const iterators and provide a nice const interface
-      mutable KLibrary *_lib;
-  };
-
-  typedef QValueList<KstSharedPtr<KST::Plugin> > PluginInfoList;
-}
-
+#include "kstdataplugin.h"
 
 static KConfig *kConfigObject = 0L;
 static QMap<QString,QString> urlMap;
@@ -327,7 +114,7 @@ static void scanPlugins() {
       }
     }
 
-    KstSharedPtr<KST::Plugin> p = new KST::Plugin(*it);
+    KstSharedPtr<KST::Plugin> p = new KST::DataSourcePlugin(*it);
     tmpList.append(p);
   }
 
@@ -356,7 +143,7 @@ QStringList KstDataSource::pluginList() {
 namespace {
 class PluginSortContainer {
   public:
-    KstSharedPtr<KST::Plugin> plugin;
+    KstSharedPtr<KST::DataSourcePlugin> plugin;
     int match;
     int operator<(const PluginSortContainer& x) const {
       return match > x.match; // yes, this is by design.  biggest go first
@@ -370,27 +157,33 @@ class PluginSortContainer {
 
 static QValueList<PluginSortContainer> bestPluginsForSource(const QString& filename, const QString& type) {
   QValueList<PluginSortContainer> bestPlugins;
-  scanPlugins();
+  if (pluginInfo.isEmpty()) {
+    scanPlugins();
+  }
 
   KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
 
   if (!type.isEmpty()) {
     for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
-      if ((*it)->provides(type)) {
-        PluginSortContainer psc;
-        psc.match = 100;
-        psc.plugin = *it;
-        bestPlugins.append(psc);
-        return bestPlugins;
+      if (KST::DataSourcePlugin *p = kst_cast<KST::DataSourcePlugin>(*it)) {
+        if (p->provides(type)) {
+          PluginSortContainer psc;
+          psc.match = 100;
+          psc.plugin = p;
+          bestPlugins.append(psc);
+          return bestPlugins;
+        }
       }
     }
   }
 
   for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
     PluginSortContainer psc;
-    if ((psc.match = (*it)->understands(kConfigObject, filename)) > 0) {
-      psc.plugin = *it;
-      bestPlugins.append(psc);
+    if (KST::DataSourcePlugin *p = kst_cast<KST::DataSourcePlugin>(*it)) {
+      if ((psc.match = p->understands(kConfigObject, filename)) > 0) {
+        psc.plugin = p;
+        bestPlugins.append(psc);
+      }
     }
   }
 
@@ -406,6 +199,15 @@ static KstDataSourcePtr findPluginFor(const QString& filename, const QString& ty
   for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
     KstDataSourcePtr plugin = (*i).plugin->create(kConfigObject, filename, QString::null, e);
     if (plugin) {
+      // restore tag if present
+      QDomNodeList l = e.elementsByTagName("tag");
+      if (l.count() > 0) {
+        QDomElement e2 = l.item(0).toElement();
+        if (!e2.isNull()) {
+          kstdDebug() << "Restoring tag " << e2.text() << " to KstDataSource" << endl;
+          plugin->setTagName(KstObjectTag::fromString(e2.text()));
+        }
+      }
       return plugin;
     }
   }
@@ -465,8 +267,10 @@ KstDataSourceConfigWidget* KstDataSource::configWidgetForPlugin(const QString& p
   KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
 
   for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
-    if ((*it)->service->property("Name").toString() == plugin) {
-      return (*it)->configWidget(kConfigObject, QString::null);
+    if (KST::DataSourcePlugin *p = kst_cast<KST::DataSourcePlugin>(*it)) {
+      if (p->service->property("Name").toString() == plugin) {
+        return p->configWidget(kConfigObject, QString::null);
+      }
     }
   }
 
@@ -577,7 +381,7 @@ QStringList KstDataSource::matrixListForSource(const QString& filename, const QS
 
 
 KstDataSourcePtr KstDataSource::loadSource(QDomElement& e) {
-  QString filename, type;
+  QString filename, type, tag;
 
   QDomNode n = e.firstChild();
   while (!n.isNull()) {
@@ -609,20 +413,65 @@ KstDataSource::KstDataSource(KConfig *cfg, const QString& filename, const QStrin
   Q_UNUSED(type)
   _valid = false;
   _reusable = true;
-  _numFramesScalar = new KstScalar(QString("%1-frames").arg(filename));
+  _writable = false;
+
+  QString shortFilename = filename;
+  while (shortFilename.at(shortFilename.length() - 1) == '/') {
+    shortFilename.truncate(shortFilename.length() - 1);
+  }
+  shortFilename = shortFilename.section('/', -1);
+  QString tn = i18n("DS-%1").arg(shortFilename);
+  int count = 1;
+
+  KstObject::setTagName(KstObjectTag(tn, KstObjectTag::globalTagContext));  // are DataSources always top-level?
+  while (KstData::self()->dataSourceTagNameNotUnique(tagName(), false)) {
+    KstObject::setTagName(KstObjectTag(tn + QString("-%1").arg(count++), KstObjectTag::globalTagContext));  // are DataSources always top-level?
+  }
+
+  _numFramesScalar = new KstScalar(KstObjectTag("frames", tag()));
   // Don't set provider - this is always up-to-date
 }
 
 
 KstDataSource::~KstDataSource() {
+//  kstdDebug() << "KstDataSource destructor: " << tag().tagString() << endl;
   KST::scalarList.lock().writeLock();
+//  kstdDebug() << "  removing numFrames scalar" << endl;
   KST::scalarList.remove(_numFramesScalar);
   KST::scalarList.lock().unlock();
+
+//  kstdDebug() << "  removing metadata strings" << endl;
+  KST::stringList.lock().writeLock();
+  KST::stringList.setUpdateDisplayTags(false);
+  for (QDictIterator<KstString> it(_metaData); it.current(); ++it) {
+//    kstdDebug() << "    removing " << it.current()->tag().tagString() << endl;
+    KST::stringList.remove(it.current());
+  }
+  KST::stringList.setUpdateDisplayTags(true);
+  KST::stringList.lock().unlock();
+
   _numFramesScalar = 0L;
 }
 
 
+void KstDataSource::setTagName(const KstObjectTag& in_tag) {
+  if (in_tag == tag()) {
+    return;
+  }
+
+  KstObject::setTagName(in_tag);
+  _numFramesScalar->setTagName(KstObjectTag("frames", tag()));
+  for (QDictIterator<KstString> it(_metaData); it.current(); ++it) {
+    KstObjectTag stag = it.current()->tag();
+    stag.setContext(tag().fullTag());
+    it.current()->setTagName(stag);
+  }
+}
+
+
 KstObject::UpdateType KstDataSource::update(int u) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   Q_UNUSED(u)
   return KstObject::NO_CHANGE;
 }
@@ -652,6 +501,17 @@ int KstDataSource::readField(double *v, const QString& field, int s, int n) {
   return -1;
 }
 
+bool KstDataSource::isWritable() const {
+  return _writable;
+}
+
+int KstDataSource::writeField(const double *v, const QString& field, int s, int n) {
+  Q_UNUSED(v)
+  Q_UNUSED(field)
+  Q_UNUSED(s)
+  Q_UNUSED(n)
+  return -1;
+}
 
 int KstDataSource::readMatrix(KstMatrixData* data, const QString& matrix, int xStart, int yStart, int xNumSteps, int yNumSteps, int skip) {
   Q_UNUSED(data)
@@ -748,6 +608,7 @@ void KstDataSource::save(QTextStream &ts, const QString& indent) {
       break;
     }
   }
+  ts << indent << "<tag>" << QStyleSheet::escape(tag().tagString()) << "</tag>" << endl;
   ts << indent << "<filename>" << name << "</filename>" << endl;
   ts << indent << "<type>" << QStyleSheet::escape(fileType()) << "</type>" << endl;
 }
@@ -783,14 +644,14 @@ bool KstDataSource::reset() {
 }
 
 
-const QMap<QString, QString>& KstDataSource::metaData() const {
+const QDict<KstString>& KstDataSource::metaData() const {
   return _metaData;
 }
 
 
 const QString& KstDataSource::metaData(const QString& key) const {
-  if (_metaData.contains(key)) {
-    return _metaData[key];
+  if (_metaData[key]) {
+    return _metaData[key]->value();
   } else {
     return QString::null;
   }
@@ -803,7 +664,7 @@ bool KstDataSource::hasMetaData() const {
 
 
 bool KstDataSource::hasMetaData(const QString& key) const {
-  return _metaData.contains(key);
+  return (_metaData[key] != NULL);
 }
 
 
