@@ -42,8 +42,9 @@
 
 /** Create a KstRVector: raw data from a file */
 KstRVector::KstRVector(KstDataSourcePtr in_file, const QString &in_field,
-                       const QString &in_tag, int in_f0, int in_n,
-                       int skip, bool in_DoSkip, bool in_DoAve)
+                       KstObjectTag in_tag,
+                       int in_f0, int in_n, int skip, bool in_DoSkip,
+                       bool in_DoAve)
 : KstVector(in_tag) {
   commonRVConstructor(in_file, in_field, in_f0, in_n, skip,
       in_DoSkip, in_DoAve);
@@ -52,8 +53,8 @@ KstRVector::KstRVector(KstDataSourcePtr in_file, const QString &in_field,
 
 KstRVector::KstRVector(const QDomElement &e, const QString &o_file,
                        int o_n, int o_f, int o_s, bool o_ave)
-: KstVector() {
-  KstDataSourcePtr in_file;
+: KstVector(e) {
+  KstDataSourcePtr in_file, in_provider;
   QString in_field;
   int in_f0 = 0;
   int in_n = -1;
@@ -66,16 +67,20 @@ KstRVector::KstRVector(const QDomElement &e, const QString &o_file,
   while (!n.isNull()) {
     QDomElement e = n.toElement();
     if (!e.isNull()) {
-      if (e.tagName() == "tag") {
-        setTagName(e.text());
-      } else if (e.tagName() == "filename") {
+      if (e.tagName() == "provider") {
         KST::dataSourceList.lock().readLock();
-        if (o_file == "|") {
-          in_file = *KST::dataSourceList.findFileName(e.text());
-        } else {
-          in_file = *KST::dataSourceList.findFileName(o_file);
-        }
+        in_provider = *KST::dataSourceList.findTag(e.text());
         KST::dataSourceList.lock().unlock();
+      } else if (e.tagName() == "filename") {
+        if (!in_provider) {
+          KST::dataSourceList.lock().readLock();
+          if (o_file == "|") {
+            in_file = *KST::dataSourceList.findFileName(e.text());
+          } else {
+            in_file = *KST::dataSourceList.findFileName(o_file);
+          }
+          KST::dataSourceList.lock().unlock();
+        }
       } else if (e.tagName() == "field") {
         in_field = e.text();
       } else if (e.tagName() == "start") {
@@ -96,6 +101,16 @@ KstRVector::KstRVector(const QDomElement &e, const QString &o_file,
     n = n.nextSibling();
   }
 
+  if (in_provider) {
+    // provider overrides filename
+    in_file = in_provider;
+  }
+  if (in_file) {
+    // use datasource as tag context for this RVector
+    // allow unique vector names to be displayed at top-level
+    setTagName(KstObjectTag(tag().tag(), in_file->tag(), false));
+  }
+
   if (o_n > -2) {
     in_n = o_n;
   }
@@ -159,10 +174,12 @@ void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
 
 
 void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
-                        const QString &in_tag,
+                        KstObjectTag in_tag,
                         int in_f0, int in_n,
                         int in_skip, bool in_DoSkip,
                         bool in_DoAve) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   Skip = in_skip;
   DoSkip = in_DoSkip;
   DoAve = in_DoAve;
@@ -175,7 +192,9 @@ void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
   ReqF0 = in_f0;
   ReqNF = in_n;
   _field = in_field;
-  setTagName(in_tag);
+  if (in_tag != tag()) {
+    setTagName(in_tag);
+  }
 
   if (_file) {
     _file->writeLock();
@@ -193,6 +212,8 @@ void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
 
 
 void KstRVector::changeFile(KstDataSourcePtr in_file) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   if (!in_file) {
     KstDebug::self()->log(i18n("Data file for vector %1 was not opened.").arg(tagName()), KstDebug::Warning);
   }
@@ -200,6 +221,7 @@ void KstRVector::changeFile(KstDataSourcePtr in_file) {
   if (_file) {
     _file->writeLock();
   }
+  setTagName(KstObjectTag(tag().tag(), _file->tag(), false));
   reset();
   if (_file) {
     _file->unlock();
@@ -210,6 +232,8 @@ void KstRVector::changeFile(KstDataSourcePtr in_file) {
 void KstRVector::changeFrames(int in_f0, int in_n,
                               int in_skip, bool in_DoSkip,
                               bool in_DoAve) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   if (_file) {
     _file->writeLock();
   }
@@ -234,6 +258,8 @@ void KstRVector::changeFrames(int in_f0, int in_n,
 
 
 void KstRVector::setFromEnd() {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   ReqF0 = -1;
   if (ReqNF < 2) {
     ReqNF = numFrames();
@@ -304,13 +330,19 @@ int KstRVector::reqStartFrame() const {
 
 /** Save vector information */
 void KstRVector::save(QTextStream &ts, const QString& indent, bool saveAbsolutePosition) {
-  if (_file) {    
+  if (_saveData) {
+    // This is ugly.  Really we need a way to change vector types at runtime.
+    ts << indent << "<avector>" << endl;
+    KstVector::save(ts, indent + "  ", saveAbsolutePosition);
+    ts << indent << "</avector>" << endl;
+  } else if (_file) {    
     ts << indent << "<vector>" << endl;
-
-    ts << indent << "  <tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+    KstVector::save(ts, indent + "  ", saveAbsolutePosition);
     _file->readLock();
+    ts << indent << "  <provider>" << QStyleSheet::escape(_file->tag().tagString()) << "</provider>" << endl;
     ts << indent << "  <filename>" << QStyleSheet::escape(_file->fileName()) << "</filename>" << endl;
     _file->unlock();
+
     ts << indent << "  <field>" << _field << "</field>" << endl;
     if (saveAbsolutePosition) {
       ts << indent << "  <start>" << F0 << "</start>" << endl;
@@ -370,6 +402,8 @@ QString KstRVector::label() const {
 
 
 void KstRVector::reset() { // must be called with a lock
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   _dontUseSkipAccel = false;
   if (_file) {
     SPF = _file->samplesPerFrame(_field);
@@ -408,6 +442,8 @@ void KstRVector::checkIntegrity() {
 
 /** Update an RVECTOR */
 KstObject::UpdateType KstRVector::update(int update_counter) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   bool force = dirty();
   setDirty(false);
   if (KstObject::checkUpdateCounter(update_counter) && !force) {
@@ -426,6 +462,27 @@ KstObject::UpdateType KstRVector::update(int update_counter) {
   return setLastUpdateResult(rc);
 }
 
+// Some things to consider about the following routine...
+// Frames:
+//    Some data sources have data devided into frames.  Each field
+//    has a fixed number of samples per frame.  For some (eg, ascii files)
+//    each frame has 1 sample.  For others (eg, dirfiles) you may have more.
+//    Different fields in the same data source may have different samples per frame.
+//    Within a data source, it is assumed that the first sample of each frame is
+//    simultaneous between fields.
+// Last Frame Read:
+//    Only read the first sample of the last frame read, in cases where there are more
+//    than one sample per frame.   This allows for more sensible association of vectors
+//    into curves, when the X and Y vectors have different numbers of samples per frame.
+//    The rule is that we assume is that the first sample of each frame is simultaneous.
+// Skip reading:  
+//    -'Skip' means read 1 sample each 'Skip' frames (not read one sample,
+//     then skip 'Skip' samples or something else).
+//    -In order that the data are not re-drawn each time a new sample arrives, and to
+//     ensure the re-usability (via shifting) of previously read data, and to make a
+//     region of data look the same regardless of the chouse of f0, all samples
+//     read with skip enabled are read on 'skip boundries'... ie, the first samples of
+//     frame 0, Skip, 2*Skip... N*skip, and never M*Skip+1.
 
 KstObject::UpdateType KstRVector::doUpdate(bool force) {
   int i, k, shift, n_read=0;
@@ -582,7 +639,7 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
       _v[0] = KST::NOPOINT;
       n_read = 1;
     } else if (_file->samplesPerFrame(_field) > 1) {
-      assert(new_nf - NF - 1 > 0 || new_nf - NF - 1 == -1);
+      assert(new_nf - NF - 1 > 0 || new_nf - NF - 1 == -1 || force);
       assert(new_f0 + NF >= 0);
       assert(new_f0 + new_nf - 1 >= 0);
       n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF - 1);
@@ -660,6 +717,8 @@ int KstRVector::fileLength() const {
 
 
 void KstRVector::reload() {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   if (_file) {
     _file->writeLock();
     if (_file->reset()) { // try the efficient way first
@@ -690,7 +749,8 @@ KstDataSourcePtr KstRVector::dataSource() const {
 
 
 KstRVectorPtr KstRVector::makeDuplicate() const {
-  return new KstRVector(_file, _field, tagName() + "'", ReqF0, ReqNF, Skip, DoSkip, DoAve);
+  QString newTag = tag().tag() + "'";
+  return new KstRVector(_file, _field, KstObjectTag(newTag, tag().context()), ReqF0, ReqNF, Skip, DoSkip, DoAve);
 }
 
 
