@@ -15,12 +15,15 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <qstylesheet.h>
+
+#include <klocale.h>
+
 #include "kstscalar.h"
 #include "kstdatacollection.h"
-#include <klocale.h>
+#include "defaultprimitivenames.h"
 #include "ksdebug.h"
 
-#include <qstylesheet.h>
 
 static int iAnonymousScalarCounter = 0;
 
@@ -38,30 +41,24 @@ void KstScalar::clearScalarsDirty() {
 }
 
 /** Create the base scalar */
-KstScalar::KstScalar(const QString& in_tag, KstObject *provider, double val, bool orphan, bool displayable, bool doLock, bool editable)
+KstScalar::KstScalar(KstObjectTag in_tag, KstObject *provider, double val, bool orphan, bool displayable, bool editable)
 : KstPrimitive(provider), _value(val), _orphan(orphan), _displayable(displayable), _editable(editable) {
-  QString _tag = in_tag;
+  QString _tag = in_tag.tag();
   if (_tag.isEmpty()) {
     QString nt = i18n("Anonymous Scalar %1");
 
     do {
       _tag = nt.arg(iAnonymousScalarCounter++);
-    } while (KstData::self()->vectorTagNameNotUniqueInternal(_tag));
+    } while (KstData::self()->vectorTagNameNotUniqueInternal(_tag));  // FIXME: why vector?
+    KstObject::setTagName(KstObjectTag(_tag, in_tag.context()));
   } else {
-    while (KstData::self()->vectorTagNameNotUniqueInternal(_tag)) {
-      _tag += '\'';
-    }
+    KstObject::setTagName(KST::suggestUniqueScalarTag(in_tag));
   }
-  setTagName(_tag);
 
-  // FIXME: passing in a lock variable indicates a design problem
-  if (doLock) {
-    KST::scalarList.lock().writeLock();
-  }
+
+  KST::scalarList.lock().writeLock();
   KST::scalarList.append(this);
-  if (doLock) {
-    KST::scalarList.lock().unlock();
-  }
+  KST::scalarList.lock().unlock();
 }
 
 
@@ -76,7 +73,7 @@ KstScalar::KstScalar(const QDomElement& e)
     QDomElement e = n.toElement();
     if(!e.isNull()) {
       if (e.tagName() == "tag") {
-        setTagName(e.text());
+        setTagName(KstObjectTag::fromString(e.text()));
       } else if (e.tagName() == "orphan") {
         _orphan = true;
       } else if (e.tagName() == "value") {
@@ -101,6 +98,8 @@ KstScalar::~KstScalar() {
 
 
 KstObject::UpdateType KstScalar::update(int updateCounter) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   bool force = dirty();
   setDirty(false);
 
@@ -110,6 +109,7 @@ KstObject::UpdateType KstScalar::update(int updateCounter) {
 
   double v = value();
   if (_provider) {
+    KstWriteLocker pl(_provider);
     _provider->update(updateCounter);
   } else if (force) {
     return setLastUpdateResult(UPDATE);
@@ -120,7 +120,7 @@ KstObject::UpdateType KstScalar::update(int updateCounter) {
 
 
 void KstScalar::save(QTextStream &ts, const QString& indent) {
-  ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  ts << indent << "<tag>" << QStyleSheet::escape(tag().tagString()) << "</tag>" << endl;
   if (_orphan) {
     ts << indent << "<orphan/>" << endl;
   }
@@ -137,15 +137,6 @@ KstScalar& KstScalar::operator=(double v) {
 }
 
 
-bool KstScalar::isGlobal() const {
-  KST::scalarList.lock().readLock();
-  // can't use find() due to constness issues
-  bool rc = KST::scalarList.findTag(tagName()) != KST::scalarList.end();
-  KST::scalarList.lock().unlock();
-  return rc;
-}
-
-
 void KstScalar::setValue(double inV) {
   if (_value != inV) {
     setDirty();
@@ -196,5 +187,15 @@ void KstScalar::setEditable(bool editable) {
 }
 
 
+void KstScalar::setTagName(const KstObjectTag& newTag) {
+  if (newTag == tag()) {
+    return;
+  }
+
+  KstWriteLocker l(&KST::scalarList.lock());
+
+  KST::scalarList.doRename(this, newTag);
+}
+
 #include "kstscalar.moc"
 // vim: et ts=2 sw=2
