@@ -20,11 +20,15 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include <qcstring.h>
 #include <qdeepcopy.h>
 #include <qstylesheet.h>
 
-#include "ksdebug.h"
+#include <kglobal.h>
 #include <klocale.h>
+#include <kmdcodec.h>
+
+#include "ksdebug.h"
 #include "kstdatacollection.h"
 #include "defaultprimitivenames.h"
 #include "kstmath.h"
@@ -40,15 +44,14 @@ static int anonymousVectorCounter = 1;
 #define INITSIZE 1
 
 /** Create a vector */
-KstVector::KstVector(const QString& name, int size, KstObject *provider, bool isScalarList)
+KstVector::KstVector(KstObjectTag in_tag, int size, KstObject *provider, bool isScalarList)
 : KstPrimitive(provider), _nsum(0), _scalars(isScalarList ? 0 : 11) {
   //kstdDebug() << "+++ CREATING VECTOR: " << (void*) this << endl;
-  KstObject::setTagName(name);
   _editable = false;
   NumShifted = 0;
   NumNew = 0;
+  _saveData = false;
   _isScalarList = isScalarList;
-  _label = QString::null;
 
   _saveable = false;
 
@@ -56,16 +59,14 @@ KstVector::KstVector(const QString& name, int size, KstObject *provider, bool is
     size = INITSIZE;
   }
 
-  if (name.isEmpty()) {
+  if (!in_tag.isValid()) {
     QString nt = i18n("Anonymous Vector %1");
 
     do {
-      KstObject::setTagName(nt.arg(anonymousVectorCounter++));
+      KstObject::setTagName(KstObjectTag(nt.arg(anonymousVectorCounter++), in_tag.context()));
     } while (KstData::self()->vectorTagNameNotUnique(tagName(), false));
   } else {
-    while (KstData::self()->vectorTagNameNotUnique(tagName(), false)) {
-      KstObject::setTagName(tagName() + '\'');
-    }
+    KstObject::setTagName(KST::suggestUniqueVectorTag(in_tag));
   }
 
   _v = static_cast<double*>(KST::malloc(size * sizeof(double)));
@@ -79,16 +80,84 @@ KstVector::KstVector(const QString& name, int size, KstObject *provider, bool is
 
   CreateScalars();
   blank();
+
+  KST::vectorList.lock().writeLock();
+  KST::vectorList.append(this);
+  KST::vectorList.lock().unlock();
+}
+
+
+KstVector::KstVector(const QDomElement& e)
+: KstPrimitive(), _nsum(0), _scalars(11) {
+  QByteArray qba;
+  _v = 0L;
+  _size = 0;
+  int sz = INITSIZE;
+  KstObjectTag in_tag = KstObjectTag::invalidTag;
+
+  _editable = false;
+  NumShifted = 0;
+  NumNew = 0;
+  _isScalarList = false;
+  _saveable = false;
+  _saveData = false;
+
+  QDomNode n = e.firstChild();
+  while (!n.isNull()) {
+    QDomElement e = n.toElement();
+    if (!e.isNull()) {
+      if (e.tagName() == "tag") {
+        in_tag = KstObjectTag::fromString(e.text());
+      } else if (e.tagName() == "data") {
+        QCString qcs(e.text().latin1());
+        QByteArray qbca;
+        KCodecs::base64Decode(qcs, qbca);
+        qba = qUncompress(qbca);
+        sz = kMax((size_t)(INITSIZE), qba.size()/sizeof(double));
+      }
+    }
+    n = n.nextSibling();
+  }
+
+  if (!in_tag.isValid()) {
+    QString nt = i18n("Anonymous Vector %1");
+
+    do {
+      KstObject::setTagName(KstObjectTag(nt.arg(anonymousVectorCounter++), in_tag.context()));
+    } while (KstData::self()->vectorTagNameNotUnique(tagName(), false));
+  } else {
+    KstObject::setTagName(KST::suggestUniqueVectorTag(in_tag));
+  }
+
+  CreateScalars();
+  resize(sz, true);
+
+  if (!qba.isEmpty()) {
+    _saveable = true;
+    _saveData = true;
+    QDataStream qds(qba, IO_ReadOnly);
+    for (int i = 0; !qds.atEnd(); ++i) {
+      qds >> _v[i];
+    }
+  }
+
+  _is_rising = false;
+
+  KST::vectorList.lock().writeLock();
+  KST::vectorList.append(this);
+  KST::vectorList.lock().unlock();
 }
 
 
 KstVector::~KstVector() {
   //kstdDebug() << "+++ DELETING VECTOR: " << (void*) this << endl;
   KST::scalarList.lock().writeLock();
+  KST::scalarList.setUpdateDisplayTags(false);
   for (QDictIterator<KstScalar> it(_scalars); it.current(); ++it) {
     KST::scalarList.remove(it.current());
     it.current()->_KShared_unref();
   }
+  KST::scalarList.setUpdateDisplayTags(true);
   KST::scalarList.lock().unlock();
 
   if (_v) {
@@ -243,42 +312,56 @@ double KstVector::value(int i) {
 void KstVector::CreateScalars() {
   if (!_isScalarList) {
     _min = _max = _mean = _minPos = 0.0;
+
+    KstWriteLocker sl(&KST::scalarList.lock());
+    KST::scalarList.setUpdateDisplayTags(false);
+
     KstScalarPtr sp;
-    _scalars.insert("max", sp = new KstScalar(tagName() + "-Max", this));
+    _scalars.insert("max", sp = new KstScalar(KstObjectTag("Max", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("min", sp = new KstScalar(tagName() + "-Min", this));
+    _scalars.insert("min", sp = new KstScalar(KstObjectTag("Min", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("last", sp = new KstScalar(tagName() + "-Last", this));
+    _scalars.insert("last", sp = new KstScalar(KstObjectTag("Last", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("mean", sp = new KstScalar(tagName() + "-Mean", this));
+    _scalars.insert("first", sp = new KstScalar(KstObjectTag("First", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("sigma", sp = new KstScalar(tagName() + "-Sigma", this));
+    _scalars.insert("mean", sp = new KstScalar(KstObjectTag("Mean", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("rms", sp = new KstScalar(tagName() + "-Rms", this));
+    _scalars.insert("sigma", sp = new KstScalar(KstObjectTag("Sigma", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("ns", sp = new KstScalar(tagName() + "-NS", this));
+    _scalars.insert("rms", sp = new KstScalar(KstObjectTag("Rms", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("sum", sp = new KstScalar(tagName() + "-Sum", this));
+    _scalars.insert("ns", sp = new KstScalar(KstObjectTag("NS", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("sumsquared", sp = new KstScalar(tagName() + "-SumSquared", this));
+    _scalars.insert("sum", sp = new KstScalar(KstObjectTag("Sum", tag()), this));
     sp->_KShared_ref();
-    _scalars.insert("minpos", sp = new KstScalar(tagName() + "-MinPos", this));
+    _scalars.insert("sumsquared", sp = new KstScalar(KstObjectTag("SumSquared", tag()), this));
     sp->_KShared_ref();
+    _scalars.insert("minpos", sp = new KstScalar(KstObjectTag("MinPos", tag()), this));
+    sp->_KShared_ref();
+
+    KST::scalarList.setUpdateDisplayTags(true);
   }
 }
 
 void KstVector::RenameScalars() {
   if (!_isScalarList) {
-    _scalars["max"]->setTagName(tagName() + "-Max");
-    _scalars["min"]->setTagName(tagName() + "-Min");
-    _scalars["last"]->setTagName(tagName() + "-Last");
-    _scalars["mean"]->setTagName(tagName() + "-Mean");
-    _scalars["sigma"]->setTagName(tagName() + "-Sigma");
-    _scalars["rms"]->setTagName(tagName() + "-Rms");
-    _scalars["ns"]->setTagName(tagName() + "-NS");
-    _scalars["sum"]->setTagName(tagName() + "-Sum");
-    _scalars["sumsquared"]->setTagName(tagName() + "-SumSquared");
-    _scalars["minpos"]->setTagName(tagName() + "-MinPos");
+    KstWriteLocker sl(&KST::scalarList.lock());
+    KST::scalarList.setUpdateDisplayTags(false);
+
+    _scalars["max"]->setTagName(KstObjectTag("Max", tag()));
+    _scalars["min"]->setTagName(KstObjectTag("Min", tag()));
+    _scalars["last"]->setTagName(KstObjectTag("Last", tag()));
+    _scalars["first"]->setTagName(KstObjectTag("First", tag()));
+    _scalars["mean"]->setTagName(KstObjectTag("Mean", tag()));
+    _scalars["sigma"]->setTagName(KstObjectTag("Sigma", tag()));
+    _scalars["rms"]->setTagName(KstObjectTag("Rms", tag()));
+    _scalars["ns"]->setTagName(KstObjectTag("NS", tag()));
+    _scalars["sum"]->setTagName(KstObjectTag("Sum", tag()));
+    _scalars["sumsquared"]->setTagName(KstObjectTag("SumSquared", tag()));
+    _scalars["minpos"]->setTagName(KstObjectTag("MinPos", tag()));
+
+    KST::scalarList.setUpdateDisplayTags(true);
   }
 }
 
@@ -375,11 +458,11 @@ bool KstVector::resize(int sz, bool reinit) {
 
 KstObject::UpdateType KstVector::internalUpdate(KstObject::UpdateType providerRC) {
   int i, i0;
-  double sum, sum2, last, v;
+  double sum, sum2, last, first, v;
   double last_v;
   double dv2 = 0.0, dv, no_spike_max_dv;
   
-  _max = _min = sum = sum2 = _minPos = last = KST::NOPOINT;
+  _max = _min = sum = sum2 = _minPos = last = first = KST::NOPOINT;
   _nsum = 0;
   
   if (_size > 0) {
@@ -398,6 +481,7 @@ KstObject::UpdateType KstVector::internalUpdate(KstObject::UpdateType providerRC
         _scalars["min"]->setValue(_min);
         _scalars["minpos"]->setValue(_minPos);
         _scalars["last"]->setValue(last);
+        _scalars["first"]->setValue(first);
       }
       _ns_max = _ns_min = 0;
 
@@ -460,6 +544,7 @@ KstObject::UpdateType KstVector::internalUpdate(KstObject::UpdateType providerRC
     _ns_max = _ns_min = last_v = _v[i0];
 
     last = _v[_size-1];
+    first = _v[0];
 
     for (i = i0; i < _size; i++) {
       v = _v[i]; // get rid of redirections
@@ -490,6 +575,7 @@ KstObject::UpdateType KstVector::internalUpdate(KstObject::UpdateType providerRC
       _scalars["min"]->setValue(_min);
       _scalars["minpos"]->setValue(_minPos);
       _scalars["last"]->setValue(last);
+      _scalars["first"]->setValue(first);
     }
 
     updateScalars();
@@ -504,14 +590,29 @@ KstObject::UpdateType KstVector::internalUpdate(KstObject::UpdateType providerRC
 
 void KstVector::save(QTextStream &ts, const QString& indent, bool saveAbsolutePosition) {
   Q_UNUSED(saveAbsolutePosition)
-    Q_UNUSED(ts)
-    Q_UNUSED(indent)
-    abort();
+  ts << indent << "<tag>" << QStyleSheet::escape(tag().tagString()) << "</tag>" << endl;
+  if (_saveData) {
+    QByteArray qba(length()*sizeof(double));
+    QDataStream qds(qba, IO_WriteOnly);
+
+    for (int i = 0; i < length(); i++) {
+      qds << _v[i];
+    }
+
+    ts << indent << "<data>" << KCodecs::base64Encode(qCompress(qba)) << "</data>" << endl;
+  }
 }
 
 
-void KstVector::setTagName(const QString& newTag) {
-  KstObject::setTagName(newTag);
+void KstVector::setTagName(const KstObjectTag& newTag) {
+  if (newTag == tag()) {
+    return;
+  }
+
+  KstWriteLocker l(&KST::vectorList.lock());
+
+  KST::vectorList.doRename(this, newTag);
+
   RenameScalars();
 }
 
@@ -542,7 +643,7 @@ void KstVector::newSync() {
 }
 
 
-KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QString &tag) {
+KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const KstObjectTag& tag) {
   if (n < 2) {
     n = 2;
   }
@@ -558,13 +659,12 @@ KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QStrin
     x1 = x0 + 0.1;
   }
 
-  QString t = tag;
+  QString t = tag.tag();
   if (t.isEmpty()) {
     t = KST::suggestVectorName("X(" + QString::number(x0) + ".." + QString::number(x1) + ")");
   }
 
-  KstVectorPtr xv = new KstVector(t, n);
-  KST::addVectorToList(xv);
+  KstVectorPtr xv = new KstVector(KstObjectTag(t, tag.context()), n);
   xv->_saveable = false;
 
   for (int i = 0; i < n; i++) {
@@ -608,6 +708,16 @@ void KstVector::setEditable(bool editable) {
 }
 
 
+bool KstVector::saveData() const {
+  return _saveData;
+}
+
+
+void KstVector::setSaveData(bool save) {
+  _saveData = save;
+}
+
+
 #undef ZERO_MEMORY
 #undef INITSIZE
 
