@@ -46,7 +46,7 @@ void DataWizard::init()
     _url->setURL(default_source);
     _url->completionObject()->setDir(QDir::currentDirPath());
     _url->setFocus();
-    
+
     // x vector selection
     connect(_xAxisCreateFromField, SIGNAL(toggled(bool)), _xVector, SLOT(setEnabled(bool)));
     connect(_xAxisUseExisting, SIGNAL(toggled(bool)), _xVectorExisting, SLOT(setEnabled(bool)));
@@ -58,9 +58,17 @@ void DataWizard::init()
     _xVectorExisting->_editVector->hide();
 
     _up->setPixmap(BarIcon("up"));
+    _up->setAccel(ALT+Key_Up);
+
     _down->setPixmap(BarIcon("down"));
+    _down->setAccel(ALT+Key_Down);
+
     _add->setPixmap(BarIcon("forward"));
+    _add->setAccel(ALT+Key_S);
+
     _remove->setPixmap(BarIcon("back"));
+    _remove->setAccel(ALT+Key_R);
+
     loadSettings();
 
     QToolTip::add(_up, i18n("Raise in plot order: Alt+Up"));
@@ -229,8 +237,12 @@ void DataWizard::sourceChanged(const QString& text)
 	    _xVector->setCurrentText(defaultX);
 	} else {
 	    _xVector->setCurrentItem(0);
-	}                
+	}
 	_file = file;
+        // we probably don't want to use an
+        //existing vector since the data source just changed...
+        _xAxisCreateFromField->setChecked(true); 
+
     } else {
 	_fileType->setText(QString::null);
 	setNextEnabled(_pageDataSource, false);
@@ -282,11 +294,10 @@ void DataWizard::showPage( QWidget *page )
 	QString save = _filterList->currentText();
 	_filterList->clear();
     } else if (page == _pagePlot) {
-	QString vectorName = _xAxisCreateFromField->isChecked() ? 
-	    _xVector->currentText() : _xVectorExisting->selectedVector();
-	KST::vectorDefaults.setWizardXVector(_xVector->currentText());
-	KST::vectorDefaults.sync();
-
+        if (_xAxisCreateFromField->isChecked()) {
+	    KST::vectorDefaults.setWizardXVector(_xVector->currentText());
+	    KST::vectorDefaults.sync();
+        }
 	// count the vectors we are about to make, so we can guess defaults
 	int n_curves = _vectorsToPlot->childCount();
 
@@ -460,7 +471,7 @@ void DataWizard::newFilter()
 void DataWizard::finished()
 {
     KstApp *app = KstApp::inst();
-    KstVectorList l;
+    KstVectorList vlist;
     QString name = KST::suggestVectorName(_xVector->currentText());
     QValueList<QColor> colors;
     QColor color;
@@ -468,9 +479,8 @@ void DataWizard::finished()
     uint n_steps = 0;
     int ptype = 0;
     int prg = 0;
-  
-    saveSettings();
-    
+    int fontSize;
+
     KstDataSourcePtr ds = *KST::dataSourceList.findReusableFileName(_file);
     if (!ds) {
 	for (KstDataSourceList::Iterator i = _sourceCache.begin(); i != _sourceCache.end(); ++i) {
@@ -538,14 +548,13 @@ void DataWizard::finished()
 	}
     }
 
-
     // memory estimate for the y vectors
     {
 	QListViewItemIterator it(_vectorsToPlot);
 	int fftLen = int(pow(2.0, double(_kstFFTOptions->FFTLen->text().toInt() - 1)));
 	while (it.current()) {
+
 	    QListViewItem *i = it.current();
-	    if (i->isSelected()) {
 		QString field = i->text(0);
 
 		if (_kstDataRange->ReadToEnd->isChecked() || nValue < 0) {
@@ -565,12 +574,10 @@ void DataWizard::finished()
 		if (_radioButtonPlotPSD->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
 		    memoryRequested += fftLen * 6;
 		}
-	    }
 	    ++it;
 	}
     }
 
-
     ds->unlock();
     if (memoryRequested > memoryAvailable) {
 	KMessageBox::sorry(this, i18n("You requested to read in %1 MB of data but it seems that you only have approximately %2 MB of usable memory available.  You cannot load this much data.").arg(memoryRequested/(1024*1024)).arg(memoryAvailable/(1024*1024)));
@@ -600,18 +607,23 @@ void DataWizard::finished()
 
 	// create the x-vector
 	xv = new KstRVector(ds, _xVector->currentText(),
-		name, _kstDataRange->CountFromEnd->isChecked() ? -1 : f0Value,
+		KstObjectTag(name, ds->tag(), false),
+		_kstDataRange->CountFromEnd->isChecked() ? -1 : f0Value,
 		_kstDataRange->ReadToEnd->isChecked() ? -1 : nValue,
 		_kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0, 
 		_kstDataRange->DoSkip->isChecked(),
 		_kstDataRange->DoFilter->isChecked());
-	KST::addVectorToList(xv);
 
 	app->slotUpdateProgress(n_steps, ++prg, i18n("Creating vectors..."));
     } else {
 	xv = *(KST::vectorList.findTag(_xVectorExisting->selectedVector()));
 	app->slotUpdateProgress(n_steps, prg, i18n("Creating vectors..."));
     }
+    // Next time we use the wizard this session, we probably will want to use the
+    // same X vector, so... lets set that as the default.
+    _xAxisUseExisting->setChecked(true);
+    _xVectorExisting->update();
+    _xVectorExisting->setSelection(xv->tag().displayString());
 
     // create the y-vectors
     {
@@ -620,14 +632,14 @@ void DataWizard::finished()
 	    QListViewItem *i = it.current();
 	    name = KST::suggestVectorName(i->text(0));
 
-	    KstVectorPtr v = new KstRVector(ds, i->text(0), name,
+	    KstVectorPtr v = new KstRVector(ds, i->text(0),
+		    KstObjectTag(name, ds->tag(), false),
 		    _kstDataRange->CountFromEnd->isChecked() ? -1 : f0Value,
 		    _kstDataRange->ReadToEnd->isChecked() ? -1 : nValue,
 		    _kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0, 
 		    _kstDataRange->DoSkip->isChecked(),
 		    _kstDataRange->DoFilter->isChecked());
-	    KST::addVectorToList(v);
-	    l.append(v);
+	    vlist.append(v);
 	    ++n_curves;
 	    app->slotUpdateProgress(n_steps, ++prg, i18n("Creating vectors..."));
 	    ++it;
@@ -665,6 +677,8 @@ void DataWizard::finished()
 	return;
     }
 
+    fontSize = qRound(getFontSize(vlist.count(), w));
+
     // create the necessary plots
     app->slotUpdateProgress(n_steps, prg, i18n("Creating plots..."));
     KstViewObjectList plots;
@@ -674,7 +688,7 @@ void DataWizard::finished()
 	plots.append(p.data());
 	if (_radioButtonPlotDataPSD->isChecked()) {
 	    if (_newWindows->isChecked()) {
-		newName += "-PSD";
+		newName += i18n("-Spectra");
 		QString n = app->newWindow(newName);
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
@@ -685,21 +699,21 @@ void DataWizard::finished()
 	}
     } else if (_multiplePlots->isChecked()) {
 	Kst2DPlotPtr p;
-	for (uint i = 0; i < l.count(); ++i) {
+	for (uint i = 0; i < vlist.count(); ++i) {
 	    p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createObject<Kst2DPlot>(KST::suggestPlotName(), false)));
 	    plots.append(p.data());
 	}
 	if (_radioButtonPlotDataPSD->isChecked()) {
 	    if (_newWindows->isChecked()) {
                 w->view()->cleanup(signed(sqrt(plots.count())));
-		newName += "-PSD";
+		newName += i18n("-Spectra");
 		QString n = app->newWindow(newName);
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
-	    for (uint i = 0; i < l.count(); ++i) {
+	    for (uint i = 0; i < vlist.count(); ++i) {
 		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createObject<Kst2DPlot>(KST::suggestPlotName(), false)));
 		plots.append(p.data());
-		p->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay());
+	        p->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay());
 		p->setYAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay());
 	    }
 	}
@@ -722,13 +736,13 @@ void DataWizard::finished()
 	if (_radioButtonPlotDataPSD->isChecked()) {
 	    if (_newWindows->isChecked()) {
                 w->view()->cleanup(signed(sqrt(plots.count())));
-		QString n = app->newWindow(newName + "-PSD");
+		QString n = app->newWindow(newName + i18n("-Spectra"));
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
 	    for (int i = 0; i < _plotNumber->value(); ++i) {
 		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createObject<Kst2DPlot>(KST::suggestPlotName(), false)));
 		plots.append(p.data());
-		p->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay());
+	        p->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay());
 		p->setYAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay());
 	    }
 	}
@@ -736,14 +750,14 @@ void DataWizard::finished()
 
     // Reorder the vectors if the user wants it
     if (_orderInColumns->isChecked()) {
-	const KstVectorList lOld = l;
+	const KstVectorList lOld = vlist;
 	const int count = lOld.count();
 	const int cols = signed(sqrt(plots.count()));
 	const int rows = cols + (count - cols * cols) / cols;
 	int overflow = count % cols;
 	int row = 0, col = 0;
 	for (int i = 0; i < count; ++i) {
-	    l[row * cols + col] = lOld[i];
+	    vlist[row * cols + col] = lOld[i];
 	    ++row;
 	    if (row >= rows) {
 		if (overflow > 0) {
@@ -759,9 +773,9 @@ void DataWizard::finished()
     // create the data curves
     app->slotUpdateProgress(n_steps, prg, i18n("Creating curves..."));
     KstViewObjectList::Iterator pit = plots.begin();
-    for (KstVectorList::Iterator it = l.begin(); it != l.end(); ++it) {
+    for (KstVectorList::Iterator it = vlist.begin(); it != vlist.end(); ++it) {
 	if (_radioButtonPlotData->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
-	    name = KST::suggestCurveName((*it)->tagName(), false);
+	    name = KST::suggestCurveName((*it)->tag(), false);
 	    Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(*pit);
 	    if (plot) {
 		KstVCurveList vcurves = kstObjectSubList<KstBaseCurve,KstVCurve>(plot->Curves);
@@ -818,9 +832,9 @@ void DataWizard::finished()
 	KstVCurvePtr c;
 	int indexColor = 0;
 	ptype = 0;
-	app->slotUpdateProgress(n_steps, prg, i18n("Creating PSDs..."));
+	app->slotUpdateProgress(n_steps, prg, i18n("Creating spectra..."));
 	
-	for (KstVectorList::Iterator it = l.begin(); it != l.end(); ++it) {
+	for (KstVectorList::Iterator it = vlist.begin(); it != vlist.end(); ++it) {
 	    if ((*it)->length() > 0) {
 		Kst2DPlotPtr plot;
 		KstViewObjectList::Iterator startPlot = pit;
@@ -836,7 +850,7 @@ void DataWizard::finished()
 			assert(pit != startPlot);
 		    }
 		}
-	        name = KST::suggestPSDName((*it)->tagName());
+	        name = KST::suggestPSDName((*it)->tag());
 
 		KstPSDPtr p = new KstPSD(name, *it,
 			_kstFFTOptions->SampRate->text().toDouble(),
@@ -852,9 +866,9 @@ void DataWizard::finished()
 		p->setInterpolateHoles(_kstFFTOptions->InterpolateHoles->isChecked());
 		if (_radioButtonPlotPSD->isChecked() || colors.count() <= (unsigned long)indexColor) {
 		    KstVCurveList vcurves = kstObjectSubList<KstBaseCurve,KstVCurve>(plot->Curves);
-		    c = new KstVCurve(KST::suggestCurveName(name, true), p->vX(), p->vY(), 0L, 0L, 0L, 0L, KstColorSequence::next(vcurves, plot->backgroundColor()));
+		    c = new KstVCurve(KST::suggestCurveName(p->tag(), true), p->vX(), p->vY(), 0L, 0L, 0L, 0L, KstColorSequence::next(vcurves, plot->backgroundColor()));
 		} else {
-		    c = new KstVCurve(KST::suggestCurveName(name, true), p->vX(), p->vY(), 0L, 0L, 0L, 0L, colors[indexColor]);
+		    c = new KstVCurve(KST::suggestCurveName(p->tag(), true), p->vX(), p->vY(), 0L, 0L, 0L, 0L, colors[indexColor]);
 		    indexColor++;
 		}
                 c->setLineWidth(KstSettings::globalSettings()->defaultLineWeight);
@@ -887,7 +901,7 @@ void DataWizard::finished()
 		}
 	    }
 	}
-	app->slotUpdateProgress(n_steps, ++prg, i18n("Creating PSDs..."));
+	app->slotUpdateProgress(n_steps, ++prg, i18n("Creating spectra..."));
     }
 
     // legends and labels
@@ -903,6 +917,7 @@ void DataWizard::finished()
 	    continue;
 	}
 	pp->generateDefaultLabels(xl, yl, tl);
+
 	if (_legendsOn->isChecked()) {
             pp->getOrCreateLegend();
 	} else if (_legendsAuto->isChecked()) {
@@ -910,6 +925,9 @@ void DataWizard::finished()
                 pp->getOrCreateLegend();
 	    }
 	}
+
+	pp->setPlotLabelFontSizes(fontSize);
+
 	++pit;
     }
 
@@ -929,6 +947,9 @@ void DataWizard::finished()
     if (!wasPaused) {
 	app->setPaused(false);
     }
+
+    saveSettings();
+
 }
 
 
@@ -1310,4 +1331,38 @@ void DataWizard::vectorsDroppedBack(QDropEvent *e)
     updateVectorPageButtons();
 }
 
+double DataWizard::getFontSize(int count, KstViewWindow *w) {
+  double size;
+  double rows, cols;
+
+  // if there are already plots in the window, use the the first one's font size.
+  Kst2DPlotList plotList = w->view()->findChildrenType<Kst2DPlot>(false);
+  if (!plotList.isEmpty()) {
+    return(plotList[0]->xTickLabel()->fontSize());
+  }
+
+  if (_cycleThrough->isChecked()) {
+    count = _plotNumber->value();
+  } else if (!_multiplePlots->isChecked()) {
+    count = 1;
+  }
+
+  size = (double)KstSettings::globalSettings()->plotFontSize;
+
+  if (_reGrid->isChecked()) {
+    cols = _plotColumns->value();
+  } else {
+    cols = floor(sqrt(double(count)));
+  }
+  rows = cols + int(count - cols * cols) / int(cols);
+
+  size *= (rows + cols)/(cols/rows + rows/cols);
+  size -= (double)KstSettings::globalSettings()->plotFontSize;
+  size *=0.7;
+
+  if (size>22) size = 22;
+
+  return size;
+}
+
 // vim: ts=8 sw=4 noet
