@@ -73,7 +73,6 @@
 #include "kstmatrixdialog_i.h"
 #include "kstmatrixdefaults.h"
 #include "kstmonochromedialog_i.h"
-#include "kstplotdialog_i.h"
 #include "kstplugindialog_i.h"
 #include "kstprintoptionspage.h"
 #include "kstpsddialog_i.h"
@@ -84,6 +83,7 @@
 #include "kstvectordialog_i.h"
 #include "kstviewmanager_i.h"
 #include "kstviewscalarsdialog_i.h"
+#include "kstviewstringsdialog_i.h"
 #include "kstviewvectorsdialog_i.h"
 #include "kstviewmatricesdialog_i.h"
 #include "kstviewfitsdialog_i.h"
@@ -96,7 +96,7 @@
 #include "sysinfo.h"
 #include "updatethread.h"
 #include "vectorsavedialog.h"
-
+#include "kstobjectdefaults.h"
 
 #define KST_STATUSBAR_DATA   1
 #define KST_STATUSBAR_STATUS 2
@@ -144,10 +144,10 @@ KstApp::KstApp(QWidget *parent, const char *name)
 
   /* create dialogs */
   debugDialog = new KstDebugDialogI(this);
-  _plotDialog = new KstPlotDialogI(doc, this);
   dataManager = new KstDataManagerI(doc, this);
   viewManager = new KstViewManagerI(doc, this);
   viewScalarsDialog = new KstViewScalarsDialogI(this);
+  viewStringsDialog = new KstViewStringsDialogI(this);
   viewVectorsDialog = new KstViewVectorsDialogI(this);
   viewMatricesDialog = new KstViewMatricesDialogI(this);
   viewFitsDialog = new KstViewFitsDialogI(this);
@@ -162,20 +162,6 @@ KstApp::KstApp(QWidget *parent, const char *name)
   _quickStartDialog = new KstQuickStartDialogI(this, 0 , true);
 #endif
 
-  connect(KstVectorDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstCurveDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstCsdDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstEqDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstHsDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstPsdDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstPluginDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstFitDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstFilterDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstEventMonitorI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstImageDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(KstMatrixDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-  connect(this, SIGNAL(mdiModeHasBeenChangedTo(KMdi::MdiMode)), SLOT(fixKMdi()));
-
   initActions();
   readOptions();
   XYZoomAction->setChecked(true);
@@ -195,9 +181,6 @@ KstApp::KstApp(QWidget *parent, const char *name)
   connect(graphFileDialog, SIGNAL(graphFileReq(const QString&,const QString&,int,int,bool,int)), this, SLOT(immediatePrintToPng(const QString&,const QString&,int,int,bool,int)));
   connect(graphFileDialog, SIGNAL(graphFileEpsReq(const QString&,int,int,bool,int)), this, SLOT(immediatePrintToEps(const QString&,int,int,bool,int)));
 
-  /*** plot dialog ***/
-  connect(_plotDialog, SIGNAL(docChanged()), this, SLOT(registerDocChange()));
-
   // data manager signals
   connect(dataManager, SIGNAL(docChanged()), this, SLOT(registerDocChange()));
   connect(doc, SIGNAL(updateDialogs()), this, SLOT(updateDialogs()));
@@ -214,31 +197,12 @@ KstApp::KstApp(QWidget *parent, const char *name)
   connect(this, SIGNAL(settingsChanged()), this, SLOT(slotSettingsChanged()));
 
   QTimer::singleShot(0, this, SLOT(updateActions()));
-
-  if (!isFakingSDIApplication()) {
-    bool addedWindowMenu = false;
-    int menuCount = menuBar()->count();
-    for (int menuIndex = 0; menuIndex < menuCount; menuIndex++) {
-      int menuId = menuBar()->idAt(menuIndex);
-      if (menuId != -1) {
-        const QString& menuTitle = menuBar()->text(menuId);
-        if (menuTitle == i18n("&Help")) {
-          menuBar()->insertItem(i18n("&Window"), windowMenu(), 100, menuIndex);
-          addedWindowMenu = true;
-          break;
-        }
-      }
-    }
-
-    if (!addedWindowMenu) {
-      menuBar()->insertItem(i18n("&Window"), windowMenu());
-    }
-    connect(windowMenu(), SIGNAL(aboutToShow()), this, SLOT(addNewWindowMenu()));
-  }
 }
 
 
 KstApp::~KstApp() {
+  destroyDebugNotifier();
+
   delete _plotHolderWhileOpeningDocument;
   _plotHolderWhileOpeningDocument = 0L;
 
@@ -250,6 +214,7 @@ KstApp::~KstApp() {
   }
 
   KstDataSource::cleanupForExit(); // must be before deletions
+  KstDataObject::cleanupForExit(); // must be before deletions
   delete _updateThread;
   _updateThread = 0L;
   delete _dcopIface;
@@ -264,6 +229,18 @@ KstApp::~KstApp() {
 }
 
 
+QSize KstApp::sizeHint() const
+{
+  QSize size;
+  QRect rect( KGlobalSettings::desktopGeometry(KstApp::inst()) );
+
+  size.setWidth(3*rect.width()/4);
+  size.setHeight(3*rect.height()/4);
+
+  return size;
+}
+
+
 void KstApp::initialize() {
   KstSettings::checkUpdates();
   kConfigObject = new KConfig("kstdatarc", false, false);
@@ -322,18 +299,6 @@ void KstApp::loadExtensions() {
 }
 
 
-void KstApp::addNewWindowMenu() {
-  int id = windowMenu()->insertItem(i18n("&New..."), this, SLOT(slotFileNewWindow()), 0, -1, 0);
-  windowMenu()->setWhatsThis(id, i18n("Create a new plot window."));
-
-  id = windowMenu()->insertItem(i18n("&Rename..."), this, SLOT(slotFileRenameWindow()), 0, -1, 0);
-  windowMenu()->setWhatsThis(id, i18n("Rename an existing plot window."));
-  if (!activeView()) {
-    windowMenu()->setItemEnabled(id, false);
-  }
-}
-
-
 const QString& KstApp::defaultFont() const {
   return _defaultFont;
 }
@@ -397,6 +362,9 @@ void KstApp::updateActions() {
 
 
 void KstApp::initActions() {
+  KAction *newTab = new KAction(i18n("&New tab..."), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(), "file_new_window");
+  newTab->setWhatsThis(i18n("Create a new tab."));
+
   fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
   fileSave->setWhatsThis(i18n("Save to current Kst plot file."));
 
@@ -451,6 +419,12 @@ void KstApp::initActions() {
   PauseAction->setWhatsThis(i18n("When paused, new data will not be read."));
   connect(PauseAction, SIGNAL(toggled(bool)), this, SLOT(updatePausedState(bool)));
 
+  /************/
+  _saveData = new KToggleAction(i18n("Save Da&ta"), 0,0,0,0,
+				actionCollection(), "save_vector_data");
+  _saveData->setToolTip(i18n("Save Vector Data To Disk"));
+  _saveData->setWhatsThis(i18n("When selected, data in vectors will be saved into the Kst file."));
+
   /************/
   XYZoomAction = new KRadioAction(i18n("XY Mouse &Zoom"), "kst_zoomxy",
                                   KShortcut(Key_F2),
@@ -500,13 +474,6 @@ void KstApp::initActions() {
   NewPlotAction->setWhatsThis(i18n("Create a new plot in the\n"
                                       "current window."));
 
-  /************/
-  PlotDialogAction = new KAction(i18n("Edit &Plot..."), "kst_editplots", 0,
-                                 this, SLOT(showPlotDialog()),
-                                 actionCollection(), "plotdialog_action");
-  PlotDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
-                                      "to edit plot settings."));
-
   /************/
   DataManagerAction = new KAction(i18n("&Data Manager"), "kst_datamanager", 0,
                                   dataManager, SLOT(show_I()),
@@ -538,12 +505,12 @@ void KstApp::initActions() {
                                        "to create a new curve."));
 
   /************/
-  CsdDialogAction = new KAction(i18n("New Cumulative &Spectral Decay..."), "kst_csdnew", 0,
+  CsdDialogAction = new KAction(i18n("New &Spectrogram..."), "kst_csdnew", 0,
                                 KstCsdDialogI::globalInstance(),
                                 SLOT(show()), actionCollection(),
                                 "csddialog_action");
   CsdDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
-                                     "to create a new cumulative spectral decay."));
+                                     "to create a new spectrogram."));
 
 
   /************/
@@ -565,19 +532,18 @@ void KstApp::initActions() {
                                     "to create a new histogram."));
 
   /************/
-  PsdDialogAction = new KAction(i18n("New Power &Spectrum..."),
+  PsdDialogAction = new KAction(i18n("New &Spectrum..."),
                                 "kst_psdnew", 0,
                                 KstPsdDialogI::globalInstance(),
                                 SLOT(show()), actionCollection(),
                                 "psddialog_action");
   PsdDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
-                                     "to create a new power spectrum."));
+                                     "to create a new spectrum."));
 
   /************/
   PluginDialogAction = new KAction(i18n("New &Plugin..."),
                                   "kst_pluginnew", 0,
-                                   KstPluginDialogI::globalInstance(),
-                                   SLOT(show()), actionCollection(),
+                                   this, SLOT(selectDataPlugin()), actionCollection(),
                                    "plugindialog_action");
   PluginDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
                                         "to create a new plugin instance."));
@@ -635,6 +601,16 @@ void KstApp::initActions() {
                                        "viewscalarsdialog_action");
   ViewScalarsDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
                                             "to view scalar values."));
+  ViewScalarsDialogAction->setEnabled(false);
+
+  /************/
+  ViewStringsDialogAction = new KAction(i18n("View Strin&g Values"),
+                                       0, 0, this,
+                                       SLOT(showViewStringsDialog()),
+                                       actionCollection(),
+                                       "viewstringsdialog_action");
+  ViewStringsDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
+                                            "to view string values."));
 
   /************/
   ViewVectorsDialogAction = new KAction(i18n("View Vec&tor Values"),
@@ -644,6 +620,7 @@ void KstApp::initActions() {
                                        "viewvectorsdialog_action");
   ViewVectorsDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
                                             "to view vector values."));
+  ViewVectorsDialogAction->setEnabled(false);
 
   /************/
   ViewMatricesDialogAction = new KAction(i18n("View &Matrix Values"),
@@ -653,6 +630,7 @@ void KstApp::initActions() {
                                        "viewmatricesdialog_action");
   ViewMatricesDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
                                             "to view matrix values."));
+  ViewMatricesDialogAction->setEnabled(false);
 
   /************/
   ViewFitsDialogAction = new KAction(i18n("View &Fit Results"),
@@ -662,6 +640,7 @@ void KstApp::initActions() {
                                        "viewfitsdialog_action");
   ViewFitsDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
                                             "to view fit values."));
+  ViewFitsDialogAction->setEnabled(false);
 
   /************/
   ChangeNptsDialogAction = new KAction(i18n("Change Data Sample &Ranges..."),
@@ -736,12 +715,13 @@ void KstApp::initActions() {
   SamplesFromEndAction->setWhatsThis(i18n("Read current data from end of file."));
 
   /************/
+#if 0
   PluginManagerAction = new KAction(i18n("&Plugins..."), 0, 0,
                                  this, SLOT(showPluginManager()),
                                  actionCollection(), "pluginmanager_action");
   PluginManagerAction->setWhatsThis(i18n("Bring up a dialog box\n"
                                            "to manage plugins."));
-
+#endif
 
   /************/
   ExtensionManagerAction = new KAction(i18n("&Extensions..."), 0, 0,
@@ -753,7 +733,7 @@ void KstApp::initActions() {
 
   /************/
   DataWizardAction = new KAction(i18n("Data &Wizard"), "wizard", 
-                                 KShortcut(Key_W),
+                                 KShortcut(CTRL+ALT+Key_W),
                                  this, SLOT(showDataWizard()),
                                  actionCollection(), "datawizard_action");
   DataWizardAction->setWhatsThis(i18n("Bring up a wizard\n"
@@ -786,13 +766,13 @@ void KstApp::initActions() {
   _tiedZoomAction->setWhatsThis(i18n("Apply zoom actions to all plots\n"
                                      "(not just the active one)."));
 
-  _gfxRectangleAction = new KRadioAction(i18n("&Rectangle"), "kst_gfx_rectangle", 
+  _gfxRectangleAction = new KRadioAction(i18n("&Box"), "kst_gfx_rectangle", 
                                   KShortcut(Key_F8),
                                   this, SLOT(toggleMouseMode()),
                                   actionCollection(), "rectangle_action");
   _gfxRectangleAction->setExclusiveGroup("gfx");
-  _gfxRectangleAction->setToolTip(i18n("Draw rectangle"));
-  _gfxRectangleAction->setWhatsThis(i18n("Draw rectangle"));
+  _gfxRectangleAction->setToolTip(i18n("Draw box"));
+  _gfxRectangleAction->setWhatsThis(i18n("Draw box"));
 
   _gfxEllipseAction = new KRadioAction(i18n("&Ellipse"), "kst_gfx_ellipse",
                                   KShortcut(Key_F9),
@@ -809,6 +789,14 @@ void KstApp::initActions() {
   _gfxPictureAction->setExclusiveGroup("gfx");
   _gfxPictureAction->setToolTip(i18n("Insert picture"));
   _gfxPictureAction->setWhatsThis(i18n("Insert picture"));
+  
+  _gfx2DPlotAction = new KRadioAction(i18n("&Plot"), "kst_newplot",
+                                   KShortcut(CTRL+Key_2),
+                                   this, SLOT(toggleMouseMode()),
+                                   actionCollection(), "2dplot_action");
+  _gfx2DPlotAction->setExclusiveGroup("gfx");
+  _gfx2DPlotAction->setToolTip(i18n("Insert Plot"));
+  _gfx2DPlotAction->setWhatsThis(i18n("Insert Plot"));
 
   _gfxArrowAction = new KRadioAction(i18n("&Arrow"), "kst_gfx_arrow",
                                    KShortcut(Key_F11),
@@ -859,6 +847,7 @@ void KstApp::initActions() {
   _gfxLineAction->plug(mouseModeMenu);
   _gfxArrowAction->plug(mouseModeMenu);
   _gfxPictureAction->plug(mouseModeMenu);
+  _gfx2DPlotAction->plug(mouseModeMenu);
 
   toolBar()->insertButton("thumbnail", 5, mouseModeMenu, true, i18n("Select the desired mode"));
   toggleMouseMode();
@@ -972,6 +961,51 @@ void KstApp::delayedDocInit() {
 }
 
 
+void KstApp::selectDataPlugin() {
+  QStringList l;
+
+  //The new KstDataObject plugins...
+  QStringList newPlugins;
+  const KstPluginInfoList pluginInfo = KstDataObject::pluginInfoList();
+  {
+    KstPluginInfoList::ConstIterator it = pluginInfo.begin();
+    for (; it != pluginInfo.end(); ++it) {
+      newPlugins << it.key();
+    }
+  }
+
+  l += newPlugins;
+
+  //The old C style plugins...
+  QStringList oldPlugins;
+  const QMap<QString,QString> readable = PluginCollection::self()->readableNameList();
+  {
+    QMap<QString,QString>::const_iterator it = readable.begin();
+    for (; it != readable.end(); ++it) {
+      oldPlugins << it.key();
+    }
+  }
+
+  l += oldPlugins;
+
+  bool ok = false;
+  QStringList plugin = KInputDialog::getItemList(i18n("Data Plugins"), i18n("Create..."), l, 0, false, &ok, this);
+
+  if (!ok || plugin.isEmpty()) {
+    return;
+  }
+
+  const QString p = plugin.join("");
+
+  if (newPlugins.contains(p)) {
+    KstDataObjectPtr ptr = KstDataObject::plugin(p);
+    ptr->showDialog(true);
+  } else if (oldPlugins.contains(p)) {
+    KstPluginDialogI::globalInstance()->showNew(readable[p]);
+  }
+}
+
+
 void KstApp::forceUpdate() {
   _updateThread->forceUpdate();
 }
@@ -1052,12 +1086,6 @@ bool KstApp::openDocumentFile(const QString& in_filename,
   return rc;
 }
 
-
-KstPlotDialogI *KstApp::plotDialog() const {
-  return _plotDialog;
-}
-
-
 KstMonochromeDialogI *KstApp::monochromeDialog() const {
   return _monochromeDialog;
 }
@@ -1072,11 +1100,9 @@ void KstApp::saveOptions() {
   config->setGroup("General Options");
   _recent->saveEntries(config, "Recent Files");
 
-  config->writeEntry("MDIMode", (int)mdiMode());
-
   KST::vectorDefaults.writeConfig(config);
   KST::matrixDefaults.writeConfig(config);
-
+  KST::objectDefaults.writeConfig(config);
   config->sync();
 }
 
@@ -1085,25 +1111,11 @@ void KstApp::readOptions() {
   config->setGroup("General Options");
   _recent->loadEntries(config, "Recent Files");
 
-  int mdiMode = config->readNumEntry("MDIMode", (int)KMdi::TabPageMode);
-
   KST::vectorDefaults.readConfig(config);
   KST::matrixDefaults.readConfig(config);
+  KST::objectDefaults.readConfig(config);
 
-  switch (mdiMode) {
-    case KMdi::ToplevelMode:
-      switchToToplevelMode();
-      break;
-    case KMdi::ChildframeMode:
-      switchToChildframeMode();
-      break;
-    case KMdi::TabPageMode:
-      switchToTabPageMode();
-      break;
-    default:
-      switchToTabPageMode();
-      break;
-  }
+  switchToTabPageMode();
 }
 
 
@@ -1360,8 +1372,6 @@ void KstApp::immediatePrintActiveWindowToEps(const QString& filename, int width,
 }
 
 
-// FIXME: this redefines soup.  It doesn't handle error conditions correctly
-//        and it's hard to understand (10 levels of nested braces!!).  Refator
 void KstApp::slotFilePrint() {
   KstViewWindow *currentWin = dynamic_cast<KstViewWindow*>(activeWindow());
   int currentPage = 0;
@@ -1532,63 +1542,9 @@ void KstApp::slotFilePrint() {
         if (!firstPage && !printer.newPage()) {
           break;
         }
-        if (lineAdjust != 0) {
-          tlv->forEachChild(&Kst2DPlot::pushAdjustLineWidth, lineAdjust, false);
-        }
-        if (monochrome) {
-          tlv->forEachChild2(&Kst2DPlot::pushPlotColors);
-          tlv->forEachChild<const QColor&>(&Kst2DPlot::pushCurveColor, Qt::black, false);
-          // additional pushes for enhanced readability
-          if (enhanceReadability) {
-            Kst2DPlotList plotList = tlv->findChildrenType<Kst2DPlot>(false);
-            for (Kst2DPlotList::Iterator it = plotList.begin(); it != plotList.end(); ++it ) {
-              (*it)->changeToMonochrome(pointStyleOrder, lineStyleOrder, lineWidthOrder, maxLineWidth, pointDensity);
-            }
-          }
-        }
 
-        if (datetimeFooter) {
-          QDateTime dateTime = QDateTime::currentDateTime();
-          QString title = i18n("Page: %1  Name: %2  Date: %3").arg(pages).arg(win->caption()).arg(dateTime.toString(Qt::ISODate));
-          QRect rect(0, 0, size.width(), size.height());
-          QRect rectBounding;
+        win->print(paint, size, pages, lineAdjust, monochrome, enhanceReadability, datetimeFooter,  maintainAspectRatio, pointStyleOrder, lineStyleOrder, lineWidthOrder, maxLineWidth, pointDensity);
 
-          rectBounding = paint.boundingRect(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
-          rect.setTop(size.height() - (2 * rectBounding.height()));
-          paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
-          size.setHeight(rect.top());
-        }
-
-        if (maintainAspectRatio) {
-          const QRect geom(win->view()->geometry());
-          const double ratioWindow = double(geom.width()) / double(geom.height());
-          const double ratioPrinter = double(size.width()) / double(size.height());
-          if (ratioWindow > ratioPrinter) {
-            size.setHeight(int(double(size.width()) / ratioWindow));
-          } else if (ratioWindow < ratioPrinter) {
-            size.setWidth(int(ratioWindow * double(size.height())));
-          }
-        }
-
-        win->view()->resizeForPrint(size);
-        win->view()->paint(paint, QRegion());
-
-        if (lineAdjust != 0) {
-          tlv->forEachChild2(&Kst2DPlot::popLineWidth);
-        }
-        if (monochrome) {
-          tlv->forEachChild2(&Kst2DPlot::popPlotColors);
-          tlv->forEachChild2(&Kst2DPlot::popCurveColor);
-          // additional pops to undo enhanced readability
-          if (enhanceReadability) {
-            Kst2DPlotList plotList = tlv->findChildrenType<Kst2DPlot>(false);
-            for (Kst2DPlotList::Iterator it = plotList.begin(); it != plotList.end(); ++it ) {
-              (*it)->undoChangeToMonochrome(pointStyleOrder, lineStyleOrder, lineWidthOrder);
-            }
-          }
-        }
-        win->view()->revertForPrint();
-        
         firstPage = false;
       }
     }
@@ -1878,6 +1834,10 @@ void KstApp::toggleMouseMode() {
     action = _gfxPictureAction;
     mode = KstTopLevelView::CreateMode;
     createType = "Picture";
+  } else if (_gfx2DPlotAction->isChecked()) {
+    action = _gfx2DPlotAction;
+    mode = KstTopLevelView::CreateMode;
+    createType = "Plot";
   } else if (_gfxArrowAction->isChecked()) {
     action = _gfxArrowAction;
     mode = KstTopLevelView::CreateMode;
@@ -1964,23 +1924,9 @@ void KstApp::paintAll(KstPainter::PaintType pt) {
   QTime t;
   t.start();
 #endif
-  if (mdiMode() == KMdi::TabPageMode || mdiMode() == KMdi::IDEAlMode) { // Optimization
-    KstViewWindow *view = dynamic_cast<KstViewWindow*>(activeWindow());
-    if (view) {
-      view->view()->paint(pt);
-    }
-  } else {
-    KMdiIterator<KMdiChildView*> *it = createIterator();
-    if (it) {
-      while (it->currentItem()) {
-        KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
-        if (view) {
-          view->view()->paint(pt);
-        }
-        it->next();
-      }
-      deleteIterator(it);
-    }
+  KstViewWindow *view = dynamic_cast<KstViewWindow*>(activeWindow());
+  if (view) {
+    view->view()->paint(pt);
   }
 #ifdef PAINTTIMER
   int x = t.elapsed();
@@ -1988,25 +1934,15 @@ void KstApp::paintAll(KstPainter::PaintType pt) {
 #endif
 }
 
-void KstApp::showPlotDialog() {
-  KMdiChildView *win = activeWindow();
-
-  if (win && _plotDialog->isHidden()) {
-    _plotDialog->show_I(win->caption(), QString::null);
-  } else {
-    _plotDialog->show_I();
-  }
-}
 
 void KstApp::newPlot() {
   KstViewWindow *w = dynamic_cast<KstViewWindow*>(activeWindow());
-  if (w) {
-    w->createObject<Kst2DPlot>(KST::suggestPlotName(), false);
+  if (!w) {
+    newWindow(false);
+    w = dynamic_cast<KstViewWindow*>(activeWindow());
+    assert(w);
   }
-}
-
-void KstApp::showPlotDialog(const QString& window, const QString& plot) {
-  _plotDialog->show_I(window, plot);
+  w->createPlot(KST::suggestPlotName(), false);
 }
 
 
@@ -2025,6 +1961,11 @@ void KstApp::showViewScalarsDialog() {
 }
 
 
+void KstApp::showViewStringsDialog() {
+  viewStringsDialog->showViewStringsDialog();
+}
+
+
 void KstApp::showViewVectorsDialog() {
   viewVectorsDialog->showViewVectorsDialog();
 }
@@ -2104,6 +2045,7 @@ void KstApp::updateDataNotifier() {
 void KstApp::updateDataDialogs(bool dm, bool vm) {
 
   ViewScalarsDialogAction->setEnabled(viewScalarsDialog->hasContent());
+  ViewStringsDialogAction->setEnabled(viewStringsDialog->hasContent());
   ViewVectorsDialogAction->setEnabled(viewVectorsDialog->hasContent());
   ViewMatricesDialogAction->setEnabled(viewMatricesDialog->hasContent());
   ViewFitsDialogAction->setEnabled(viewFitsDialog->hasContent());
@@ -2111,6 +2053,9 @@ void KstApp::updateDataDialogs(bool dm, bool vm) {
   if (!viewScalarsDialog->isHidden()) {
     viewScalarsDialog->updateViewScalarsDialog();
   }
+  if (!viewStringsDialog->isHidden()) {
+    viewStringsDialog->updateViewStringsDialog();
+  }
   if (!viewVectorsDialog->isHidden()) {
     viewVectorsDialog->updateViewVectorsDialog();
   }
@@ -2179,9 +2124,6 @@ void KstApp::updateDialogs(bool onlyVisible) {
     if (!onlyVisible || KstMatrixDialogI::globalInstance()->isShown()) {
       KstMatrixDialogI::globalInstance()->update();
     }
-    if (!onlyVisible || _plotDialog->isShown()) {
-      _plotDialog->update();
-    }
     if (!onlyVisible || changeFileDialog->isShown()) {
       changeFileDialog->updateChangeFileDialog();
     }
@@ -2194,12 +2136,10 @@ void KstApp::updateDialogs(bool onlyVisible) {
     if (!onlyVisible || changeNptsDialog->isShown()) {
       changeNptsDialog->updateChangeNptsDialog();
     }
-    if (!onlyVisible) { // FIXME: might want to make this sensible one day
-      updateDataDialogs(false);
-    }
     if (!onlyVisible || vectorSaveDialog->isShown()) {
       vectorSaveDialog->init();
     }
+    updateDataDialogs(false);
     updateDataManager(onlyVisible);
     updateViewManager(onlyVisible);
 #ifdef BENCHMARK
@@ -2219,7 +2159,6 @@ void KstApp::updateDialogsForWindow() {
     KstImageDialogI::globalInstance()->updateWindow();
     updateDataManager(false);
     updateViewManager(false);
-    _plotDialog->updateWindow();
   }
 }
 
@@ -2437,15 +2376,15 @@ QString KstApp::windowName(bool prompt, const QString& nameOriginal, bool rename
 
       if (rename) {
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
-        name = KInputDialog::getText(i18n("Kst"), i18n("Enter a new name for the window:"), name, &ok, parent, 0L, &val).stripWhiteSpace();
+        name = KInputDialog::getText(i18n("Kst"), i18n("Enter a new name for the tab:"), name, &ok, parent, 0L, &val).stripWhiteSpace();
 #else
-        name = KLineEditDlg::getText(i18n("Enter a new name for the window:"), name, &ok, parent, &val).stripWhiteSpace();
+        name = KLineEditDlg::getText(i18n("Enter a new name for the tab:"), name, &ok, parent, &val).stripWhiteSpace();
 #endif
       } else {
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
-        name = KInputDialog::getText(i18n("Kst"), i18n("Enter a name for the new window:"), name, &ok, parent, 0L, &val).stripWhiteSpace();
+        name = KInputDialog::getText(i18n("Kst"), i18n("Enter a name for the new tab:"), name, &ok, parent, 0L, &val).stripWhiteSpace();
 #else
-        name = KLineEditDlg::getText(i18n("Enter a name for the new window:"), name, &ok, parent, &val).stripWhiteSpace();
+        name = KLineEditDlg::getText(i18n("Enter a name for the new tab:"), name, &ok, parent, &val).stripWhiteSpace();
 #endif
       }
       if (ok && name==defaultTag) {
