@@ -95,6 +95,13 @@ public:
   Kst2DPlot(const Kst2DPlot& plot, const QString& name);
   virtual ~Kst2DPlot();
 
+  QWidget *configWidget();
+
+  // handle custom widget, if any: is called by KstEditViewObjectDialogI
+  bool fillConfigWidget(QWidget *w, bool isNew) const;
+  bool readConfigWidget(QWidget *w);
+  void connectConfigWidget(QWidget *parent, QWidget *w) const;
+    
   static Kst2DPlotList globalPlotList();
   static Kst2DPlotPtr findPlotByName(const QString& name);
   static bool checkRange(double& min_in, double& max_in);
@@ -117,6 +124,7 @@ public:
   void clearCurves();
   void setXScaleMode(KstScaleModeType scalemode);
   void setYScaleMode(KstScaleModeType scalemode);
+  void nextImageColorScale();
 
   void pushAdjustLineWidth(int adjustment);
   void popLineWidth();
@@ -166,8 +174,8 @@ public:
   void getScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
 
   // get expressions for the scale
-  void getXScaleExps(QString& minExp, QString& maxExp);
-  void getYScaleExps(QString& minExp, QString& maxExp);
+  void getXScaleExps(QString& minExp, QString& maxExp) const;
+  void getYScaleExps(QString& minExp, QString& maxExp) const;
 
   void getLScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
 
@@ -180,7 +188,8 @@ public:
   KstPlotLabel *xTickLabel() const;
   KstPlotLabel *yTickLabel() const;
   KstPlotLabel *fullTickLabel() const;
-
+  void setPlotLabelFontSizes(int size);
+  
 
   QRect GetPlotRegion() const;
   QRect GetWinRegion() const;
@@ -251,8 +260,8 @@ public:
   bool setPlotMarker(const double xValue, bool isRisng = false, bool isFalling = false, bool isVectorValue = false);
   bool removePlotMarker(const double xValue);
   void setPlotMarkerList(const KstMarkerList& newMarkers);
-  const KstMarkerList plotMarkers(const double minX, const double maxX);
-  const KstMarkerList& plotMarkers();
+  const KstMarkerList plotMarkers(const double minX, const double maxX) const;
+  const KstMarkerList& plotMarkers() const;
   bool nextMarker(const double currentPosition, double& marker);
   bool prevMarker(const double currentPosition, double& marker);
   void moveToNextMarker(KstViewWidget*);
@@ -311,13 +320,13 @@ public:
   int axisPenWidth() const { return _axisPenWidth; }
 
   void setColorMarkers(QColor color) { _colorMarkers = color; }
-  QColor colorMarkers() { return _colorMarkers; }
+  QColor colorMarkers() const { return _colorMarkers; }
   void setLineWidthMarkers(int lineWidth) { _lineWidthMarkers = lineWidth; }
-  int lineWidthMarkers() { return _lineWidthMarkers; }
+  int lineWidthMarkers() const { return _lineWidthMarkers; }
   void setLineStyleMarkers(int lineStyle) { _lineStyleMarkers = lineStyle; }
-  int lineStyleMarkers() { return _lineStyleMarkers; }
+  int lineStyleMarkers() const { return _lineStyleMarkers; }
   void setDefaultColorMarker(bool defaultColor) { _defaultMarkerColor = defaultColor; }
-  bool defaultColorMarker( ) { return _defaultMarkerColor; }
+  bool defaultColorMarker() const { return _defaultMarkerColor; }
   
   // set and get tick mark display options
   void setXTicksInPlot(bool yes);
@@ -366,7 +375,7 @@ public:
   bool undoChangeToMonochrome(int pointStylePriority, int lineStylePriority, int lineWidthPriority);
 
   //convenience routines for working with viewLegends
-  KstViewLegendPtr legend();
+  KstViewLegendPtr legend() const;
 
   KstViewLegendPtr getOrCreateLegend();
   
@@ -401,9 +410,6 @@ public slots:
   void filterCurve(int id);
   void removeCurve(int id);
 
-  // used in layout mode
-  bool showDialog(KstTopLevelViewPtr invoker, bool isNew = false);
-
 protected slots:
   void menuMoveUp();
   void menuMoveDown();
@@ -426,6 +432,7 @@ protected slots:
   void menuZoomSpikeInsensitiveMax();
   void menuNextMarker();
   void menuPrevMarker();
+  void menuNextImageColorScale();
 
   void timezoneChanged(const QString& tz, int utcOffset);
 
@@ -558,7 +565,7 @@ private:
   QRect PlotAndAxisRegion;
 
   void updateScale();
-  void adjustFontSize();
+  //void adjustFontSize();
 
   KstMouse _mouse;
   QMap<int, QString> _curveEditMap, _curveFitMap, _curveRemoveMap, _objectEditMap;
@@ -594,6 +601,8 @@ private:
 
   int _xMajorTicks;
   int _yMajorTicks;
+  
+  int _i_per; // index for next image color range
 
   KstPlotLabel *_xLabel, *_yLabel, *_topLabel, *_xTickLabel, *_yTickLabel, *_fullTickLabel;
 
@@ -635,6 +644,8 @@ private:
 
   KstScaleModeType _xScaleModeDefault;
   KstScaleModeType _yScaleModeDefault;
+  
+  int _tabToShow;
 };
 
 typedef KstSharedPtr<Kst2DPlot> Kst2DPlotPtr;
