@@ -21,6 +21,7 @@
 #include <qlistbox.h>
 #include <qspinbox.h>
 #include <qvbox.h>
+#include <qradiobutton.h>
 
 // include files for KDE
 #include <kcombobox.h>
@@ -39,6 +40,7 @@
 #include "kstviewwindow.h"
 #include "kstuinames.h"
 #include "vectorselector.h"
+#include "kstplotlabel.h"
 
 const QString& KstCurveDialogI::defaultTag = KGlobal::staticQString("<Auto Name>");
 
@@ -133,12 +135,12 @@ void KstCurveDialogI::fillFieldsForEdit() {
     _legendText->setText(cp->legendText());
   }
 
-  _w->_xVector->setSelection(cp->xVTag());
-  _w->_yVector->setSelection(cp->yVTag());
-  _w->_xError->setSelection(cp->xETag());
-  _w->_yError->setSelection(cp->yETag());
-  _w->_xMinusError->setSelection(cp->xEMinusTag());
-  _w->_yMinusError->setSelection(cp->yEMinusTag());
+  _w->_xVector->setSelection(cp->xVTag().displayString());
+  _w->_yVector->setSelection(cp->yVTag().displayString());
+  _w->_xError->setSelection(cp->xETag().displayString());
+  _w->_yError->setSelection(cp->yETag().displayString());
+  _w->_xMinusError->setSelection(cp->xEMinusTag().displayString());
+  _w->_yMinusError->setSelection(cp->yEMinusTag().displayString());
 
   if (cp->xEMinusTag() == cp->xETag()) {
     _w->_checkBoxXMinusSameAsPlus->setChecked(true);
@@ -166,6 +168,7 @@ void KstCurveDialogI::fillFieldsForEdit() {
   adjustSize();
   resize(minimumSizeHint());
   setFixedHeight(height());
+  _w->_interp->setCurrentItem(cp->interp());
 }
 
 
@@ -176,11 +179,20 @@ void KstCurveDialogI::fillFieldsForNew() {
   _legendText->setText(defaultTag);
   _w->_curvePlacement->update();
 
+  // set the X Axis Vector to the X axis vector of 
+  // the last curve on the global curve list...
+  if (curves.count() >0) {
+    _w->_xVector->setSelection(curves.last()->xVTag().displayString());
+  }
+
   // for some reason the lower widget needs to be shown first to prevent overlapping?
   _w->_curveAppearance->hide();
   _w->_curvePlacement->show();
   _w->_curveAppearance->show();
   _w->_curveAppearance->reset();
+  _w->_interp->setCurrentItem(0);
+  _w->_checkBoxIgnoreAutoscale->setChecked(false);
+
   adjustSize();
   resize(minimumSizeHint());
   setFixedHeight(height());
@@ -224,7 +236,7 @@ bool KstCurveDialogI::newObject() {
   }
   KST::vectorList.lock().unlock();
   if (!VX || !VY) {
-    kstdFatal() << "Bug in kst: the XVector field in plotDialog refers to "
+    kstdFatal() << "Bug in kst: the XVector field refers to "
                 << "a non existent vector...." << endl;
   }
 
@@ -234,7 +246,7 @@ bool KstCurveDialogI::newObject() {
 
   QString tag_name = _tagName->text();
   if (tag_name == defaultTag) {
-    tag_name = KST::suggestCurveName(VY->tagName());
+    tag_name = KST::suggestCurveName(VY->tag());
   }
 
   // verify that the curve name is unique
@@ -292,6 +304,8 @@ bool KstCurveDialogI::newObject() {
   curve->setPointDensity(_w->_curveAppearance->pointDensity());
   curve->setIgnoreAutoScale(_w->_checkBoxIgnoreAutoscale->isChecked());
 
+  curve->setInterp(KstVCurve::InterpType(_w->_interp->currentItem()));
+
   if (_w->_curvePlacement->existingPlot() || _w->_curvePlacement->newPlot()) {
     KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_w->_curvePlacement->_plotWindow->currentText()));
     if (!w) {
@@ -310,7 +324,7 @@ bool KstCurveDialogI::newObject() {
 
       if (_w->_curvePlacement->newPlot()) {
         // assign curve to plot
-        QString name = w->createObject<Kst2DPlot>(KST::suggestPlotName());
+	QString name = w->createPlot(KST::suggestPlotName());
         plot = kst_cast<Kst2DPlot>(w->view()->findChild(name));
         if (_w->_curvePlacement->reGrid()) {
           w->view()->cleanup(_w->_curvePlacement->columns());
@@ -420,6 +434,10 @@ bool KstCurveDialogI::editSingleObject(KstVCurvePtr cvPtr) {
     cvPtr->setIgnoreAutoScale(_w->_checkBoxIgnoreAutoscale->isChecked());
   }
 
+  if (_interpComboDirty) {
+    cvPtr->setInterp(KstVCurve::InterpType(_w->_interp->currentItem()));
+  }
+
   cvPtr->unlock();
   return true;
 }
@@ -444,6 +462,7 @@ bool KstCurveDialogI::editObject() {
     _comboLineStyleDirty = _w->_curveAppearance->_comboLineStyle->currentItem() > 0;
     _comboPointDensityDirty = _w->_curveAppearance->_comboPointDensity->currentItem() > 0;
     _barStyleDirty = _w->_curveAppearance->_barStyle->currentItem() > 0;
+    _interpComboDirty = (_w->_interp->currentItem()<4);
 
     bool didEdit = false;
 
@@ -478,7 +497,7 @@ bool KstCurveDialogI::editObject() {
     }
 
     cp->writeLock();
-    cp->setTagName(tag_name);
+    cp->setTagName(KstObjectTag(tag_name, cp->tag().context())); // FIXME: doesn't allow changing tag context
     QString legend_text = _legendText->text();
     if (legend_text==defaultTag) {
       cp->setLegendText(QString(""));
@@ -505,6 +524,7 @@ bool KstCurveDialogI::editObject() {
     _comboLineStyleDirty = true;
     _spinBoxLineWidthDirty = true;
     _barStyleDirty = true;
+    _interpComboDirty = true;
     _checkBoxIgnoreAutoscaleDirty = true;
     if (!editSingleObject(cp)) {
       return false;
@@ -533,6 +553,9 @@ void KstCurveDialogI::populateEditMultiple() {
   _w->_yMinusError->_vector->insertItem("", 0);
   _w->_yMinusError->_vector->setCurrentItem(0);
 
+  _w->_interp->insertItem("", 4);
+  _w->_interp->setCurrentItem(4);
+  
   // single blank characters to differentiate between QPixmaps
   _w->_curveAppearance->_combo->insertItem(" ", 0);
   _w->_curveAppearance->_combo->setCurrentItem(0);
@@ -542,7 +565,7 @@ void KstCurveDialogI::populateEditMultiple() {
   _w->_curveAppearance->_comboLineStyle->setCurrentItem(0);
   _w->_curveAppearance->_barStyle->insertItem(" ", 0);
   _w->_curveAppearance->_barStyle->setCurrentItem(0);
-
+  
   _w->_curveAppearance->_spinBoxLineWidth->setMinValue(_w->_curveAppearance->_spinBoxLineWidth->minValue() - 1);
   _w->_curveAppearance->_spinBoxLineWidth->setSpecialValueText(" ");
   _w->_curveAppearance->_spinBoxLineWidth->setValue(_w->_curveAppearance->_spinBoxLineWidth->minValue());
@@ -559,7 +582,7 @@ void KstCurveDialogI::populateEditMultiple() {
   _w->_curveAppearance->_showBars->setNoChange();
   _w->_checkBoxIgnoreAutoscale->setTristate(true);
   _w->_checkBoxIgnoreAutoscale->setNoChange();
-
+  
   toggledXErrorSame(false);
   toggledYErrorSame(false);
 
@@ -587,6 +610,7 @@ void KstCurveDialogI::populateEditMultiple() {
   _spinBoxLineWidthDirty = false;
   _barStyleDirty = false;
   _checkBoxIgnoreAutoscaleDirty = false;
+  _interpComboDirty = false;
 }
 
 
@@ -635,6 +659,7 @@ void KstCurveDialogI::cleanup() {
     _w->_curveAppearance->_barStyle->removeItem(0);
     _w->_curveAppearance->_spinBoxLineWidth->setSpecialValueText(QString::null);
     _w->_curveAppearance->_spinBoxLineWidth->setMinValue(_w->_curveAppearance->_spinBoxLineWidth->minValue() + 1);
+    _w->_interp->removeItem(4);
   }
 }
 
