@@ -19,12 +19,23 @@
 
 // include files for Qt
 #include <qptrstack.h>
+#include <qtoolbox.h>
+
+#include <qtoolbutton.h>
+#include <qstylefactory.h>
+#include <qstyle.h>
+
+static QStyle *windowsStyle = 0;
 
 // include files for KDE
 #include "ksdebug.h"
 #include <klistview.h>
 #include <kmessagebox.h>
+#include <kinputdialog.h>
 #include <kstandarddirs.h>
+#include <kcombobox.h>
+
+#include <klistviewsearchline.h>
 
 // application specific includes
 #include "datasourcemetadatadialog.h"
@@ -46,6 +57,7 @@
 #include "kstviewwindow.h"
 #include "matrixselector.h"
 #include "vectorselector.h"
+#include "plugincollection.h"
 
 
 static QMap<int,Kst2DPlotPtr> PlotMap;
@@ -59,10 +71,10 @@ static QMap<int,Kst2DPlotPtr> PlotMap;
 #define RTTI_OBJ_STATIC_MATRIX   4207
 
 KstObjectItem::KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm, int localUseCount)
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_DATA_VECTOR), _name(x->tagName()), _dm(dm) {
+: QObject(), KListViewItem(parent), _rtti(RTTI_OBJ_DATA_VECTOR), _tag(x->tag()), _dm(dm) {
   assert(x);
   _inUse = false;
-  setText(0, x->tagName());
+  setText(0, x->tag().tag());
   setText(1, i18n("Data Vector"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
@@ -70,21 +82,21 @@ KstObjectItem::KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI
 
 
 KstObjectItem::KstObjectItem(QListView *parent, KstSVectorPtr x, KstDataManagerI *dm, int localUseCount)
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_STATIC_VECTOR), _name(x->tagName()), _dm(dm) {
+: QObject(), KListViewItem(parent), _rtti(RTTI_OBJ_STATIC_VECTOR), _tag(x->tag()), _dm(dm) {
   assert(x);
   _inUse = false;
-  setText(0, x->tagName());
+  setText(0, x->tag().tag());
   setText(1, i18n("Static Vector"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
 }
 
 
-KstObjectItem::KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm, int localUseCount)
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_VECTOR), _name(x->tagName()), _dm(dm) {
+KstObjectItem::KstObjectItem(KListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm, int localUseCount)
+: QObject(), KListViewItem(parent), _rtti(RTTI_OBJ_VECTOR), _tag(x->tag()), _dm(dm) {
   assert(x);
   _inUse = false;
-  setText(0, x->tagName());
+  setText(0, x->tag().tag());
   setText(1, i18n("Slave Vector"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
@@ -92,10 +104,10 @@ KstObjectItem::KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManag
 
 
 KstObjectItem::KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManagerI *dm, int localUseCount)
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_OBJECT), _name(x->tagName()), _dm(dm) {
+: QObject(), KListViewItem(parent), _rtti(RTTI_OBJ_OBJECT), _tag(x->tag()), _dm(dm) {
   assert(x);
   _inUse = false;
-  setText(0, x->tagName());
+  setText(0, x->tag().tag());
   for (KstVectorMap::Iterator i = x->outputVectors().begin();
       i != x->outputVectors().end();
       ++i) {
@@ -114,10 +126,10 @@ KstObjectItem::KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManag
 
 
 KstObjectItem::KstObjectItem(QListView *parent, KstRMatrixPtr x, KstDataManagerI *dm, int localUseCount) 
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_DATA_MATRIX), _name(x->tagName()), _dm(dm) {
+: QObject(), KListViewItem(parent), _rtti(RTTI_OBJ_DATA_MATRIX), _tag(x->tag()), _dm(dm) {
   assert(x);
   _inUse = false;
-  setText(0, x->tagName());
+  setText(0, x->tag().tag());
   setText(1, i18n("Data Matrix"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
@@ -125,21 +137,21 @@ KstObjectItem::KstObjectItem(QListView *parent, KstRMatrixPtr x, KstDataManagerI
 
 
 KstObjectItem::KstObjectItem(QListView *parent, KstSMatrixPtr x, KstDataManagerI *dm, int localUseCount) 
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_STATIC_MATRIX), _name(x->tagName()), _dm(dm) {
+: QObject(), KListViewItem(parent), _rtti(RTTI_OBJ_STATIC_MATRIX), _tag(x->tag()), _dm(dm) {
   assert(x);
   _inUse = false;
-  setText(0, x->tagName());
+  setText(0, x->tag().tag());
   setText(1, i18n("Static Matrix"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
 }
 
 
-KstObjectItem::KstObjectItem(QListViewItem *parent, KstMatrixPtr x, KstDataManagerI *dm, int localUseCount) 
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_MATRIX), _name(x->tagName()), _dm(dm) {
+KstObjectItem::KstObjectItem(KListViewItem *parent, KstMatrixPtr x, KstDataManagerI *dm, int localUseCount) 
+: QObject(), KListViewItem(parent), _rtti(RTTI_OBJ_MATRIX), _tag(x->tag()), _dm(dm) {
   assert(x);
   _inUse = false;
-  setText(0, x->tagName());
+  setText(0, x->tag().tag());
   setText(1, i18n("Slave Matrix"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
@@ -151,7 +163,7 @@ KstObjectItem::~KstObjectItem() {
 
 
 KstDataObjectPtr KstObjectItem::dataObject() {
-  return *KST::dataObjectList.findTag(_name);
+  return *KST::dataObjectList.findTag(_tag.tag());
 }
 
 
@@ -159,7 +171,9 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
   switch (_rtti) {
     case RTTI_OBJ_DATA_VECTOR:
     {
-      KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(_name));
+      KST::vectorList.lock().readLock();
+      KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(_tag));
+      KST::vectorList.lock().unlock();
       if (x) {
         x->readLock();
         // getUsage: subtract 1 for KstRVectorPtr x
@@ -192,7 +206,9 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
     }
     case RTTI_OBJ_STATIC_VECTOR:
     {
-      KstSVectorPtr x = kst_cast<KstSVector>(*KST::vectorList.findTag(_name));
+      KST::vectorList.lock().readLock();
+      KstSVectorPtr x = kst_cast<KstSVector>(*KST::vectorList.findTag(_tag));
+      KST::vectorList.lock().unlock();
       if (x) {
         x->readLock();
         // getUsage: subtract 1 for KstRVectorPtr x
@@ -222,7 +238,9 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
     }
     case RTTI_OBJ_VECTOR:
     {
-      KstVectorPtr x = *KST::vectorList.findTag(_name);
+      KST::vectorList.lock().readLock();
+      KstVectorPtr x = *KST::vectorList.findTag(_tag);
+      KST::vectorList.lock().unlock();
       if (x) {
         x->readLock();
         // getUsage:
@@ -247,7 +265,9 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
     }
     case RTTI_OBJ_OBJECT:
     {
-      KstDataObjectPtr x = *KST::dataObjectList.findTag(_name);
+      KST::dataObjectList.lock().readLock();
+      KstDataObjectPtr x = *KST::dataObjectList.findTag(_tag.tag());
+      KST::dataObjectList.lock().unlock();
       if (x) {
         x->readLock();
         QString field = x->typeString();
@@ -282,7 +302,7 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
 
           for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
             KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-            if (vl.findTag(oi->tagName()) == vlEnd) {
+            if (vl.findTag(oi->tag().tag()) == vlEnd) {
               trash.push(i);
             }
           }
@@ -292,10 +312,10 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
           // get the output vectors
           for (KstVectorMap::Iterator p = vl.begin(); p != vlEnd; ++p) {
             bool found = false;
-            QString tn = p.data()->tagName();
+            QString tn = p.data()->tag().tag();
             for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
               KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-              if (oi->tagName() == tn) {
+              if (oi->tag().tag() == tn) {
                 oi->update();
                 found = true;
                 break;
@@ -312,10 +332,10 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
           // also get the output matrices
           for (KstMatrixMap::Iterator p = ml.begin(); p != mlEnd; ++p) {
             bool found = false;
-            QString tn = p.data()->tagName();
+            QString tn = p.data()->tag().tag();
             for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
               KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-              if (oi->tagName() == tn) {
+              if (oi->tag().tag() == tn) {
                 oi->update();
                 found = true;
                 break;
@@ -334,7 +354,9 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
     }
     case RTTI_OBJ_DATA_MATRIX:
     {
-      KstRMatrixPtr x = kst_cast<KstRMatrix>(*KST::matrixList.findTag(_name));
+      KST::matrixList.lock().readLock();
+      KstRMatrixPtr x = kst_cast<KstRMatrix>(*KST::matrixList.findTag(_tag));
+      KST::matrixList.lock().unlock();
       if (x) {
         x->readLock();
           // getUsage: subtract 1 for KstRMatrixPtr x
@@ -360,7 +382,9 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
     }
     case RTTI_OBJ_STATIC_MATRIX:
     {
-      KstSMatrixPtr x = kst_cast<KstSMatrix>(*KST::matrixList.findTag(_name));
+      KST::matrixList.lock().readLock();
+      KstSMatrixPtr x = kst_cast<KstSMatrix>(*KST::matrixList.findTag(_tag));
+      KST::matrixList.lock().unlock();
       if (x) {
         x->readLock();
           // getUsage: subtract 1 for KstRMatrixPtr x
@@ -384,7 +408,9 @@ void KstObjectItem::update(bool recursive, int localUseCount) {
     }
     case RTTI_OBJ_MATRIX:
     {
-      KstMatrixPtr x = *KST::matrixList.findTag(_name);
+      KST::matrixList.lock().readLock();
+      KstMatrixPtr x = *KST::matrixList.findTag(_tag);
+      KST::matrixList.lock().unlock();
       if (x) {
         x->readLock();
           // getUsage:
@@ -422,7 +448,7 @@ void KstObjectItem::updateButtons() {
 void KstObjectItem::reload() {
   if (_rtti == RTTI_OBJ_DATA_VECTOR) {
     KstReadLocker ml(&KST::vectorList.lock());
-    KstVectorList::Iterator v = KST::vectorList.findTag(_name);
+    KstVectorList::Iterator v = KST::vectorList.findTag(_tag);
     if (v != KST::vectorList.end()) {
       KstRVectorPtr r = kst_cast<KstRVector>(*v);
       if (r) {
@@ -433,7 +459,7 @@ void KstObjectItem::reload() {
     }
   } else if (_rtti == RTTI_OBJ_DATA_MATRIX) {
     KstReadLocker ml(&KST::matrixList.lock());
-    KstMatrixList::Iterator m = KST::matrixList.findTag(_name);
+    KstMatrixList::Iterator m = KST::matrixList.findTag(_tag);
     if (m != KST::matrixList.end()) {
       KstRMatrixPtr r = kst_cast<KstRMatrix>(*m);
       if (r) {
@@ -448,29 +474,29 @@ void KstObjectItem::reload() {
 
 void KstObjectItem::makeCurve() {
   KstCurveDialogI::globalInstance()->show();
-  KstCurveDialogI::globalInstance()->setVector(_name);
+  KstCurveDialogI::globalInstance()->setVector(_tag.tag());
 }
 
 void KstObjectItem::makeCSD() {
   KstCsdDialogI::globalInstance()->show();
-  KstCsdDialogI::globalInstance()->setVector(_name);
+  KstCsdDialogI::globalInstance()->setVector(_tag.tag());
 }
 
 void KstObjectItem::makePSD() {
   KstPsdDialogI::globalInstance()->show();
-  KstPsdDialogI::globalInstance()->setVector(_name);
+  KstPsdDialogI::globalInstance()->setVector(_tag.tag());
 }
 
 
 void KstObjectItem::makeHistogram() {
   KstHsDialogI::globalInstance()->show();
-  KstHsDialogI::globalInstance()->setVector(_name);
+  KstHsDialogI::globalInstance()->setVector(_tag.tag());
 }
 
 
 void KstObjectItem::makeImage() {
   KstImageDialogI::globalInstance()->show();
-  KstImageDialogI::globalInstance()->setMatrix(_name);
+  KstImageDialogI::globalInstance()->setMatrix(_tag.tag());
 }
 
 
@@ -478,7 +504,7 @@ void KstObjectItem::showMetadata() {
   if (_rtti == RTTI_OBJ_DATA_VECTOR) {
     DataSourceMetaDataDialog *dlg = new DataSourceMetaDataDialog(_dm, 0, false, WDestructiveClose);
     KstReadLocker vl(&KST::vectorList.lock());
-    KstVectorList::Iterator m = KST::vectorList.findTag(_name);
+    KstVectorList::Iterator m = KST::vectorList.findTag(_tag);
     KstRVectorPtr r = kst_cast<KstRVector>(*m);
     KstDataSourcePtr dsp;
     if (r) {
@@ -491,7 +517,7 @@ void KstObjectItem::showMetadata() {
   } else if (_rtti == RTTI_OBJ_DATA_MATRIX) {
     DataSourceMetaDataDialog *dlg = new DataSourceMetaDataDialog(_dm, 0, false, WDestructiveClose);
     KstReadLocker ml(&KST::matrixList.lock());
-    KstMatrixList::Iterator m = KST::matrixList.findTag(_name);
+    KstMatrixList::Iterator m = KST::matrixList.findTag(_tag);
     KstRMatrixPtr r = kst_cast<KstRMatrix>(*m);
     KstDataSourcePtr dsp;
     if (r) {
@@ -511,7 +537,7 @@ void KstObjectItem::activateHint(int id) {
   int cnt = 0;
   for (KstCurveHintList::ConstIterator i = hints->begin(); i != hints->end(); ++i) {
     if (cnt == id) {
-      KstBaseCurvePtr c = (*i)->makeCurve(KST::suggestCurveName(d->tagName(), false), KstColorSequence::next());
+      KstBaseCurvePtr c = (*i)->makeCurve(KST::suggestCurveName(d->tag(), false), KstColorSequence::next());
       if (c) {
         KST::dataObjectList.lock().writeLock();
         KST::dataObjectList.append(c.data());
@@ -570,7 +596,6 @@ const QPixmap& KstDataManagerI::yesPixmap() const {
   return _yesPixmap;
 }
 
-
 KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* name, bool modal, WFlags fl)
 : KstDataManager(parent, name, modal, fl) {
   doc = in_doc;
@@ -584,26 +609,99 @@ KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* na
       this, SLOT(edit_I()));
   connect(DataView, SIGNAL(currentChanged(QListViewItem *)),
       this, SLOT(currentChanged(QListViewItem *)));
-  connect(DataView, SIGNAL(selectionChanged(QListViewItem *)),
-      this, SLOT(currentChanged(QListViewItem *)));
-
-  connect(NewCurve, SIGNAL(clicked()), KstCurveDialogI::globalInstance(), SLOT(show()));
-  connect(NewHs, SIGNAL(clicked()), KstHsDialogI::globalInstance(), SLOT(show()));
-  connect(NewEq, SIGNAL(clicked()), KstEqDialogI::globalInstance(), SLOT(show()));
-  connect(NewPlugin, SIGNAL(clicked()), KstPluginDialogI::globalInstance(), SLOT(show()));
-  connect(NewEvent, SIGNAL(clicked()), KstEventMonitorI::globalInstance(), SLOT(show()));
-  connect(NewPSD, SIGNAL(clicked()), KstPsdDialogI::globalInstance(), SLOT(show()));
-  connect(NewVector, SIGNAL(clicked()), KstVectorDialogI::globalInstance(), SLOT(show()));
-  connect(OpenPlotDialog, SIGNAL(clicked()),
-      KstApp::inst(), SLOT(showPlotDialog()));
-
-  connect(NewImage, SIGNAL(clicked()), KstImageDialogI::globalInstance(), SLOT(show()));
-
-  connect(NewMatrix, SIGNAL(clicked()), KstMatrixDialogI::globalInstance(), SLOT(show()));
-  
-  connect(NewCSD, SIGNAL(clicked()), KstCsdDialogI::globalInstance(), SLOT(show()));
-
+  connect(DataView, SIGNAL(selectionChanged()),
+      this, SLOT(selectionChanged()));
   connect(DataView, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)), this, SLOT(contextMenu(QListViewItem*, const QPoint&, int)));
+
+  _searchWidget = new KListViewSearchLineWidget(DataView, SearchBox);
+  QValueList<int> cols;
+  cols.append(0);
+  _searchWidget->createSearchLine(DataView);
+  _searchWidget->searchLine()->setSearchColumns(cols);
+
+  QMainWindow *main = static_cast<QMainWindow*>(parent);
+  main->setUsesTextLabel(true);
+
+  _primitive = new QToolBar(i18n("Create Primitive"), main, this);
+  _primitive->setFrameStyle(QFrame::NoFrame);
+  _primitive->setOrientation(Qt::Vertical);
+  _primitive->setBackgroundMode(PaletteBase);
+
+  _data = new QToolBar(i18n("Create Data Object"), main, this);
+  _data->setFrameStyle(QFrame::NoFrame);
+  _data->setOrientation(Qt::Vertical);
+  _data->setBackgroundMode(PaletteBase);
+
+  _fits = new QToolBar(i18n("Create Fit"), main, this);
+  _fits->setFrameStyle(QFrame::NoFrame);
+  _fits->setOrientation(Qt::Vertical);
+  _fits->setBackgroundMode(PaletteBase);
+
+  _filters = new QToolBar(i18n("Create Filter"), main, this);
+  _filters->setFrameStyle(QFrame::NoFrame);
+  _filters->setOrientation(Qt::Vertical);
+  _filters->setBackgroundMode(PaletteBase);
+
+  ToolBox->setUpdatesEnabled(false);
+
+  _primitive->setUpdatesEnabled(false);
+  _primitive->clear();
+
+  _data->setUpdatesEnabled(false);
+  _data->clear();
+
+  _fits->setUpdatesEnabled(false);
+  _fits->clear();
+
+  _filters->setUpdatesEnabled(false);
+  _filters->clear();
+
+  //Create canonical actions...
+//   createObjectAction(i18n("Scalar"), _primitive, KstScalarDialogI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Vector"), _primitive, KstVectorDialogI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Matrix"), _primitive, KstMatrixDialogI::globalInstance(), SLOT(show()));
+//   createObjectAction(i18n("String"), _primitive, KstStringDialogI::globalInstance(), SLOT(show()));
+
+  createObjectAction(i18n("Curve"), _data, KstCurveDialogI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Equation"), _data, KstEqDialogI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Histogram"), _data, KstHsDialogI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Spectrum"), _data, KstPsdDialogI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Event Monitor"), _data, KstEventMonitorI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Image"), _data, KstImageDialogI::globalInstance(), SLOT(show()));
+  createObjectAction(i18n("Spectrogram"), _data, KstCsdDialogI::globalInstance(), SLOT(show()));
+
+  //Create plugin actions...
+  setupPluginActions();
+
+  //TODO sort the actions in each box alphabetically?
+
+  QWidget *priw = new QWidget(_primitive);
+  priw->setBackgroundMode(PaletteBase);
+  _primitive->setStretchableWidget(priw);
+
+  QWidget *datw = new QWidget(_data);
+  datw->setBackgroundMode(PaletteBase);
+  _data->setStretchableWidget(datw);
+
+  QWidget *fitw = new QWidget(_fits);
+  fitw->setBackgroundMode(PaletteBase);
+  _fits->setStretchableWidget(fitw);
+
+  QWidget *filw = new QWidget(_filters);
+  filw->setBackgroundMode(PaletteBase);
+  _filters->setStretchableWidget(filw);
+
+  ToolBox->setUpdatesEnabled(true);
+
+  _primitive->setUpdatesEnabled(true);
+  _data->setUpdatesEnabled(true);
+  _fits->setUpdatesEnabled(true);
+  _filters->setUpdatesEnabled(true);
+
+  ToolBox->addItem(_primitive, i18n("Create Primitive"));
+  ToolBox->addItem(_data, i18n("Create Data Object"));
+  ToolBox->addItem(_fits, i18n("Create Fit"));
+  ToolBox->addItem(_filters, i18n("Create Filter"));
 }
 
 
@@ -611,6 +709,102 @@ KstDataManagerI::~KstDataManagerI() {
 }
 
 
+void KstDataAction::addedTo(QWidget *actionWidget, QWidget *container) {
+  Q_UNUSED(container)
+  if ( ::qt_cast<QToolButton*>(actionWidget) ) {
+    if (!windowsStyle)
+      windowsStyle = QStyleFactory::create("windows");
+    actionWidget->setStyle(windowsStyle);
+
+    ( (QToolButton*)actionWidget )->setUsesTextLabel(true);
+    ( (QToolButton*)actionWidget )->setTextPosition(QToolButton::Right);
+    actionWidget->setBackgroundMode(PaletteBase);
+  }
+}
+
+
+void KstDataManagerI::createObjectAction(const QString &txt, QToolBar *bar,
+                                         QObject *receiver, const char *slot) {
+  KstDataAction *a = new KstDataAction(txt, QKeySequence(), bar);
+  a->addTo(bar);
+  if (receiver && slot) {
+    connect(a, SIGNAL(activated()), receiver, slot);
+  }
+}
+
+
+void KstDataManagerI::setupPluginActions() {
+
+  //The new KstDataObject plugins...
+  {
+    const KstPluginInfoList newPlugins = KstDataObject::pluginInfoList();
+    KstPluginInfoList::ConstIterator it = newPlugins.begin();
+    for (; it != newPlugins.end(); ++it) {
+
+      KstDataObjectPtr ptr = KstDataObject::plugin(it.key());
+      if (!ptr)
+        continue;
+
+      switch(it.data()) {
+      case KstDataObject::Generic:
+        createObjectAction(it.key(), _data, ptr, SLOT(showNewDialog()));
+        break;
+      case KstDataObject::Primitive:
+        createObjectAction(it.key(), _primitive, ptr, SLOT(showNewDialog()));
+        break;
+      case KstDataObject::Fit:
+        createObjectAction(it.key(), _fits, ptr, SLOT(showNewDialog()));
+        break;
+      case KstDataObject::Filter:
+        createObjectAction(it.key(), _filters, ptr, SLOT(showNewDialog()));
+        break;
+      default:
+        break;
+      }
+    }
+  }
+
+  //The old C style plugins...
+  QStringList oldPlugins;
+  const QMap<QString,QString> readable =
+    PluginCollection::self()->readableNameList();
+  QMap<QString,QString>::const_iterator it = readable.begin();
+  for (; it != readable.end(); ++it) {
+    oldPlugins << it.key();
+  }
+
+  {
+    QStringList::ConstIterator it = oldPlugins.begin();
+    for (; it != oldPlugins.end(); ++it) {
+      if (KstSharedPtr<Plugin> p = PluginCollection::self()->plugin(readable[*it])) {
+        if (p->data()._isFit)
+          createObjectAction(*it, _fits, this, SLOT(showOldPlugin()));
+        else if (p->data()._isFilter)
+          createObjectAction(*it, _filters, this, SLOT(showOldPlugin()));
+        else
+          createObjectAction(*it, _data, this, SLOT(showOldPlugin()));
+      }
+    }
+  }
+}
+
+
+void KstDataManagerI::showOldPlugin() {
+
+  if (QAction *a = ::qt_cast<QAction*>(sender())) {
+    const QMap<QString,QString> readable =
+      PluginCollection::self()->readableNameList();
+    KstPluginDialogI::globalInstance()->showNew(readable[a->text()]);
+  }
+}
+
+
+void KstDataManagerI::doubleClicked(QListViewItem *i) {
+  if (i && DataView->selectedItems().contains(i)) {
+    edit_I();
+  }
+}
+
 void KstDataManagerI::show_I() {
   show();
   raise();
@@ -627,6 +821,8 @@ void KstDataManagerI::updateContents() {
     KstObjectItem *oi = static_cast<KstObjectItem*>(i);
     oi->update();
   }
+
+  _searchWidget->searchLine()->updateSearch();
 }
 
 
@@ -646,17 +842,17 @@ void KstDataManagerI::update() {
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     KstObjectItem *oi = static_cast<KstObjectItem*>(i);
     if (i->rtti() == RTTI_OBJ_OBJECT) {
-      if (KST::dataObjectList.findTag(oi->tagName()) == KST::dataObjectList.end()) {
+      if (KST::dataObjectList.findTag(oi->tag().tagString()) == KST::dataObjectList.end()) {
         trash.push(i);
       }
     } else if (i->rtti() == RTTI_OBJ_DATA_MATRIX || 
                i->rtti() == RTTI_OBJ_MATRIX ||
                i->rtti() == RTTI_OBJ_STATIC_MATRIX) {
-      if (KST::matrixList.findTag(oi->tagName()) == KST::matrixList.end()) {
+      if (KST::matrixList.findTag(oi->tag().tagString()) == KST::matrixList.end()) {
         trash.push(i);  
       }
     } else {
-      if (KST::vectorList.findTag(oi->tagName()) == KST::vectorList.end()) {
+      if (KST::vectorList.findTag(oi->tag().tagString()) == KST::vectorList.end()) {
         trash.push(i);
       }
     }
@@ -671,10 +867,11 @@ void KstDataManagerI::update() {
   for (KstDataObjectList::iterator it = KST::dataObjectList.begin();
                                     it != KST::dataObjectList.end();
                                                                ++it) {
+    KstReadLocker dol(*it);
     bool found = false;
     for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
       KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-      if (oi->rtti() == RTTI_OBJ_OBJECT && oi->tagName() == (*it)->tagName()) {
+      if (oi->rtti() == RTTI_OBJ_OBJECT && oi->tag().tagString() == (*it)->tag().tagString()) {
         oi->update();
         found = true;
         break;
@@ -686,17 +883,16 @@ void KstDataManagerI::update() {
     }
   }
 
-  KST::matrixList.lock().unlock();
-  KST::vectorList.lock().unlock();
   KST::dataObjectList.lock().unlock();
 
   // update the data vectors
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (KstRVectorList::iterator it = rvl.begin(); it != rvl.end(); ++it) {
+    KstReadLocker vl(*it);
     bool found = false;
     for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
       KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-      if (oi->rtti() == RTTI_OBJ_DATA_VECTOR && oi->tagName() == (*it)->tagName()) {
+      if (oi->rtti() == RTTI_OBJ_DATA_VECTOR && oi->tag().tagString() == (*it)->tag().tagString()) {
         oi->update(true, 1);
         found = true;
         break;
@@ -711,10 +907,11 @@ void KstDataManagerI::update() {
   // update the static vectors
   KstSVectorList svl = kstObjectSubList<KstVector,KstSVector>(KST::vectorList);
   for (KstSVectorList::iterator it = svl.begin(); it != svl.end(); ++it) {
+    KstReadLocker vl(*it);
     bool found = false;
     for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
       KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-      if (oi->rtti() == RTTI_OBJ_STATIC_VECTOR && oi->tagName() == (*it)->tagName()) {
+      if (oi->rtti() == RTTI_OBJ_STATIC_VECTOR && oi->tag().tagString() == (*it)->tag().tagString()) {
         oi->update(true, 1);
         found = true;
         break;
@@ -726,13 +923,16 @@ void KstDataManagerI::update() {
     }
   }
 
+  KST::vectorList.lock().unlock();
+
   // update the data matrices 
   KstRMatrixList rml = kstObjectSubList<KstMatrix,KstRMatrix>(KST::matrixList);
   for (KstRMatrixList::iterator it = rml.begin(); it != rml.end(); ++it) {
+    KstReadLocker ml(*it);
     bool found = false;
     for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
       KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-      if (oi->rtti() == RTTI_OBJ_DATA_MATRIX && oi->tagName() == (*it)->tagName()) {
+      if (oi->rtti() == RTTI_OBJ_DATA_MATRIX && oi->tag().tagString() == (*it)->tag().tagString()) {
         oi->update(true, 1);
         found = true;
         break;
@@ -747,10 +947,11 @@ void KstDataManagerI::update() {
   // update the static matrices
   KstSMatrixList sml = kstObjectSubList<KstMatrix,KstSMatrix>(KST::matrixList);
   for (KstSMatrixList::iterator it = sml.begin(); it != sml.end(); ++it) {
+    KstReadLocker ml(*it);
     bool found = false;
     for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
       KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-      if (oi->rtti() == RTTI_OBJ_STATIC_MATRIX && oi->tagName() == (*it)->tagName()) {
+      if (oi->rtti() == RTTI_OBJ_STATIC_MATRIX && oi->tag().tagString() == (*it)->tag().tagString()) {
         oi->update(true, 1);
         found = true;
         break;
@@ -762,6 +963,8 @@ void KstDataManagerI::update() {
     }
   }
 
+  KST::matrixList.lock().unlock();
+
   // is this really necessary?  I would think not...
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     if (i == currentItem) {
@@ -781,115 +984,121 @@ void KstDataManagerI::update() {
 
 
 void KstDataManagerI::edit_I() {
-  QListViewItem *qi;
+  QListViewItem *qi = 0;
 
-  if (DataView->selectedItems().count() > 0) {
+  if (!DataView->selectedItems().isEmpty()) {
     qi = DataView->selectedItems().at(0);
   } else {
-    KMessageBox::sorry(this, i18n("A data item must be selected to edit."));
+    // This error message is odd to say the least.
+    //KMessageBox::sorry(this, i18n("A data item must be selected to edit."));
     return;
   }
 
-  if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
-    emit editDataVector(qi->text(0));
-  }
-
-  if (qi->rtti() == RTTI_OBJ_STATIC_VECTOR) {
-    emit editStaticVector(qi->text(0));
-  }
-
-  if (qi->rtti() == RTTI_OBJ_OBJECT) {
-    static_cast<KstObjectItem*>(qi)->dataObject()->showDialog();
-  }
-  
-  if (qi->rtti() == RTTI_OBJ_DATA_MATRIX) {
-    emit editDataMatrix(qi->text(0));  
-  }
-  
-  if (qi->rtti() == RTTI_OBJ_STATIC_MATRIX) {
-    emit editStaticMatrix(qi->text(0));  
+  switch (qi->rtti()) {
+    case RTTI_OBJ_DATA_VECTOR:
+      emit editDataVector(qi->text(0));
+      break;
+    case RTTI_OBJ_STATIC_VECTOR:
+      emit editStaticVector(qi->text(0));
+      break;
+    case RTTI_OBJ_OBJECT:
+      static_cast<KstObjectItem*>(qi)->dataObject()->showDialog(false);
+      break;
+    case RTTI_OBJ_DATA_MATRIX:
+      emit editDataMatrix(qi->text(0));  
+      break;
+    case RTTI_OBJ_STATIC_MATRIX:
+      emit editStaticMatrix(qi->text(0));  
+      break;
+  default:
+      break;
   }
 }
 
 
 void KstDataManagerI::delete_I() {
   QListViewItem *qi = DataView->selectedItems().at(0);
+  if (!qi) {
+    return;
+  }
   KstObjectItem *koi = static_cast<KstObjectItem*>(qi);
 
   if (koi->removable()) {
     if (qi->rtti() == RTTI_OBJ_OBJECT) {
-      doc->removeDataObject(koi->tagName());
+      doc->removeDataObject(koi->tag().tagString());
     } else if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
       KST::vectorList.lock().writeLock();
-      KST::vectorList.removeTag(koi->tagName());
+      KST::vectorList.removeTag(koi->tag().tagString());
       KST::vectorList.lock().unlock();
       doUpdates();
     } else if (qi->rtti() == RTTI_OBJ_STATIC_VECTOR) {
       KST::vectorList.lock().writeLock();
-      KST::vectorList.removeTag(koi->tagName());
+      KST::vectorList.removeTag(koi->tag().tagString());
       KST::vectorList.lock().unlock();
       doUpdates();
     } else if (qi->rtti() == RTTI_OBJ_DATA_MATRIX) {
       KST::matrixList.lock().writeLock();
-      KST::matrixList.removeTag(koi->tagName());
+      KST::matrixList.removeTag(koi->tag().tagString());
       KST::matrixList.lock().unlock();  
+      doUpdates();
     } else if (qi->rtti() == RTTI_OBJ_STATIC_MATRIX) {
       KST::matrixList.lock().writeLock();
-      KST::matrixList.removeTag(koi->tagName());
+      KST::matrixList.removeTag(koi->tag().tagString());
       KST::matrixList.lock().unlock();  
+      doUpdates();
     }
     update();
   } else {
     // Don't prompt for base curves
     KstBaseCurvePtr bc = kst_cast<KstBaseCurve>(koi->dataObject());
-    if (bc || KMessageBox::warningYesNo(this, i18n("There are other objects in memory that depend on %1.  Do you wish to delete them too?").arg(koi->tagName())) == KMessageBox::Yes) {
+    if (bc || KMessageBox::warningYesNo(this, i18n("There are other objects in memory that depend on %1.  Do you wish to delete them too?").arg(koi->tag().tag())) == KMessageBox::Yes) {
 
       if (qi->rtti() == RTTI_OBJ_OBJECT) {
         koi->dataObject()->deleteDependents();
-        doc->removeDataObject(koi->tagName());
+        doc->removeDataObject(koi->tag().tagString());
       } else if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
-        KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(koi->tagName()));
+        KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(koi->tag().tagString()));
         if (x) {
           x->deleteDependents();
           x = 0L;
           KST::vectorList.lock().writeLock();
-          KST::vectorList.removeTag(koi->tagName());
+          KST::vectorList.removeTag(koi->tag().tagString());
           KST::vectorList.lock().unlock();
           doUpdates();
         } else {
           KMessageBox::sorry(this, i18n("Unknown error deleting data vector."));
         }
       } else if (qi->rtti() == RTTI_OBJ_STATIC_VECTOR) {
-        KstSVectorPtr x = kst_cast<KstSVector>(*KST::vectorList.findTag(koi->tagName()));
+        KstSVectorPtr x = kst_cast<KstSVector>(*KST::vectorList.findTag(koi->tag().tagString()));
         if (x) {
           x->deleteDependents();
           x = 0L;
           KST::vectorList.lock().writeLock();
-          KST::vectorList.removeTag(koi->tagName());
+          KST::vectorList.removeTag(koi->tag().tagString());
           KST::vectorList.lock().unlock();
           doUpdates();
         } else {
           KMessageBox::sorry(this, i18n("Unknown error deleting static vector."));
         }
       } else if (qi->rtti() == RTTI_OBJ_DATA_MATRIX) {
-        KstRMatrixPtr x = kst_cast<KstRMatrix>(*KST::matrixList.findTag(koi->tagName()));
+        KstRMatrixPtr x = kst_cast<KstRMatrix>(*KST::matrixList.findTag(koi->tag().tagString()));
         if (x) {
           x->deleteDependents();
           x = 0L;
           KST::matrixList.lock().writeLock();
-          KST::matrixList.removeTag(koi->tagName());
+          KST::matrixList.removeTag(koi->tag().tagString());
           KST::matrixList.lock().unlock();
           doUpdates();
         } else {
           KMessageBox::sorry(this, i18n("Unknown error deleting data matrix."));
         }
       } else if (qi->rtti() == RTTI_OBJ_STATIC_MATRIX) {
-        KstSMatrixPtr x = kst_cast<KstSMatrix>(*KST::matrixList.findTag(koi->tagName()));
+        KstSMatrixPtr x = kst_cast<KstSMatrix>(*KST::matrixList.findTag(koi->tag().tagString()));
         if (x) {
           x->deleteDependents();
           x = 0L;
           KST::matrixList.lock().writeLock();
-          KST::matrixList.removeTag(koi->tagName());
+          KST::matrixList.removeTag(koi->tag().tagString());
           KST::matrixList.lock().unlock();
           doUpdates();
         } else {
@@ -944,15 +1153,15 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
 
   if (koi->rtti() == RTTI_OBJ_DATA_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
-    id = m->insertItem(i18n("Make &Power Spectrum..."), koi, SLOT(makePSD()));
-    id = m->insertItem(i18n("Make Cumulative &Spectral Decay..."), koi, SLOT(makeCSD()));
+    id = m->insertItem(i18n("Make Spect&rum..."), koi, SLOT(makePSD()));
+    id = m->insertItem(i18n("Make &Spectrogram..."), koi, SLOT(makeCSD()));
     id = m->insertItem(i18n("Make &Histogram..."), koi, SLOT(makeHistogram()));
     id = m->insertItem(i18n("&Reload"), koi, SLOT(reload()));
     id = m->insertItem(i18n("Meta &Data"), koi, SLOT(showMetadata()));
   } else if (koi->rtti() == RTTI_OBJ_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
-    id = m->insertItem(i18n("Make &Power Spectrum..."), koi, SLOT(makePSD()));
-    id = m->insertItem(i18n("Make Cumulative &Spectral Decay..."), koi, SLOT(makeCSD()));
+    id = m->insertItem(i18n("Make Spect&rum..."), koi, SLOT(makePSD()));
+    id = m->insertItem(i18n("Make &Spectrogram..."), koi, SLOT(makeCSD()));
     id = m->insertItem(i18n("Make &Histogram..."), koi, SLOT(makeHistogram()));
   } else if (koi->rtti() == RTTI_OBJ_DATA_MATRIX) {
     id = m->insertItem(i18n("Make &Image..."), koi, SLOT(makeImage()));  
@@ -976,10 +1185,10 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
         for (Kst2DPlotList::Iterator i = plots.begin(); i != plots.end(); ++i) {
           Kst2DPlotPtr plot = *i;
           if (!plot->Curves.contains(c)) {
-            addMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(addToPlot(int)), 0, id);
+            addMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tag().tag()), koi, SLOT(addToPlot(int)), 0, id);
             haveAdd = true;
           } else {
-            removeMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(removeFromPlot(int)), 0, id);
+            removeMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tag().tag()), koi, SLOT(removeFromPlot(int)), 0, id);
             haveRemove = true;
           }
           PlotMap[id++] = plot;
@@ -1013,9 +1222,23 @@ void KstDataManagerI::doUpdates() {
 
 
 void KstDataManagerI::currentChanged(QListViewItem *i) {
-  if (i) {
+  if (i && !DataView->selectedItems().isEmpty()) {
     KstObjectItem *koi = static_cast<KstObjectItem*>(i);
     koi->updateButtons();
+  } else {
+    Edit->setEnabled(false);
+    Delete->setEnabled(false);
+  }
+}
+
+
+void KstDataManagerI::selectionChanged() {
+  if (!DataView->selectedItems().isEmpty()) {
+    KstObjectItem *koi = static_cast<KstObjectItem*>(DataView->selectedItems().first());
+    koi->updateButtons();
+  } else {
+    Edit->setEnabled(false);
+    Delete->setEnabled(false);
   }
 }
 
