@@ -15,6 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
+//#define PURGEDEBUG
+
 #include <sys/types.h>
 
 #ifdef HAVE_SYS_STAT_H
@@ -55,7 +57,8 @@
 #include "ksthistogram.h"
 #include "kstimage.h"
 #include "kstmatrixdefaults.h"
-#include "kstplugin.h"
+#include "kstobjectdefaults.h"
+#include "kstcplugin.h"
 #include "kstpsd.h"
 #include "kstrvector.h"
 #include "kstvcurve.h"
@@ -299,6 +302,25 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   app->slotUpdateProgress(count, handled, readingDocument);
 
   kapp->dcopClient()->setAcceptCalls(false);
+
+  // disable display tags for speed
+  {
+    KstWriteLocker ml(&KST::matrixList.lock());
+    KST::matrixList.setUpdateDisplayTags(false);
+  }
+  {
+    KstWriteLocker sl(&KST::scalarList.lock());
+    KST::scalarList.setUpdateDisplayTags(false);
+  }
+  {
+    KstWriteLocker sl(&KST::stringList.lock());
+    KST::stringList.setUpdateDisplayTags(false);
+  }
+  {
+    KstWriteLocker vl(&KST::vectorList.lock());
+    KST::vectorList.setUpdateDisplayTags(false);
+  }
+
   while (!n.isNull()) {
     QDomElement e = n.toElement(); // try to convert the node to an element.
     if (!e.isNull()) { // the node was really an element.
@@ -365,23 +387,31 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
           KST::dataSourceList.append(file);
         }
       } else if (e.tagName() == "scalar") {
+        KstWriteLocker sl(&KST::scalarList.lock());
         new KstScalar(e);
       } else if (e.tagName() == "string") {
+        KstWriteLocker sl(&KST::stringList.lock());
         new KstString(e);
       } else if (e.tagName() == "vector") {
+        KstWriteLocker vl(&KST::vectorList.lock());
         vector = new KstRVector(e, o_file, o_n, o_f, o_s, o_ave);
-        KST::addVectorToList(KstVectorPtr(vector));
         // Vectors are automatically added to the global list.
       } else if (e.tagName() == "svector") {
+        KstWriteLocker vl(&KST::vectorList.lock());
         KstSVectorPtr svector = new KstSVector(e);
-        KST::addVectorToList(KstVectorPtr(svector));
       } else if (e.tagName() == "avector") {
+        KstWriteLocker vl(&KST::vectorList.lock());
         KstAVectorPtr avector = new KstAVector(e);
-        avector = new KstAVector(e);
-        KST::addVectorToList(KstVectorPtr(avector));
       } else if (e.tagName() == "plugin") {
-        KstDataObjectPtr p = new KstPlugin(e);
-        if (p->isValid()) {
+        const QString name = e.attribute("name");
+        KstDataObjectPtr p;
+        if (name.isEmpty()) {
+          p = new KstCPlugin(e);
+        } else {
+          if (p = KstDataObject::createPlugin(name))
+            p->load(e);
+        }
+        if (p) {
           KstWriteLocker dowl(&KST::dataObjectList.lock());
           KST::dataObjectList.append(p);
         }
@@ -424,14 +454,14 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         Kst2DPlotPtr plot = new Kst2DPlot(e);
         app->plotHolderWhileOpeningDocument()->insert(plot->tagName(), plot);
       } else if (e.tagName() == "amatrix") {
+        KstWriteLocker ml(&KST::matrixList.lock());
         KstAMatrixPtr p = new KstAMatrix(e);
-        KST::addMatrixToList(KstMatrixPtr(p));
       } else if (e.tagName() == "smatrix") {
+        KstWriteLocker ml(&KST::matrixList.lock());
         KstSMatrixPtr p = new KstSMatrix(e);
-        KST::addMatrixToList(KstMatrixPtr(p));
       } else if (e.tagName() == "rmatrix") {
+        KstWriteLocker ml(&KST::matrixList.lock());
         KstRMatrixPtr p = new KstRMatrix(e);
-        KST::addMatrixToList(KstMatrixPtr(p));
       } else if (e.tagName() == "image") {
         KstDataObjectPtr p = new KstImage(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -466,9 +496,27 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     kapp->eventLoop()->processEvents(QEventLoop::ExcludeSocketNotifiers, 10);
   }
 
+  // reenable display tags
+  {
+    KstWriteLocker ml(&KST::matrixList.lock());
+    KST::matrixList.setUpdateDisplayTags(true);
+  }
+  {
+    KstWriteLocker sl(&KST::scalarList.lock());
+    KST::scalarList.setUpdateDisplayTags(true);
+  }
+  {
+    KstWriteLocker sl(&KST::stringList.lock());
+    KST::stringList.setUpdateDisplayTags(true);
+  }
+  {
+    KstWriteLocker vl(&KST::vectorList.lock());
+    KST::vectorList.setUpdateDisplayTags(true);
+  }
+
   if (warnOldKstFile) {
     QApplication::restoreOverrideCursor();
-    KMessageBox::sorry(KstApp::inst(), i18n("You tried to load an old Kst file.  Curves created by equations or PSDs will not be loaded."));
+    KMessageBox::sorry(KstApp::inst(), i18n("You tried to load an old Kst file.  Curves created by equations or spectra will not be loaded."));
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
   }
 
@@ -669,6 +717,7 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   // save vectors
   for (KstVectorList::Iterator it = KST::vectorList.begin(); it != KST::vectorList.end(); ++it) {
     if ((*it)->saveable()) {
+      (*it)->setSaveData(KstApp::inst()->saveData());
       (*it)->save(ts, "  ", saveAbsoluteVectorPositions);
     }
   }
@@ -780,6 +829,7 @@ bool KstDoc::saveDocument(const QString& filename, bool saveAbsoluteVectorPositi
 void KstDoc::deleteContents() {
   KST::vectorDefaults.sync();
   KST::matrixDefaults.sync();
+  KST::objectDefaults.sync();
 
   KstApp *app = KstApp::inst();
   if (app) { // Can be null on application exit
@@ -937,6 +987,10 @@ RemoveStatus KstDoc::removeDataObject(const QString& tag) {
 
 
 void KstDoc::purge() {
+#ifdef PURGEDEBUG
+  kstdDebug() << "Purging unused objects" << endl;
+#endif
+
   QString purging = i18n("Purging unused objects");
   bool modified = false;
   bool again = true;
@@ -958,9 +1012,13 @@ void KstDoc::purge() {
     // ASSUMPTION: this only gets called from the data manager!
     KST::dataObjectList.lock().writeLock();
     for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
-      //kstdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+#ifdef PURGEDEBUG
+      kstdDebug() << "OBJECT: " << (*it)->tag().displayString() << " (" << (void*)(*it) << ") USAGE: " << (*it)->getUsage() << endl;
+#endif
       if ((*it)->getUsage() == 0 && !kst_cast<EventMonitorEntry>(*it)) {
-        //kstdDebug() << "    -> REMOVED" << endl;
+#ifdef PURGEDEBUG
+        kstdDebug() << "    -> REMOVED" << endl;
+#endif
         KstDataObjectList::Iterator byebye = it;
         --it;
         KST::dataObjectList.remove(byebye);
@@ -973,14 +1031,26 @@ void KstDoc::purge() {
     KST::dataObjectList.lock().unlock();
   
     KST::vectorList.lock().readLock();
-    KstVectorList vectorList = QDeepCopy<KstVectorList>(KST::vectorList);
+    KstVectorList vectorList = QDeepCopy<KstVectorList>(KST::vectorList.list());
     KST::vectorList.lock().unlock();
     
     // clear unused vectors that are editable 
     for (KstVectorList::ConstIterator it = vectorList.begin(); it != vectorList.end(); ++it) {
-      //kstdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+#ifdef PURGEDEBUG
+      kstdDebug() << "VECTOR: " << (*it)->tag().displayString() << " (" << (void*)(*it) << ") USAGE: " << (*it)->getUsage() << endl;
+//      if ((*it)->provider()) {
+//        kstdDebug() << "  provider=" << (*it)->provider()->tag().displayString() << endl;
+//      }
+//      KstRVectorPtr rvp = kst_cast<KstRVector>(*it);
+//      if (rvp && rvp->_file) {
+//        KstDataSource *file = rvp->_file;
+//        kstdDebug() << "  file=" << file->tag().displayString() << " (" << (void*)(&(*file)) << "): " << file->getUsage() << endl;
+//      }
+#endif
       if ((*it)->getUsage() == 1) {
-        //kstdDebug() << "    -> REMOVED" << endl;
+#ifdef PURGEDEBUG
+        kstdDebug() << "    -> REMOVED" << endl;
+#endif
         KST::vectorList.lock().writeLock();
         KST::vectorList.remove(const_cast<KstVector*>((*it).data()));
         KST::vectorList.lock().unlock();
@@ -992,13 +1062,26 @@ void KstDoc::purge() {
     }
     
     KST::matrixList.lock().readLock();
-    KstMatrixList matrixList = QDeepCopy<KstMatrixList>(KST::matrixList);
+    KstMatrixList matrixList = QDeepCopy<KstMatrixList>(KST::matrixList.list());
     KST::matrixList.lock().unlock();
 
     // clear unused matrices that are editable
     for (KstMatrixList::ConstIterator it = matrixList.begin(); it != matrixList.end(); ++it) {
+#ifdef PURGEDEBUG
+      kstdDebug() << "MATRIX: " << (*it)->tag().displayString() << " (" << (void*)(*it) << ") USAGE: " << (*it)->getUsage() << endl;
+//      if ((*it)->provider()) {
+//        kstdDebug() << "  provider=" << (*it)->provider()->tag().displayString() << endl;
+//      }
+//      KstRMatrixPtr rmp = kst_cast<KstRMatrix>(*it);
+//      if (rmp && rmp->_file) {
+//        KstDataSource *file = rmp->_file;
+//        kstdDebug() << "  file=" << file->tag().displayString() << " (" << (void*)(&(*file)) << "): " << file->getUsage() << endl;
+//      }
+#endif
       if ((*it)->getUsage() == 1) {
-        //kstdDebug() << "    -> REMOVED" << endl;
+#ifdef PURGEDEBUG
+        kstdDebug() << "    -> REMOVED" << endl;
+#endif
         KST::matrixList.lock().writeLock();
         KST::matrixList.remove(const_cast<KstMatrix*>((*it).data()));
         KST::matrixList.lock().unlock();
@@ -1013,8 +1096,14 @@ void KstDoc::purge() {
   KstDataSourceList dataList;
   KST::dataSourceList.lock().readLock();
   for (KstDataSourceList::ConstIterator it = KST::dataSourceList.begin(); it != KST::dataSourceList.end(); ++it) {
-      if ((*it)->getUsage() == 1) {
-        //kstdDebug() << "    -> REMOVED" << endl;
+      KstDataSourcePtr ds = *it; // MUST use a reference-counted pointer to call getUsage()
+#ifdef PURGEDEBUG
+      kstdDebug() << "DATA SOURCE: " << ds->tag().displayString() << " (" << (void*)ds << ") USAGE: " << ds->getUsage() << endl;
+#endif
+      if (ds->getUsage() == 1) {
+#ifdef PURGEDEBUG
+        kstdDebug() << "    -> REMOVED" << endl;
+#endif
         dataList.append(const_cast<KstDataSource*>((*it).data()));
         modified = true;
       }
@@ -1180,10 +1269,10 @@ void KstDoc::createScalars() const {
   new KstScalar("CONST_MKSA_MOLAR_GAS", 8.314472e0, false);
   new KstScalar("CONST_MKSA_STANDARD_GAS_VOLUME", 2.2710981e-2, false);
 #endif
-  new KstScalar("CONST_PI", 0L, M_PI, false);
-  new KstScalar("C_PI", 0L, M_PI, false);
-  new KstScalar("C_R2D", 0L, 180.0/M_PI, false); // radians to degrees
-  new KstScalar("C_D2R", 0L, M_PI/180.0, false); // degrees to radians
+  new KstScalar(KstObjectTag("CONST_PI", KstObjectTag::constantTagContext), 0L, M_PI, false);
+  new KstScalar(KstObjectTag("C_PI", KstObjectTag::constantTagContext), 0L, M_PI, false);
+  new KstScalar(KstObjectTag("C_R2D", KstObjectTag::constantTagContext), 0L, 180.0/M_PI, false); // radians to degrees
+  new KstScalar(KstObjectTag("C_D2R", KstObjectTag::constantTagContext), 0L, M_PI/180.0, false); // degrees to radians
 }
 
 
