@@ -30,6 +30,7 @@
 #include <kcolorbutton.h>
 #include <kcombobox.h>
 #include <kmessagebox.h>
+#include <knuminput.h>
 
 // application specific includes
 #include "colorpalettewidget.h"
@@ -60,6 +61,7 @@ KstImageDialogI::KstImageDialogI(QWidget* parent,
   setMultiple(true);
   connect(_w->_matrix, SIGNAL(newMatrixCreated(const QString&)), this, SIGNAL(modified()));
   connect(_w->_autoThreshold, SIGNAL(clicked()), this, SLOT(calcAutoThreshold()));
+  connect(_w->_smartThreshold, SIGNAL(clicked()), this, SLOT(calcSmartThreshold()));
   connect(_w->_colorOnly, SIGNAL(clicked()), this, SLOT(updateGroups()));
   connect(_w->_contourOnly, SIGNAL(clicked()), this, SLOT(updateGroups()));
   connect(_w->_colorAndContour, SIGNAL(clicked()), this, SLOT(updateGroups()));
@@ -215,7 +217,7 @@ bool KstImageDialogI::newObject() {
   matrix->readLock();
 
   //create a unique name
-  QString tag_name = KST::suggestImageName(matrix->tagName());
+  QString tag_name = KST::suggestImageName(matrix->tag());
   if (KstData::self()->dataTagNameNotUnique(tag_name)) {
     _tagName->setFocus();
     matrix->unlock();
@@ -279,7 +281,8 @@ bool KstImageDialogI::editSingleObject(KstImagePtr imPtr) {
   if (_contourOnlyDirty || _colorOnlyDirty || _colorAndContourDirty) {
     double lowerZDouble, upperZDouble;
     if (!checkParameters(lowerZDouble, upperZDouble)) {
-      KMessageBox::sorry(this, i18n("Image type was changed: Lower Z threshold cannot be higher than Upper Z threshold."));
+      //KMessageBox::sorry(this, i18n("Image type was changed: Lower Z threshold cannot be higher than Upper Z threshold."));
+      //pMatrix->unlock();
       imPtr->unlock();
       return false;
     }
@@ -354,7 +357,8 @@ bool KstImageDialogI::editSingleObject(KstImagePtr imPtr) {
     // check parameters for color map
     if (imPtr->hasColorMap()) {
       if (pLowerZ > pUpperZ) {
-        KMessageBox::sorry(this, i18n("The Lower Z threshold cannot be higher than Upper Z threshold."));
+        //KMessageBox::sorry(this, i18n("The Lower Z threshold cannot be higher than Upper Z threshold."));
+        //pMatrix->unlock();
         imPtr->unlock();
         return false;
       }
@@ -436,7 +440,7 @@ bool KstImageDialogI::editObject() {
     }
 
     ip->writeLock();
-    ip->setTagName(tag_name);
+    ip->setTagName(KstObjectTag(tag_name, ip->tag().context())); // FIXME: doesn't allow changing tag context
     ip->unlock();
 
     // then edit the object
@@ -475,6 +479,25 @@ void KstImageDialogI::calcAutoThreshold() {
   }
 }
 
+// This should use a smart (percentile based) algorithm to
+// calculate the thresholds.  It will be expensive.
+void KstImageDialogI::calcSmartThreshold() {
+  //make sure an matrix is selected
+  if (!_w->_matrix->selectedMatrix().isEmpty()){
+    KST::matrixList.lock().readLock();
+    KstMatrixPtr matrix = *KST::matrixList.findTag(_w->_matrix->selectedMatrix());
+    KST::matrixList.lock().unlock();
+    if (matrix) {
+      matrix->readLock();
+      double per = _w->_smartThresholdValue->value()/100.0;
+
+      matrix->calcNoSpikeRange(per);
+      _w->_lowerZ->setText(QString::number(matrix->minValueNoSpike()));
+      _w->_upperZ->setText(QString::number(matrix->maxValueNoSpike()));
+      matrix->unlock();
+    }
+  }
+}
 
 void KstImageDialogI::placeInPlot(KstImagePtr image) {
   KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_w->_curvePlacement->_plotWindow->currentText()));
@@ -494,7 +517,7 @@ void KstImageDialogI::placeInPlot(KstImagePtr image) {
 
     if (_w->_curvePlacement->newPlot()) {
       /* assign image to plot */
-      QString name = w->createObject<Kst2DPlot>(KST::suggestPlotName());
+      QString name = w->createPlot(KST::suggestPlotName());
       if (_w->_curvePlacement->reGrid()) {
         w->view()->cleanup(_w->_curvePlacement->columns());
       }
@@ -535,6 +558,8 @@ void KstImageDialogI::updateEnables() {
     _w->_lowerZ->setEnabled(!_w->_realTimeAutoThreshold->isChecked());
     _w->_upperZ->setEnabled(!_w->_realTimeAutoThreshold->isChecked());
     _w->_autoThreshold->setEnabled(!_w->_realTimeAutoThreshold->isChecked() && !_editMultipleMode);
+    _w->_smartThreshold->setEnabled(!_w->_realTimeAutoThreshold->isChecked() && !_editMultipleMode);
+    _w->_smartThresholdValue->setEnabled(!_w->_realTimeAutoThreshold->isChecked() && !_editMultipleMode);
   }
 }
 
@@ -545,11 +570,19 @@ bool KstImageDialogI::checkParameters(double& lowerZDouble, double& upperZDouble
     lowerZDouble = _w->_lowerZ->text().toDouble(&ok1);
     upperZDouble = _w->_upperZ->text().toDouble(&ok2);
     if (!(ok1 && ok2)) {
-      KMessageBox::sorry(this, i18n("The upper and lower thresholds are not valid.  Enter valid decimal numbers for the thresholds."));
+      if (ok1 || ok2) {
+        if (ok1) {
+          KMessageBox::sorry(this, i18n("The upper threshold is not a valid decimal number."));
+        } else {
+          KMessageBox::sorry(this, i18n("The lower threshold is not a valid decimal number."));
+        }
+      } else {
+        KMessageBox::sorry(this, i18n("The upper and lower thresholds are not valid decimal numbers."));
+      }
       return false;
     }
     if (lowerZDouble >= upperZDouble) {
-      KMessageBox::sorry(this, i18n("The lower threshold cannot be greater than the upper threshold."));
+      KMessageBox::sorry(this, i18n("The upper threshold must be greater than the lower threshold."));
       return false;
     }
   }
