@@ -329,11 +329,10 @@ bool KstMatrixDialogI::new_IRMatrix() {
   bool doAve = _w->_doAve->isChecked();
   int skip = _w->_skip->value();
 
-  KstRMatrixPtr matrix = new KstRMatrix(file, pField, tag_name,
+  KstRMatrixPtr matrix = new KstRMatrix(file, pField,
+                                        KstObjectTag(tag_name, file->tag(), false),
                                         xStart, yStart, xNumSteps, yNumSteps,
                                         doAve, doSkip, skip);
-
-  KST::addMatrixToList(KstMatrixPtr(matrix));
   emit matrixCreated(KstMatrixPtr(matrix));
   matrix = 0L; // drop the reference
   emit modified();
@@ -375,9 +374,8 @@ bool KstMatrixDialogI::new_ISMatrix() {
     return false;
   }
 
-  KstSMatrixPtr matrix = new KstSMatrix(tag_name, nX, nY, minX, minY, xStep, yStep, zMin, zMax, xDirection);
-
-  KST::addMatrixToList(KstMatrixPtr(matrix));
+  KstSMatrixPtr matrix = new KstSMatrix(KstObjectTag(tag_name, KstObjectTag::globalTagContext),
+      nX, nY, minX, minY, xStep, yStep, zMin, zMax, xDirection);
 
   emit matrixCreated(KstMatrixPtr(matrix));
   matrix = 0L; // drop the reference
@@ -477,7 +475,7 @@ bool KstMatrixDialogI::editSingleRMatrix(KstRMatrixPtr rmp) {
   rmp->unlock();
 
   rmp->writeLock();
-  rmp->change(file, pField, rmp->tagName(), xStart, yStart, xNumSteps, yNumSteps, doAve, doSkip, skip);
+  rmp->change(file, pField, KstObjectTag(rmp->tag().tag(), rmp->tag().context()), xStart, yStart, xNumSteps, yNumSteps, doAve, doSkip, skip);
   rmp->unlock();
   return true;
 }
@@ -551,7 +549,7 @@ bool KstMatrixDialogI::editSingleSMatrix(KstSMatrixPtr smp) {
   }
 
   smp->writeLock();
-  smp->change(smp->tagName(), nX, nY, xMin, yMin, xStepSize, yStepSize, gradientZAtMin, gradientZAtMax, xDirection);
+  smp->change(KstObjectTag(smp->tag().tag(), smp->tag().context()), nX, nY, xMin, yMin, xStepSize, yStepSize, gradientZAtMin, gradientZAtMax, xDirection);
   smp->unlock();
 
   return true;
@@ -626,7 +624,7 @@ bool KstMatrixDialogI::editObject() {
     }
 
     mp->writeLock();
-    mp->setTagName(tag_name);
+    mp->setTagName(KstObjectTag(tag_name, mp->tag().context())); // FIXME: can't change tag context
     mp->unlock();
 
     // then edit the object
