@@ -69,6 +69,7 @@ KstPluginDialogI::KstPluginDialogI(QWidget* parent, const char* name, bool modal
 
   _pluginInfoGrid = 0L;
   _pluginInputOutputGrid = 0L;
+  _pluginName = QString::null;
 }
 
 
@@ -150,7 +151,7 @@ void KstPluginDialogI::updateForm() {
 
 
 void KstPluginDialogI::fillFieldsForEdit() {
-  KstPluginPtr pp = kst_cast<KstPlugin>(_dp);
+  KstCPluginPtr pp = kst_cast<KstCPlugin>(_dp);
   if (!pp) {
     return;
   }
@@ -181,7 +182,8 @@ void KstPluginDialogI::fillFieldsForEdit() {
 
 void KstPluginDialogI::fillFieldsForNew() {
   updatePluginList();
-  _w->PluginCombo->setCurrentItem(0);
+  int i = _pluginList.findIndex(_pluginName);
+  _w->PluginCombo->setCurrentItem(i);
   pluginChanged(_w->PluginCombo->currentItem());
   _tagName->setText(plugin_defaultTag);
 }
@@ -189,7 +191,7 @@ void KstPluginDialogI::fillFieldsForNew() {
 
 void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
   bool DPvalid = false;
-  KstPluginPtr pp = kst_cast<KstPlugin>(_dp);
+  KstCPluginPtr pp = kst_cast<KstCPlugin>(_dp);
 
   if (pp) {
     pp->readLock();
@@ -213,7 +215,7 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
           QString selectedVector = vs->selectedVector();
           vs->update();
           if (DPvalid) {
-            vs->setSelection(pp->inputVectors()[(*it)._name]->tagName());
+            vs->setSelection(pp->inputVectors()[(*it)._name]->tag().displayString());
           } else {
             vs->setSelection(selectedVector);
           }
@@ -226,7 +228,7 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
           QString selectedString = ss->selectedString();
           ss->update();
           if (DPvalid) {
-            ss->setSelection(pp->inputStrings()[(*it)._name]->tagName());
+            ss->setSelection(pp->inputStrings()[(*it)._name]->tag().displayString());
           } else {
             ss->setSelection(selectedString);
           }
@@ -241,7 +243,7 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
           QString selectedScalar = ss->selectedScalar();
           ss->update();
           if (DPvalid) {
-            ss->setSelection(pp->inputScalars()[(*it)._name]->tagName());
+            ss->setSelection(pp->inputScalars()[(*it)._name]->tag().displayString());
           } else {
             ss->setSelection(selectedScalar);
           }
@@ -381,24 +383,18 @@ void KstPluginDialogI::restoreInputs(const QValueList<Plugin::Data::IOValue>& ta
 }
 
 
-bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
+bool KstPluginDialogI::saveInputs(KstCPluginPtr plugin, KstSharedPtr<Plugin> p) {
   bool rc = true;
 
-  KST::vectorList.lock().readLock();
-  KST::scalarList.lock().readLock();
-  KST::stringList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
       QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
       assert(field);
       VectorSelector *vs = static_cast<VectorSelector*>(field);
+      KstReadLocker vl(&KST::vectorList.lock());
       KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
       if (v) {
-        v->writeLock(); // to match with plugin->writeLock()
-        if (plugin->inputVectors().contains((*it)._name) && plugin->inputVectors()[(*it)._name] != v) {
-          plugin->inputVectors()[(*it)._name]->unlock();
-        }
         plugin->inputVectors().insert((*it)._name, v);
       } else if (plugin->inputVectors().contains((*it)._name)) {
         plugin->inputVectors().erase((*it)._name);
@@ -408,20 +404,16 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
       QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
       assert(field);
       StringSelector *ss = static_cast<StringSelector*>(field);
+      KstWriteLocker sl(&KST::stringList.lock());
       KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
       if (s == *KST::stringList.end()) {
         QString val = ss->_string->currentText();
-        KstStringPtr newString = new KstString(ss->_string->currentText(), 0L, val, true, false);
-        newString->writeLock(); // to match with plugin->writeLock()
-        if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != s) {
-          plugin->inputStrings()[(*it)._name]->unlock();
+        KstStringPtr newString = new KstString(KstObjectTag::fromString(ss->_string->currentText()), 0L, val, true);
+        if (!newString) {
+          rc = false;
         }
         plugin->inputStrings().insert((*it)._name, newString);
       } else {
-        s->writeLock(); // to match with plugin->writeLock()
-        if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != s) {
-          plugin->inputStrings()[(*it)._name]->unlock();
-        }
         plugin->inputStrings().insert((*it)._name, s);
       }
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
@@ -430,44 +422,30 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
       QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
       assert(field);
       ScalarSelector *ss = static_cast<ScalarSelector*>(field);
+      KstWriteLocker sl(&KST::scalarList.lock());
       KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
       if (s == *KST::scalarList.end()) {
         bool ok;
         double val = ss->_scalar->currentText().toDouble(&ok);
 
         if (ok) {
-          KstScalarPtr newScalar = new KstScalar(ss->_scalar->currentText(), 0L, val, true, false, false);
-          newScalar->writeLock(); // to match with plugin->writeLock()
-          if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
-            plugin->inputScalars()[(*it)._name]->unlock();
-          }
+          KstScalarPtr newScalar = new KstScalar(KstObjectTag::fromString(ss->_scalar->currentText()), 0L, val, true, false);
           plugin->inputScalars().insert((*it)._name, newScalar);
         } else {
-          s->writeLock(); // to match with plugin->writeLock()
-          if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
-            plugin->inputScalars()[(*it)._name]->unlock();
-          }
-          plugin->inputScalars().insert((*it)._name, s);
+          rc = false;
         }
       } else {
-        s->writeLock(); // to match with plugin->writeLock()
-        if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
-          plugin->inputScalars()[(*it)._name]->unlock();
-        }
         plugin->inputScalars().insert((*it)._name, s);
       }
     } else {
     }
   }
-  KST::stringList.lock().unlock();
-  KST::scalarList.lock().unlock();
-  KST::vectorList.lock().unlock();
 
   return rc;
 }
 
 
-bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
+bool KstPluginDialogI::saveOutputs(KstCPluginPtr plugin, KstSharedPtr<Plugin> p) {
   const QValueList<Plugin::Data::IOValue>& otable = p->data()._outputs;
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
@@ -479,11 +457,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
     QLineEdit *li = static_cast<QLineEdit*>(field);
 
     if (li->text().isEmpty()) {
-      QString tagName = _tagName->text();
-      if (tagName==plugin_defaultTag) {
-        tagName = plugin->tagName();
-      }
-      li->setText(tagName + "-" + (*it)._name);
+      li->setText((*it)._name);
     }
 
     QString nt = li->text();
@@ -492,12 +466,11 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         // Implicitly creates it if it doesn't exist
         KstVectorPtr v = plugin->outputVectors()[(*it)._name];
         if (!v) {
-          v = new KstVector(nt, 0, plugin.data());
-          v->writeLock();
+          KstWriteLocker blockVectorUpdates(&KST::vectorList.lock());
+          v = new KstVector(KstObjectTag(nt, plugin->tag()), 0, plugin.data());
           plugin->outputVectors().insert((*it)._name, v);
-          KST::addVectorToList(v);
         }
-        v->setTagName(nt);
+        v->setTagName(KstObjectTag(nt, plugin->tag()));
       } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
         while (KstData::self()->vectorTagNameNotUnique(nt, false)) {
           nt += "'";
@@ -506,12 +479,11 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         if (plugin->outputVectors().contains((*it)._name)) {
           v = plugin->outputVectors()[(*it)._name];
         } else {
-          v = new KstVector(nt, 0, plugin.data());
-          v->writeLock();
+          KstWriteLocker blockVectorUpdates(&KST::vectorList.lock());
+          v = new KstVector(KstObjectTag(nt, plugin->tag()), 0, plugin.data());
           plugin->outputVectors().insert((*it)._name, v);
-          KST::addVectorToList(v);
         }
-        v->setTagName(nt);
+        v->setTagName(KstObjectTag(nt, plugin->tag()));
       }
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
       if (!KstData::self()->vectorTagNameNotUnique(nt, false)) {
@@ -519,11 +491,11 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         if (plugin->outputStrings().contains((*it)._name)) {
           s = plugin->outputStrings()[(*it)._name];
         } else {
-          s = new KstString(nt, plugin.data());
-          s->writeLock();
+          KstWriteLocker blockStringUpdates(&KST::stringList.lock());
+          s = new KstString(KstObjectTag(nt, plugin->tag()), plugin.data());
           plugin->outputStrings().insert((*it)._name, s);
         }
-        s->setTagName(nt);
+        s->setTagName(KstObjectTag(nt, plugin->tag()));
 
       } else if (plugin->outputStrings()[(*it)._name]->tagName() != nt) {
         while (KstData::self()->vectorTagNameNotUnique(nt, false)) {
@@ -533,11 +505,11 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         if (plugin->outputStrings().contains((*it)._name)) {
           s = plugin->outputStrings()[(*it)._name];
         } else {
-          s = new KstString(nt, plugin.data());
-          s->writeLock();
+          KstWriteLocker blockStringUpdates(&KST::stringList.lock());
+          s = new KstString(KstObjectTag(nt, plugin->tag()), plugin.data());
           plugin->outputStrings().insert((*it)._name, s);
         }
-        s->setTagName(nt);
+        s->setTagName(KstObjectTag(nt, plugin->tag()));
       }
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       // Nothing
@@ -547,11 +519,11 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         if (plugin->outputScalars().contains((*it)._name)) {
           s = plugin->outputScalars()[(*it)._name];
         } else {
-          s = new KstScalar(nt, plugin.data());
-          s->writeLock();
+          KstWriteLocker blockScalarUpdates(&KST::scalarList.lock());
+          s = new KstScalar(KstObjectTag(nt, plugin->tag()), plugin.data());
           plugin->outputScalars().insert((*it)._name, s);
         }
-        s->setTagName(nt);
+        s->setTagName(KstObjectTag(nt, plugin->tag()));
       } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
         while (KstData::self()->vectorTagNameNotUnique(nt, false)) {
           nt += "'";
@@ -560,11 +532,11 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         if (plugin->outputScalars().contains((*it)._name)) {
           s = plugin->outputScalars()[(*it)._name];
         } else {
-          s = new KstScalar(nt, plugin.data());
-          s->writeLock();
+          KstWriteLocker blockScalarUpdates(&KST::scalarList.lock());
+          s = new KstScalar(KstObjectTag(nt, plugin->tag()), plugin.data());
           plugin->outputScalars().insert((*it)._name, s);
         }
-        s->setTagName(nt);
+        s->setTagName(KstObjectTag(nt, plugin->tag()));
       }
     }
   }
@@ -580,33 +552,33 @@ bool KstPluginDialogI::newObject() {
     _tagName->setFocus();
     return false;
   }
-  KstPluginPtr plugin;
+  KstCPluginPtr plugin;
   int pitem = _w->PluginCombo->currentItem();
   if (pitem >= 0 && _w->PluginCombo->count() > 0) {
     KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
     if (pPtr) {
-      plugin = new KstPlugin;
-      plugin->writeLock();
+      plugin = new KstCPlugin;
+      KstWriteLocker pl(plugin);
+
+      // set the tag name before any dependents are created
+      if (tagName == plugin_defaultTag) {
+        tagName = KST::suggestPluginName(_pluginList[pitem]);
+      }
+      plugin->setTagName(KstObjectTag(tagName, KstObjectTag::globalTagContext));
+
       if (!saveInputs(plugin, pPtr)) {
-        KMessageBox::sorry(this, i18n("One or more of the inputs was undefined."));
-        plugin->unlock();
+        KMessageBox::sorry(this, i18n("There is an error in the inputs you entered."));
         plugin = 0L;
         return false;
       }
 
       plugin->setPlugin(pPtr);
 
-      if (tagName == plugin_defaultTag) {
-        tagName = KST::suggestPluginName(_pluginList[pitem]);
-      }
-      plugin->setTagName(tagName);
       if (!saveOutputs(plugin, pPtr)) {
-        KMessageBox::sorry(this, i18n("One or more of the outputs was undefined."));
-        plugin->unlock();
+        KMessageBox::sorry(this, i18n("There is an error in the outputs you entered."));
         plugin = 0L;
         return false;
       }
-      plugin->unlock();
     }
   }
 
@@ -627,41 +599,30 @@ bool KstPluginDialogI::newObject() {
 
 
 bool KstPluginDialogI::editObject() {
-  KstPluginPtr pp = kst_cast<KstPlugin>(_dp);
+  KstCPluginPtr pp = kst_cast<KstCPlugin>(_dp);
   if (!pp) { // something is dreadfully wrong - this should never happen
     return false;
   }
 
-  pp->writeLock();
+  KstWriteLocker pl(pp);
+
   if (_tagName->text() != pp->tagName() && KstData::self()->dataTagNameNotUnique(_tagName->text())) {
     _tagName->setFocus();
-    pp->unlock();
     return false;
   }
 
-  pp->setTagName(_tagName->text());
+  pp->setTagName(KstObjectTag(_tagName->text(), KstObjectTag::globalTagContext));  // FIXME: tag context always global?
 
   int pitem = _w->PluginCombo->currentItem();
   KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
 
-  // Must unlock before clear()
-  for (KstVectorMap::Iterator i = pp->inputVectors().begin(); i != pp->inputVectors().end(); ++i) {
-    (*i)->unlock();
-  }
-  for (KstScalarMap::Iterator i = pp->inputScalars().begin(); i != pp->inputScalars().end(); ++i) {
-    (*i)->unlock();
-  }
-  for (KstStringMap::Iterator i = pp->inputStrings().begin(); i != pp->inputStrings().end(); ++i) {
-    (*i)->unlock();
-  }
   pp->inputVectors().clear();
   pp->inputScalars().clear();
   pp->inputStrings().clear();
 
   // Save the vectors and scalars
   if (!saveInputs(pp, pPtr)) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
-    pp->unlock();
+    KMessageBox::sorry(this, i18n("There is an error in the inputs you entered."));
     return false;
   }
 
@@ -670,24 +631,30 @@ bool KstPluginDialogI::editObject() {
   }
 
   if (!saveOutputs(pp, pPtr)) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
-    pp->unlock();
+    KMessageBox::sorry(this, i18n("There is an error in the outputs you entered."));
     return false;
   }
 
   if (!pp->isValid()) {
     KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
-    pp->unlock();
     return false;
   }
   pp->setDirty();
-  pp->unlock();
 
   emit modified();
   return true;
 }
 
 
+void KstPluginDialogI::showNew(const QString &field) {
+  //Call init every time on showNew, because the field might equal propertyString()...
+  _pluginName = field;
+  _newDialog = true;
+  init();
+  KstDataDialog::showNew(field);
+}
+
+
 void KstPluginDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
   QString scalarLabelTemplate, vectorLabelTemplate, stringLabelTemplate;
 
