@@ -46,13 +46,14 @@
 #include "kstviewobjectfactory.h"
 #include "kstviewwidget.h"
 #include "kstgfxmousehandlerutils.h"
+#include "kstdatacollection.h"
 
 #define STICKY_THRESHOLD 10
 
 KstTopLevelView::KstTopLevelView(QWidget *parent, const char *name, WFlags w)
 : KstViewObject("TopLevelView"), _w(new KstViewWidget(this, parent, name, w)) {
   _onGrid = true;
-  setTagName(name);
+  setTagName(KstObjectTag(name, KstObjectTag::globalTagContext));  // FIXME: tag context
   commonConstructor(); 
 }
 
@@ -1721,7 +1722,7 @@ void KstTopLevelView::saveDefaults(KstViewObjectPtr object) {
 bool KstTopLevelView::handleDoubleClick(const QPoint& pos, bool shift) {
   handlePress(pos, shift);
   if (_pressTarget) {
-    _pressTarget->showDialog(this);  
+    _pressTarget->showDialog(this, false);  
   }  
   return true;
 }
