@@ -25,7 +25,6 @@
 #include <qspinbox.h>
 #include <qvbox.h>
 
-#include <kcombobox.h>
 #include <kdialogbase.h>
 #include <klineedit.h>
 #include <klocale.h>
@@ -44,6 +43,7 @@
 #include "kstvectordefaults.h"
 #include "kstvectordialog_i.h"
 #include "kstdefaultnames.h"
+#include "kstcombobox.h"
 #include "vectordialogwidget.h"
 
 
@@ -383,15 +383,14 @@ bool KstVectorDialogI::newObject() {
 
     /* create the vector */
     KstRVectorPtr vector = new KstRVector(
-        file, _w->Field->currentText(), tag_name,
+        file, _w->Field->currentText(),
+        KstObjectTag(tag_name, file->tag(), false),
         _w->_kstDataRange->CountFromEnd->isChecked() ? -1 : f0,
         _w->_kstDataRange->ReadToEnd->isChecked() ? -1 : n,
         _w->_kstDataRange->Skip->value(),
         _w->_kstDataRange->DoSkip->isChecked(),
         _w->_kstDataRange->DoFilter->isChecked());
 
-    KST::addVectorToList(KstVectorPtr(vector));
-
     emit vectorCreated(KstVectorPtr(vector));
     vector = 0L;
     emit modified();
@@ -404,8 +403,7 @@ bool KstVectorDialogI::newObject() {
       tagname = KST::suggestVectorName(QString("(%1..%2)").arg(x0).arg(x1));
     }
 
-    KstSVectorPtr svector = new KstSVector(x0, x1, n, tagname);
-    KST::addVectorToList(KstVectorPtr(svector));
+    KstSVectorPtr svector = new KstSVector(x0, x1, n, KstObjectTag(tagname, KstObjectTag::globalTagContext));
     emit vectorCreated(KstVectorPtr(svector));
     svector = 0L;
     emit modified();
@@ -572,13 +570,12 @@ bool KstVectorDialogI::editSingleObjectRV(KstVectorPtr vcPtr) {
       pDoFilter = rvp->doAve();
     }
 
-    QString pTagName = rvp->tagName();
     rvp->unlock();
 
     /* change the vector */
     rvp->writeLock();
 
-    rvp->change(file, pField, pTagName, pCountFromEnd ?  -1 : f0, pReadToEnd ?  -1 : n, pSkip, pDoSkip, pDoFilter);
+    rvp->change(file, pField, rvp->tag(), pCountFromEnd ?  -1 : f0, pReadToEnd ?  -1 : n, pSkip, pDoSkip, pDoFilter);
 
     rvp->unlock();
   } else {
@@ -590,10 +587,10 @@ bool KstVectorDialogI::editSingleObjectRV(KstVectorPtr vcPtr) {
     double x0 = _w->_xMin->text().toDouble();
     double x1 = _w->_xMax->text().toDouble();
     int n = _w->_N->value();
-    QString tagname = _tagName->text();
+//    QString tagname = _tagName->text();
     svp->writeLock();
     svp->changeRange(x0, x1, n);
-    svp->setTagName(_tagName->text()); // FIXME: doesn't verify uniqueness
+    svp->setTagName(KstObjectTag(_tagName->text(), svp->tag().context())); // FIXME: doesn't verify uniqueness, doesn't allow changing tag context
     svp->unlock();
   }
   file->unlock();
@@ -602,8 +599,6 @@ bool KstVectorDialogI::editSingleObjectRV(KstVectorPtr vcPtr) {
 
 
 bool KstVectorDialogI::editObject() {
-  KstVectorList vcList = KST::vectorList;
-
   // if editing multiple objects, edit each one
   if (_editMultipleMode) {
     // get dirties first
@@ -620,11 +615,14 @@ bool KstVectorDialogI::editObject() {
     for (uint i = 0; i < _editMultipleWidget->_objectList->count(); i++) {
       if (_editMultipleWidget->_objectList->isSelected(i)) {
         // get the pointer to the object
-        KstVectorList::Iterator vcIter = vcList.findTag(_editMultipleWidget->_objectList->text(i));
-        if (vcIter == vcList.end()) {
+        KST::vectorList.lock().readLock();
+        KstVectorList::Iterator vcIter = KST::vectorList.findTag(_editMultipleWidget->_objectList->text(i));
+        if (vcIter == KST::vectorList.end()) {
+          KST::vectorList.lock().unlock();
           return false;
         }
         KstVectorPtr vcPtr = *vcIter;
+        KST::vectorList.lock().unlock();
 
         if (!editSingleObject(vcPtr)) {
           return false;
@@ -646,7 +644,7 @@ bool KstVectorDialogI::editObject() {
       return false;
     }
 
-    _dp->setTagName(tag_name);
+    _dp->setTagName(KstObjectTag(tag_name, _dp->tag().context())); // FIXME: doesn't allow changing tag context
     _dp->unlock();
 
     // then edit the object
