@@ -23,7 +23,7 @@
 #include "kstdataobjectcollection.h"
 #include "kstgfxtextmousehandler.h"
 #include "kst.h"
-#include "kstplugin.h"
+#include "kstcplugin.h"
 #include "kstsettings.h"
 #include "ksttimers.h"
 #include "kstviewobjectfactory.h"
@@ -68,7 +68,7 @@ KstViewLabel::KstViewLabel(const QString& txt, KstLJustifyType justify, float ro
   _fontName = KstApp::inst()->defaultFont();
   _fontSize = -1;
   setFontSize(0);
-  _standardActions |= Delete | Edit;
+  _standardActions |= Delete | Edit | Rename;
   _parsed = 0L;
   _labelMargin = 0;
   _isResizable = false;
@@ -92,7 +92,7 @@ KstViewLabel::KstViewLabel(const QDomElement& e)
   _fontName = KstApp::inst()->defaultFont();
   _fontSize = -1;
   setFontSize(0);
-  _standardActions |= Delete | Edit;
+  _standardActions |= Delete | Edit | Rename;
   _parsed = 0L;
   _isResizable = false;
   reparse();
@@ -124,6 +124,7 @@ KstViewLabel::KstViewLabel(const KstViewLabel& label)
   _fontSize = label._fontSize;
   _absFontSize = label._absFontSize;
   _txt = label._txt;
+  _standardActions |= Delete | Edit | Rename;
 
   _parsed = 0L;
   reparse();
@@ -149,16 +150,38 @@ KstViewObject* KstViewLabel::copyObjectQuietly(KstViewObject& parent, const QStr
 }
 
 
+void KstViewLabel::setupConnections() {
+  for (KstScalarMap::iterator i = _scalarsUsed.begin(); i != _scalarsUsed.end(); ++i) {
+    disconnect(i.data(), SIGNAL(tagChanged()), this, SLOT(reparse()));
+    connect(i.data(), SIGNAL(tagChanged()), this, SLOT(reparse()));
+  } 
+  for (KstStringMap::iterator i = _stringsUsed.begin(); i != _stringsUsed.end(); ++i) {
+    disconnect(i.data(), SIGNAL(tagChanged()), this, SLOT(reparse()));
+    connect(i.data(), SIGNAL(tagChanged()), this, SLOT(reparse()));
+  } 
+  for (KstVectorMap::iterator i = _vectorsUsed.begin(); i != _vectorsUsed.end(); ++i) {
+    disconnect(i.data(), SIGNAL(tagChanged()), this, SLOT(reparse()));
+    connect(i.data(), SIGNAL(tagChanged()), this, SLOT(reparse()));
+  } 
+}
+
+
 void KstViewLabel::reparse() {
   delete _parsed;
   _parsed = Label::parse(_txt, _interpret);
+  collectObjects(_parsed, _vectorsUsed, _scalarsUsed, _stringsUsed);
+  _txt = labelText(_txt, _parsed, _vectorsUsed, _scalarsUsed, _stringsUsed);
   setDirty();
+  setupConnections();
 }
 
 
 void KstViewLabel::setText(const QString& text) {
   if (_txt != text) {
     _txt = text;
+    _scalarsUsed.clear();
+    _stringsUsed.clear();
+    _vectorsUsed.clear();
     reparse(); // calls setDirty()
   }
 }
@@ -219,6 +242,7 @@ void KstViewLabel::setInterpreted(bool interpreted) {
 
 
 void KstViewLabel::save(QTextStream &ts, const QString& indent) {
+  reparse();
   ts << indent << "<" << type() << ">" << endl;
   KstBorderedViewObject::save(ts, indent + "  ");
   ts << indent << "</" << type() << ">" << endl;
@@ -284,22 +308,8 @@ void KstViewLabel::drawToPainter(Label::Parsed *lp, QPainter& p) {
       break;
   }
 
-  switch (KST_JUSTIFY_V(_justify)) {
-    case KST_JUSTIFY_V_BOTTOM:
-      rc.y = _ascent - _textHeight / 2;
-      ty = cr.height() - int(_textHeight * abcos + _textWidth * absin) / 2 - _labelMargin*_ascent/10;
-      break;
-    case KST_JUSTIFY_V_CENTER:
-      rc.y = _ascent - _textHeight / 2;
-      ty = cr.height() / 2;
-      break;
-    case KST_JUSTIFY_V_NONE:
-    case KST_JUSTIFY_V_TOP:
-    default:
-      rc.y = _ascent - _textHeight / 2;
-      ty = int(_textHeight * abcos + _textWidth * absin) / 2 + _labelMargin*_ascent/10;
-      break;
-  }
+  rc.y = _ascent - _textHeight / 2;
+  ty = int(_textHeight * abcos + _textWidth * absin) / 2 + _labelMargin*_ascent/10;
 
   p.translate(tx, ty);
   p.rotate(_rotation);
@@ -312,7 +322,7 @@ void KstViewLabel::drawToPainter(Label::Parsed *lp, QPainter& p) {
   t.start();
 #endif
   if (lp && lp->chunk) {
-    renderLabel(rc, lp->chunk);
+    renderLabel(rc, lp->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
     _cache.valid = true;
   }
 #ifdef BENCHMARK
@@ -335,7 +345,7 @@ void KstViewLabel::computeTextSize(Label::Parsed *lp) {
     QTime t;
     t.start();
 #endif
-    renderLabel(rc, lp->chunk);
+    renderLabel(rc, lp->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
 #ifdef BENCHMARK
     kstdDebug() << "compute (false render) took: " << t.elapsed() << endl;
 #endif
@@ -395,16 +405,15 @@ void KstViewLabel::paintSelf(KstPainter& p, const QRegion& bounds) {
   if (p.type() == KstPainter::P_PRINT || p.type() == KstPainter::P_EXPORT) {
     int absFontSizeOld = _absFontSize;
     
-    adjustSizeForText(_parent->geometry());
+    QRect cr(contentsRectForPainter(p));
+    cr.setSize(sizeForText(_parent->geometry()));
+    setContentsRectForPainter(p, cr);    
     KstBorderedViewObject::paintSelf(p, bounds);
-    const QRect geom(contentsRectForDevice(p));
-    p.setViewport(geom);
-    p.setWindow(0, 0, geom.width(), geom.height());
-
+    
+    p.translate(cr.left(), cr.top());
     if (!_transparent) {
-      p.fillRect(0, 0, geom.width(), geom.height(), backgroundColor());
+      p.fillRect(0, 0, cr.width(), cr.height(), backgroundColor());
     }
-
     drawToPainter(_parsed, p);
     
     _absFontSize = absFontSizeOld;
@@ -475,7 +484,14 @@ int KstViewLabel::fontSize() const {
 }
 
 
-void KstViewLabel::adjustSizeForText(QRect w) {
+void KstViewLabel::adjustSizeForText(const QRect& w) {
+  QRect cr(contentsRect());
+  cr.setSize(sizeForText(w));
+  setContentsRect(cr);
+}
+
+
+QSize KstViewLabel::sizeForText(const QRect& w) {
   double x_s, y_s;
   
   x_s = y_s = _fontSize + (double)KstSettings::globalSettings()->plotFontSize;
@@ -505,12 +521,13 @@ void KstViewLabel::adjustSizeForText(QRect w) {
   }
  
   QSize sz(kMax(1, _textWidth), kMax(1, _textHeight));
+
   if (int(_rotation) != 0 && int(_rotation) != 180) {
     QPointArray pts(4);
     pts[0] = QPoint(0, 0);
     pts[1] = QPoint(0, _textHeight);
-    pts[2] = QPoint(_textWidth, 0);
-    pts[3] = QPoint(_textWidth, _textHeight);
+    pts[2] = QPoint(_textWidth, _textHeight);
+    pts[3] = QPoint(_textWidth, 0);
     double theta = M_PI * (int(_rotation) % 360) / 180;
     double sina = sin(theta);
     double cosa = cos(theta);
@@ -520,26 +537,26 @@ void KstViewLabel::adjustSizeForText(QRect w) {
 
     if (_parent) {
       QRect r(position(), pts.boundingRect().size());
+      r.setSize(r.size() + QSize(2 * _labelMargin * _ascent / 10, 2 * _labelMargin * _ascent / 10));
       sz = r.intersect(_parent->geometry()).size();
     } else {
       sz = pts.boundingRect().size();
+      sz += QSize(2 * _labelMargin * _ascent / 10, 2 * _labelMargin * _ascent / 10);
     }
   } else {
     if (_parent) {
       QRect r(position(), sz);
+      r.setSize(r.size() + QSize(2 * _labelMargin * _ascent / 10, 2 * _labelMargin * _ascent / 10));
       sz = r.intersect(_parent->geometry()).size();
     }
   }
-
-  QRect cr(contentsRect());
-  cr.setSize(sz + QSize(2 * _labelMargin * _ascent / 10, 2 * _labelMargin * _ascent / 10));
-  setContentsRect(cr);
+    
+  return sz;
 }
 
 
 bool KstViewLabel::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent) {
   KstViewObject::layoutPopupMenu(menu, pos, topLevelParent);
-  //menu->insertItem(i18n("&Adjust Size"), this, SLOT(adjustSizeForText()));
   return true;
 }
 
@@ -600,8 +617,7 @@ bool KstViewLabel::fillConfigWidget(QWidget *w, bool isNew) const {
     widget->_precision->setValue(8);
     widget->_rotation->setValue(0);
     widget->_fontSize->setValue(0);
-    widget->_horizontal->setCurrentItem(2);
-    widget->_vertical->setCurrentItem(2);
+    widget->_horizontal->setCurrentItem(0);
     widget->_fontColor->setColor(KstSettings::globalSettings()->foregroundColor);
     widget->_font->setCurrentFont(KstApp::inst()->defaultFont());
     widget->_margin->setValue(5);
@@ -632,7 +648,6 @@ bool KstViewLabel::fillConfigWidget(QWidget *w, bool isNew) const {
     widget->_rotation->setValue(double(rotation()));
     widget->_fontSize->setValue(int(fontSize()));
     widget->_horizontal->setCurrentItem(horizJustifyWrap());
-    widget->_vertical->setCurrentItem(vertJustifyWrap());
     widget->_fontColor->setColor(foregroundColor());
     widget->_font->setCurrentFont(fontName());
 
@@ -665,7 +680,6 @@ bool KstViewLabel::readConfigWidget(QWidget *w) {
   setRotation(widget->_rotation->value());
   setFontSize(widget->_fontSize->value());
   setHorizJustifyWrap(widget->_horizontal->currentItem());
-  setVertJustifyWrap(widget->_vertical->currentItem());
   setForegroundColor(widget->_fontColor->color());
   setFontName(widget->_font->currentFont());
 
@@ -691,7 +705,6 @@ void KstViewLabel::connectConfigWidget(QWidget *parent, QWidget *w) const {
   connect(widget->_fontSize, SIGNAL(valueChanged(int)), parent, SLOT(modified()));
   connect(widget->_fontSize->child("qt_spinbox_edit"), SIGNAL(textChanged(const QString&)), parent, SLOT(modified()));
   connect(widget->_horizontal, SIGNAL(activated(int)), parent, SLOT(modified()));
-  connect(widget->_vertical, SIGNAL(activated(int)), parent, SLOT(modified()));
   connect(widget->_fontColor, SIGNAL(changed(const QColor&)), parent, SLOT(modified()));  
   connect(widget->_precision, SIGNAL(valueChanged(int)), parent, SLOT(modified()));
   connect(widget->_precision->child("qt_spinbox_edit"), SIGNAL(textChanged(const QString&)), parent, SLOT(modified()));
@@ -777,45 +790,6 @@ void KstViewLabel::setHorizJustifyWrap(int justify) {
   }
   setJustification(SET_KST_JUSTIFY(justifySet, KST_JUSTIFY_V(justification())));
 }
-    
-        
-int KstViewLabel::vertJustifyWrap() const {
-  Q_UINT8 justify = KST_JUSTIFY_V(justification());
-  switch (justify) {
-    case KST_JUSTIFY_V_TOP:
-      return 0;
-      break;
-    case KST_JUSTIFY_V_BOTTOM:
-      return 1;
-      break;
-    case KST_JUSTIFY_V_CENTER:
-      return 2;
-      break;
-    default:
-      return 0;  
-  }
-}
-
-
-void KstViewLabel::setVertJustifyWrap(int justify) {
-  Q_UINT8 justifySet;
-  
-  switch (justify) {
-    case 0:
-      justifySet = KST_JUSTIFY_V_TOP;
-      break;  
-    case 1:
-      justifySet = KST_JUSTIFY_V_BOTTOM;
-      break;
-    case 2:
-      justifySet = KST_JUSTIFY_V_CENTER;
-      break;
-    default:
-      justifySet = KST_JUSTIFY_V_TOP;
-  }
-  setJustification(SET_KST_JUSTIFY(KST_JUSTIFY_H(justification()), justifySet));
-}
-    
 
 void KstViewLabel::setLabelMargin(int margin) {
   int mm = kMax(0, margin);
@@ -884,7 +858,7 @@ void KstViewLabel::DataCache::update() {
       case DataRef::DRExpression:
         {
           bool ok = false;
-          const double val = Equation::interpret((*i).name.latin1(), &ok);
+          const double val = Equation::interpret((*i).name.latin1(), &ok, (*i).name.length());
           if (QVariant(val) != (*i).value) {
             valid = false;
           }
@@ -893,7 +867,7 @@ void KstViewLabel::DataCache::update() {
       case DataRef::DRVector:
         {
           bool ok = false;
-          const double idx = Equation::interpret((*i).index.latin1(), &ok);
+          const double idx = Equation::interpret((*i).index.latin1(), &ok, (*i).index.length());
           if (idx != (*i).indexValue) {
             valid = false;
             break;
@@ -916,7 +890,7 @@ void KstViewLabel::DataCache::update() {
           KstDataObjectList::Iterator oi = KST::dataObjectList.findTag((*i).name);
           KST::dataObjectList.lock().unlock();
           if (oi != KST::dataObjectList.end()) {
-            KstPluginPtr fit = kst_cast<KstPlugin>(*oi);
+            KstCPluginPtr fit = kst_cast<KstCPlugin>(*oi);
             if (fit) {
               fit->readLock(); 
               if (fit->label((int)((*i).indexValue)) != (*i).index) {
