@@ -30,6 +30,7 @@
 #include "labelrenderer.h"
 #include "viewlegendwidget.h"
 #include "plotlistbox.h"
+#include "kstviewwindow.h"
 
 #include <kdatastream.h>
 #include <kglobal.h>
@@ -40,6 +41,7 @@
 #include <kdualcolorbutton.h>
 #include <kfontcombo.h>
 
+#include <qradiobutton.h>
 #include <qapplication.h>
 #include <qbitmap.h>
 #include <qdeepcopy.h>
@@ -202,6 +204,7 @@ const QString& KstViewLegend::fontName() const {
 
 
 void KstViewLegend::save(QTextStream &ts, const QString& indent) {
+  reparseTitle();
   ts << indent << "<" << type() << ">" << endl;
   KstBorderedViewObject::save(ts, indent + "  ");
   
@@ -242,7 +245,10 @@ void KstViewLegend::drawToPainter(KstPainter& p) {
       rc.x = 0;
       rc.y = _ascent;
       rc.xStart = rc.x;
-      renderLabel(rc, _parsedTitle->chunk);
+      if (_parsedTitle->chunk) {
+        _parsedTitle->chunk->attributes.color = foregroundColor();
+      }
+      renderLabel(rc, _parsedTitle->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
       i = 1;
       p.restore();
     }
@@ -257,7 +263,10 @@ void KstViewLegend::drawToPainter(KstPainter& p) {
         rc.x = 0;
         rc.y = _ascent;
         rc.xStart = rc.x;
-        renderLabel(rc, (*it)->parsedLegendTag()->chunk);
+        if ((*it)->parsedLegendTag()->chunk) {
+          (*it)->parsedLegendTag()->chunk->attributes.color = foregroundColor();
+        }
+        renderLabel(rc, (*it)->parsedLegendTag()->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
       }
       p.restore();
       ++i;
@@ -269,7 +278,10 @@ void KstViewLegend::drawToPainter(KstPainter& p) {
       rc.x = 0;
       rc.y = _ascent;
       rc.xStart = rc.x;
-      renderLabel(rc, _parsedTitle->chunk);
+      if (_parsedTitle->chunk) {
+        _parsedTitle->chunk->attributes.color = foregroundColor();
+      }
+      renderLabel(rc, _parsedTitle->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
       p.translate(_titleWidth + _ascent,0);
     }
 
@@ -282,7 +294,10 @@ void KstViewLegend::drawToPainter(KstPainter& p) {
         rc.x = 0;
         rc.y = _ascent;
         rc.xStart = rc.x;
-        renderLabel(rc, (*it)->parsedLegendTag()->chunk);
+        if ((*it)->parsedLegendTag()->chunk) {
+          (*it)->parsedLegendTag()->chunk->attributes.color = foregroundColor();
+        }
+        renderLabel(rc, (*it)->parsedLegendTag()->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
         p.translate((*it)->legendLabelSize().width() + _ascent,0);
       }
     }
@@ -301,7 +316,7 @@ void KstViewLegend::computeTextSize() {
   for (KstBaseCurveList::Iterator it = _curves.begin(); it != _curves.end(); it++) {
     if ((*it)->parsedLegendTag()) {
       RenderContext rc(_fontName, _absFontSize, 0L);
-      renderLabel(rc, (*it)->parsedLegendTag()->chunk);
+      renderLabel(rc, (*it)->parsedLegendTag()->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
       if (_vertical) {
         if (rc.xMax > _textWidth) {
           _textWidth = rc.xMax;
@@ -339,7 +354,7 @@ void KstViewLegend::computeTextSize() {
     if (!_parsedTitle) {
       reparseTitle();
     }
-    renderLabel(rc, _parsedTitle->chunk);
+    renderLabel(rc, _parsedTitle->chunk, _vectorsUsed, _scalarsUsed, _stringsUsed);
     _titleWidth = rc.xMax;
     _titleHeight = rc.fontHeight();
   } else {
@@ -361,14 +376,17 @@ void KstViewLegend::updateSelf() {
 void KstViewLegend::paintSelf(KstPainter& p, const QRegion& bounds) {
   if (p.type() == KstPainter::P_PRINT || p.type() == KstPainter::P_EXPORT) {
     p.save();
-    adjustSizeForText(_parent->geometry());
+    QRect cr(contentsRectForPainter(p));
+    cr.setSize(sizeForText(_parent->geometry()));
+    setContentsRectForPainter(p, cr);    
     KstBorderedViewObject::paintSelf(p, bounds);
-    const QRect cr(contentsRectForDevice(p));
+    
     p.translate(cr.left(), cr.top());
     if (!_transparent) {
       p.fillRect(0, 0, cr.width(), cr.height(), _backgroundColor);
     }
     drawToPainter(p);
+    
     p.restore();
   } else {
     const QRect cr(contentsRect());
@@ -437,7 +455,14 @@ int KstViewLegend::fontSize() const {
 }
 
 
-void KstViewLegend::adjustSizeForText(QRect w) {
+void KstViewLegend::adjustSizeForText(const QRect& w) {
+  QRect cr(contentsRect());
+  cr.setSize(sizeForText(w));
+  setContentsRect(cr);
+}
+
+
+QSize KstViewLegend::sizeForText(const QRect& w) {
   double x_s, y_s;
 
   x_s = y_s = _fontSize + (double)KstSettings::globalSettings()->plotFontSize;
@@ -478,15 +503,15 @@ void KstViewLegend::adjustSizeForText(QRect w) {
   }
 
   QSize sz(width, height);
+  
+  sz += QSize(2 * _legendMargin * _ascent / 10, 2 * _legendMargin * _ascent / 10);
 
   if (_parent) {
     QRect r(position(), sz);
     sz = r.intersect(_parent->geometry()).size();
   }
-
-  QRect cr(contentsRect());
-  cr.setSize(sz + QSize(2 * _legendMargin * _ascent / 10, 2 * _legendMargin * _ascent / 10));
-  setContentsRect(cr);
+    
+  return sz;
 }
 
 void KstViewLegend::modifiedLegendEntry() {
@@ -562,6 +587,8 @@ void KstViewLegend::setCurveList(Kst2DPlotPtr pl) {
 void KstViewLegend::reparseTitle() {
   delete _parsedTitle;
   _parsedTitle = Label::parse(_title, true, false);
+  collectObjects(_parsedTitle, _vectorsUsed, _scalarsUsed, _stringsUsed);
+  _title = labelText(_title, _parsedTitle, _vectorsUsed, _scalarsUsed, _stringsUsed);
   setDirty();
 }
 
@@ -625,24 +652,23 @@ bool KstViewLegend::fillConfigWidget(QWidget *w, bool isNew) const {
 
 /** apply properties in the custom config widget to this */
 bool KstViewLegend::readConfigWidget(QWidget *w) {
+  KstViewLegendList legends;
+  KstViewLegendPtr legendExtra;
+
   ViewLegendWidget *widget = dynamic_cast<ViewLegendWidget*>(w);
   if (!widget) {
     return false;
   }
 
-  setFontSize(widget->_fontSize->value());
-  setForegroundColor(widget->_fontColor->color());
-  setFontName(widget->_font->currentFont());
-  setTitle(widget->_title->text());
-
-  setTransparent(widget->_transparent->isChecked());
-  setBorderWidth(widget->_border->value());
-  setBorderColor(widget->_boxColors->foreground());
-  setBackgroundColor(widget->_boxColors->background());
-  setLegendMargin(widget->_margin->value());
-  setVertical(widget->_vertical->isChecked());
-  setTrackContents(widget->TrackContents->isChecked());
+  if (widget->_thisLegend->isChecked()) {
+    legends += this;
+  } else if (widget->_thisWindow->isChecked()) {
+    legends = topLevelParent()->findChildrenType<KstViewLegend>(true);
+  } else {
+    legends = globalLegendList();
+  }
 
+  // apply the curve list, but only to this legend!
   KstBaseCurveList allCurves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
   _curves.clear();
   for (unsigned i = 0; i < widget->DisplayedCurveList->count(); i++) {
@@ -652,6 +678,23 @@ bool KstViewLegend::readConfigWidget(QWidget *w) {
     }
   }
 
+  for (uint i = 0; i < legends.size(); i++) {
+    legendExtra = legends[i];
+
+    legendExtra->setFontSize(widget->_fontSize->value());
+    legendExtra->setForegroundColor(widget->_fontColor->color());
+    legendExtra->setFontName(widget->_font->currentFont());
+    legendExtra->setTitle(widget->_title->text());
+
+    legendExtra->setTransparent(widget->_transparent->isChecked());
+    legendExtra->setBorderWidth(widget->_border->value());
+    legendExtra->setBorderColor(widget->_boxColors->foreground());
+    legendExtra->setBackgroundColor(widget->_boxColors->background());
+    legendExtra->setLegendMargin(widget->_margin->value());
+    legendExtra->setVertical(widget->_vertical->isChecked());
+    legendExtra->setTrackContents(widget->TrackContents->isChecked());
+  }
+
   setDirty();
   return true;
 }
@@ -680,6 +723,8 @@ void KstViewLegend::connectConfigWidget(QWidget *parent, QWidget *w) const {
   connect(widget->_margin->child("qt_spinbox_edit"), SIGNAL(textChanged(const QString&)), parent, SLOT(modified()));
   connect(widget->_border, SIGNAL(valueChanged(int)), parent, SLOT(modified()));
   connect(widget->_border->child("qt_spinbox_edit"), SIGNAL(textChanged(const QString&)), parent, SLOT(modified()));
+  connect(widget->_thisLegend, SIGNAL(stateChanged(int)), parent, SLOT(modified()));
+
 }
 
 
@@ -721,6 +766,9 @@ const QString& KstViewLegend::title() const {
 void KstViewLegend::setTitle(const QString& title_in) {
   if (_title != title_in) {
     _title = title_in;
+    _scalarsUsed.clear();
+    _stringsUsed.clear();
+    _vectorsUsed.clear();
     reparseTitle(); // calls setDirty()
   }
 }
@@ -737,6 +785,23 @@ KstBaseCurveList& KstViewLegend::curves() {
   return _curves;
 }
 
+KstViewLegendList KstViewLegend::globalLegendList() {
+  KstViewLegendList rc;
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *it = app->createIterator();
+  if (it) {
+    while (it->currentItem()) {
+      KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
+      if (view) {
+        KstViewLegendList sub = view->view()->findChildrenType<KstViewLegend>(true);
+        rc += sub;
+      }
+      it->next();
+    }
+    app->deleteIterator(it);
+  }
+  return rc;
+}
 
 namespace {
 KstViewObject *create_KstViewLegend() {
