@@ -528,13 +528,13 @@ void KstViewManagerI::edit_I() {
     } else if (qi->rtti() == RTTI_OBJ_VIEW_OBJECT) {
       KstViewObjectPtr obj = koi->viewObject(&win);
       if (obj && win) {
-        obj->showDialog(win->view());  
+        obj->showDialog(win->view(), false);  
       }
     } else if (qi->rtti() == RTTI_OBJ_DATA_OBJECT) {
       Kst2DPlotPtr plot;
       KstDataObjectPtr obj = koi->dataObject(plot);
       if (obj) {
-        obj->showDialog();  
+        obj->showDialog(false);  
       }
     }
   } else {
