@@ -146,7 +146,7 @@ void KstViewObject::load(const QDomElement& e) {
     QDomElement el = n.toElement(); // try to convert the node to an element.
     if (!el.isNull()) { // the node was really an element.
       if (el.tagName() == "tag") {
-        setTagName(el.text());
+        setTagName(KstObjectTag::fromString(el.text()));
       } else if (el.tagName() == "transparent") {
         _transparent = true;
       } else if (el.tagName() == "columns") {
@@ -700,7 +700,7 @@ void KstViewObject::cleanup(int cols) {
   KstViewObjectList childrenCopy;
   double ave_w = 0.0;
   bool dirty = false;
-  
+
   for (KstViewObjectList::ConstIterator i = _children.begin(); i != _children.end(); ++i) {
     if ((*i)->followsFlow()) {
       childrenCopy.append(*i);
@@ -709,12 +709,12 @@ void KstViewObject::cleanup(int cols) {
   }
 
   int cnt = childrenCopy.count();
-  
+
   if (cnt < 1) {
     return;
   }
   ave_w /= double(cnt);
-  
+
   // FIXME: don't allow regrid to a number of columns that will result in
   //        >= height() plots in a column
   if (cols <= 0) {
@@ -754,13 +754,12 @@ void KstViewObject::cleanup(int cols) {
   // unless there is another plot which is closer, in which case the 
   // plot gets dumped into an un-assigned list for placement into a
   // random un-filled grid space.
-  // the Location is defined relative to the left-middle.
+  // the location is defined relative to the left-middle.
   // QUESTION: should we use the middle-middle instead?
   // NOTE: the choice of grid location assumes a regular grid, which is
   // broken when supressed axis/labels are taken into account.  This
   // could have an effect if the plots are grown by >50%.
   for (int i = 0; i < cnt; ++i) {
-    //r = int(childrenCopy[i]->aspectRatio().y*rows+0.5); //use top
     r = int((childrenCopy[i]->aspectRatio().y+childrenCopy[i]->aspectRatio().h/2)*rows); // use center
     c = int(childrenCopy[i]->aspectRatio().x*_columns+0.5);
 
@@ -796,7 +795,7 @@ void KstViewObject::cleanup(int cols) {
     }
   }
   // now dump the unassigned plots in random holes.
-  // Question: should be dump them in the closest holes?
+  // Question: should we dump them in the closest holes?
   CR = 0;
   for (int i = 0; i < n_unassigned; ++i) {
     for (; plotLoc[CR] != -1; ++CR) { }
@@ -807,7 +806,7 @@ void KstViewObject::cleanup(int cols) {
   double sum_HR = 0.0;
   KstViewObject *ob;
   double hr;
-  
+
   for (r=0; r<rows; r++) {
     HR[r] = 10.0;
     for (c=0; c<_columns; c++) {
@@ -831,7 +830,6 @@ void KstViewObject::cleanup(int cols) {
   int y = 0;
   for (r=0; r<rows; r++) {
     y += h;
-    //h = _geom.height() / rows;
     h = int(double(_geom.height()) * HR[r]/sum_HR);
     for (c=0; c<_columns; c++) {
       CR = c + r*_columns;
@@ -841,14 +839,24 @@ void KstViewObject::cleanup(int cols) {
         c = CR % _columns;
         QPoint pt(w*c, y);
 
-        // adjust the last column to be sure that we don't spill over
+        // if necessary adjust the last column so that we don't spill over
         if (c == _columns-1) {
-          sz.setWidth(_geom.width() - w*c);
+          // only adjust the final width if necessary as we would rather have a gap
+          // at the right edge of the window than a column of plots that is significantly 
+          // wider than all the others
+          if (w*_columns > _geom.width()) {
+            sz.setWidth(_geom.width() - w*c);
+          }
         }
 
-        // adjust the last row to be sure that we don't spill over
+        // if necessary adjust the last row so that we don't spill over
         if (r == rows - 1) {
-          sz.setHeight(_geom.height() - y);
+          // only adjust the final height if necessary as we would rather have a gap
+          // at the bottom edge of the window than a row of plots that is significantly 
+          // taller than all the others
+          if (y + h > _geom.height()) {
+            sz.setHeight(_geom.height() - y);
+          }
         }
 
         ob = childrenCopy[plotLoc[CR]];
@@ -1103,11 +1111,19 @@ bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObject
     rc = true;
   }
 
-  if (_standardActions & Copy) {
-    menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
+
+// Copy doesn't do anything yet, so don't put it in the UI...
+//   if (_standardActions & Copy) {
+//     menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
+//     rc = true;
+//   }
+  
+  if (_layoutActions & Rename) {
+    menu->insertItem(i18n("Re&name..."), this, SLOT(rename()));
     rc = true;
   }
 
+
   if (_standardActions & Zoom) {
     id = menu->insertItem(i18n("Maximi&ze"), this, SLOT(zoomToggle()));
     if (_maximized) {
@@ -1147,17 +1163,17 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
     menu->insertItem(i18n("&Edit..."), this, SLOT(edit()));
     rc = true;
   }
-
-  if (_layoutActions & Copy) {
-    menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
-    rc = true;
-  }
-
+  
   if (_layoutActions & Delete) {
     menu->insertItem(i18n("&Delete"), this, SLOT(deleteObject()));
     rc = true;
   }
 
+//   if (_layoutActions & Copy) {
+//     menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
+//     rc = true;
+//   }
+
   if (_layoutActions & Rename) {
     menu->insertItem(i18n("Re&name..."), this, SLOT(rename()));
     rc = true;
@@ -1263,7 +1279,7 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
 
 void KstViewObject::edit() {
   KstTopLevelViewPtr tlv = kst_cast<KstTopLevelView>(KstViewObjectPtr(_topObjectForMenu));
-  showDialog(tlv);
+  showDialog(tlv, false);
 }
 
 
@@ -1545,13 +1561,32 @@ void KstViewObject::detach() {
 
 void KstViewObject::rename() {
   bool ok = false;
+  bool done;
+  QString oldName = tagName();
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
   QString newName = KInputDialog::getText(i18n("Kst"), i18n("Enter a new name for %1:").arg(tagName()), tagName(), &ok);
 #else
   QString newName = KLineEditDlg::getText(i18n("Enter a new name for %1:").arg(tagName()), tagName(), &ok, 0L);
 #endif
+  done = !ok;
+  while (!done) {
+    setTagName(KstObjectTag(newName+"tmpholdingstring", KstObjectTag::globalTagContext));
+    if (KstData::self()->viewObjectNameNotUnique(newName)) {
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+      newName = KInputDialog::getText(i18n("Kst"), i18n("%1 is not a unique name: Enter a new name for %2:").arg(newName).arg(oldName), oldName, &ok);
+#else
+      newName = KLineEditDlg::getText(i18n("%1 is not a unique name: Enter a new name for %2:").arg(newName).arg(oldName), oldName, &ok, 0L);
+#endif
+      done = !ok;
+    } else {
+      done = ok = true;
+    }
+  }
+  
   if (ok) {
-    setTagName(newName);
+    setTagName(KstObjectTag(newName, KstObjectTag::globalTagContext)); // FIXME: handle tag context
+  } else {
+    setTagName(KstObjectTag(oldName, KstObjectTag::globalTagContext)); // FIXME: handle tag context
   }
 }
 
@@ -1595,7 +1630,7 @@ QDataStream& operator>>(QDataStream& str, KstViewObjectPtr obj) {
 void KstViewObject::readBinary(QDataStream& str) {
   QString tagName;
   str >> tagName;
-  setTagName(tagName);
+  setTagName(KstObjectTag(tagName, KstObjectTag::globalTagContext)); // FIXME: tag context
   kstdDebug() << "Decoding " << tagName << " from drag." << endl;
   // FIXME: rename objects if they cause a namespace conflict
   str >> _geom >> _backgroundColor >> _foregroundColor;
@@ -1847,18 +1882,11 @@ bool KstViewObject::followsFlow() const {
 }
 
 
-// id=0 v1.2.0 setDirty() was called.  data == 0 means false, else true
-void KstViewObject::virtual_hook(int id, void *data) {
-  switch (id) {
-    case 0:
-      if (data) {
-        invalidateClipRegion();
-      }
-      break;
-    default:
-      break;
+void KstViewObject::setDirty(bool dirty) {
+  if (dirty) {
+    invalidateClipRegion();
   }
-  KstObject::virtual_hook(id, data);
+  KstObject::setDirty(dirty);
 }
 
 
