@@ -253,6 +253,7 @@ class KST_EXPORT KstViewObject : public KstObject {
     virtual bool paste(QMimeSource* source, KstViewObjectList *list = 0L);
     
     virtual KstGfxMouseHandler *createHandler();
+    void setDirty(bool = true);
 
   public slots:
     void paint(KstPainter& p, const QRegion& bounds);
@@ -266,7 +267,7 @@ class KST_EXPORT KstViewObject : public KstObject {
     virtual void zoomToggle();
     virtual void copyObject();
     virtual KstViewObject* copyObjectQuietly(KstViewObject& parent, const QString& name = QString::null) const;
-    virtual bool showDialog(KstTopLevelViewPtr invoker, bool isNew = false);
+    virtual bool showDialog(KstTopLevelViewPtr invoker, bool isNew);
 
   protected slots:
     virtual void parentResized();
@@ -295,9 +296,6 @@ class KST_EXPORT KstViewObject : public KstObject {
     virtual KstHandlerFactoryMethod handlerFactory() const;
     virtual void writeBinary(QDataStream& str);
     virtual void readBinary(QDataStream& str);
-    // id=0 v1.2.0 setDirty() was called.  data == 0 means false, else true
-    virtual void virtual_hook(int id, void *data);
-    
     KstObject::UpdateType updateChildren(int counter);
     
     KstViewObjectList _children;
