@@ -17,6 +17,10 @@
 
 #include "updatethread.h"
 
+#ifdef MULTICORE_UPDATES
+
+#include "updatethread-multicore.cpp"
+#else
 #include <assert.h>
 
 #include <qdeepcopy.h>
@@ -31,7 +35,7 @@
 #include "threadevents.h"
 
 // 0 - none, 1 - some, 2 - lots, 3 - too much
-#define UPDATEDEBUG 1
+#define UPDATEDEBUG 0
 
 UpdateThread::UpdateThread(KstDoc *doc)
 : QThread(), _paused(false), _done(false), _statusMutex(false), _doc(doc) {
@@ -50,6 +54,14 @@ void UpdateThread::run() {
   bool force;
   int  updateTime;
 
+#if UPDATEDEBUG > 0
+  kstdDebug() << "Update thread running, tid=" << (int)QThread::currentThread() << endl;
+#if UPDATEDEBUG > 2
+  kstdDebug() << "dataObjectList lock is at " << (void*)(&KST::dataObjectList.lock()) << endl;
+  kstdDebug() << "dataSourceList lock is at " << (void*)(&KST::dataSourceList.lock()) << endl;
+#endif
+#endif
+
   _done = false;
 
   while (!_done) {
@@ -164,11 +176,11 @@ bool UpdateThread::doUpdates(bool force, bool *gotData) {
     // Update all curves
     for (uint i = 0; i < cl.count(); ++i) {
       KstBaseCurvePtr bcp = cl[i];
+      bcp->writeLock();
       assert(bcp.data());
 #if UPDATEDEBUG > 1
       kstdDebug() << "updating curve: " << (void*)bcp << " - " << bcp->tagName() << endl;
 #endif
-      bcp->writeLock();
       KstObject::UpdateType ut = bcp->update(_updateCounter);
       bcp->unlock();
 
@@ -196,11 +208,11 @@ bool UpdateThread::doUpdates(bool force, bool *gotData) {
     // Update all data objects
     for (uint i = 0; i < dol.count(); ++i) {
       KstDataObjectPtr dp = dol[i];
+      dp->writeLock();
       assert(dp.data());
 #if UPDATEDEBUG > 1
       kstdDebug() << "updating data object: " << (void*)dp << " - " << dp->tagName() << endl;
 #endif
-      dp->writeLock();
       dp->update(_updateCounter);
       dp->unlock();
 
@@ -236,7 +248,7 @@ bool UpdateThread::doUpdates(bool force, bool *gotData) {
     KstScalar::clearScalarsDirty(); // Must do this first and take a risk of
                                     // falling slightly behind
     KST::scalarList.lock().readLock();
-    KstScalarList sl = QDeepCopy<KstScalarList>(KST::scalarList); // avoid deadlock on exit
+    KstScalarList sl = QDeepCopy<KstScalarList>(KST::scalarList.list()); // avoid deadlock on exit
     KST::scalarList.lock().unlock();
     for (KstScalarList::ConstIterator i = sl.begin(); i != sl.end(); ++i) {
       KstScalarPtr sp = *i;
@@ -306,4 +318,5 @@ bool UpdateThread::paused() const {
   return _paused;
 }
 
+#endif
 // vim: ts=2 sw=2 et
