@@ -16,20 +16,23 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "kstvcurve.h"
-
-#include "kstdataobjectcollection.h"
-#include <kstdatacollection.h>
 #include "kstdataobject.h"
-#include "kstdebug.h"
 
-#include <qdeepcopy.h>
+#include "ksdebug.h"
+#include "kstdebug.h"
+#include "kstdatacollection.h"
+#include "kstdataobjectcollection.h"
+
 #include <qtimer.h>
+#include <qdeepcopy.h>
 
 #include <assert.h>
 
-#include "ksdebug.h"
 #include <klocale.h>
+#include <klibloader.h>
+#include <kparts/componentfactory.h>
+
+#include "kstdataplugin.h"
 
 //#define LOCKTRACE
 
@@ -80,10 +83,97 @@ KstDataObject::~KstDataObject() {
     KST::matrixList.remove(it.data());       
   }
   KST::matrixList.lock().unlock();
-  //kstdDebug() << "+++ DESTROYING DATA OBJECT: " << (void*)this << endl;
+//  kstdDebug() << "Destroying Data Object: " << tag().displayString() << endl;
   delete _curveHints;
 }
 
+static QMap<QString, KstDataObjectPtr> pluginInfo;
+void KstDataObject::cleanupForExit() {
+  pluginInfo.clear(); //FIXME?
+}
+
+
+KstDataObjectPtr KstDataObject::createPlugin(KService::Ptr service) {
+  int err = 0;
+  KstDataObject *object =
+      KParts::ComponentFactory::createInstanceFromService<KstDataObject>(service, 0, "",
+      QStringList(), &err);
+
+  KstSharedPtr<KST::Plugin> p = new KST::DataObjectPlugin(service);
+
+  if (object && p->key()) {
+    const QString name = service->property("Name").toString();
+    const QString description = service->property("Comment").toString();
+    const QString author = service->property("X-Kst-Plugin-Author").toString();
+    const QString version = service->property("X-Kst-Plugin-Version").toString();
+    const QString library = service->library();
+    Q_ASSERT( !name.isEmpty() );
+    Q_ASSERT( !library.isEmpty() );
+    object->setName(name);
+    object->setAuthor(author);
+    object->setDescription(description);
+    object->setVersion(version);
+    object->setLibrary(library);
+
+    KstDebug::self()->log(i18n("Loaded data-object plugin %1.").arg(service->name()));
+    return object;
+  }
+
+  KstDebug::self()->log(i18n("Could not load data-object plugin %1.").arg(service->name()), KstDebug::Error);
+  return 0L;
+}
+
+
+// Scans for plugins and stores the information for them
+void KstDataObject::scanPlugins() {
+  KstDebug::self()->log(i18n("Scanning for data-object plugins."));
+
+  pluginInfo.clear(); //FIXME?
+
+  KService::List sl = KServiceType::offers("Kst Data Object");
+  for (KService::List::ConstIterator it = sl.begin(); it != sl.end(); ++it) {
+    if (KstDataObjectPtr object = createPlugin(*it)) {
+      pluginInfo.insert((*it)->name(), KstDataObjectPtr(object));
+    }
+  }
+}
+
+
+KstPluginInfoList KstDataObject::pluginInfoList() {
+  if (pluginInfo.isEmpty()) {
+    scanPlugins();
+  }
+
+  KstPluginInfoList list;
+  QMap<QString, KstDataObjectPtr>::ConstIterator it = pluginInfo.begin();
+  for (; it != pluginInfo.end(); ++it) {
+    list.insert(it.key(), it.data()->kind());
+  }
+  return list;
+}
+
+
+KstDataObjectPtr KstDataObject::plugin(const QString& name) {
+    if (pluginInfo.contains(name)) {
+        return pluginInfo[name];
+    }
+    return 0L;
+}
+
+
+KstDataObjectPtr KstDataObject::createPlugin(const QString& name) {
+  KService::List sl = KServiceType::offers("Kst Data Object");
+  for (KService::List::ConstIterator it = sl.begin(); it != sl.end(); ++it) {
+    if ((*it)->name() != name) {
+      continue;
+    } else if (KstDataObjectPtr object = createPlugin(*it)) {
+      return object;
+    }
+  }
+  return 0L;
+}
+
+
 double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSize) const {
   if (!v) {
     return 0L;
@@ -96,11 +186,18 @@ double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSi
   return v->realloced(memptr, newSize);
 }
 
+
+void KstDataObject::load(const QDomElement &e) {
+  Q_UNUSED(e)
+}
+
+
 void KstDataObject::save(QTextStream& ts, const QString& indent) {
   Q_UNUSED(ts)
   Q_UNUSED(indent)
 }
 
+
 bool KstDataObject::loadInputs() {
   bool rc = true;
   QValueList<QPair<QString,QString> >::Iterator i;
@@ -165,6 +262,7 @@ bool KstDataObject::loadInputs() {
   return rc;
 }
 
+
 int KstDataObject::getUsage() const {
   int rc = 0;
 
@@ -196,97 +294,48 @@ int KstDataObject::getUsage() const {
 }
 
 
-void KstDataObject::showDialog() {
-  QTimer::singleShot(0, this, SLOT(_showDialog()));
+void KstDataObject::showDialog(bool isNew) {
+  if (isNew) {
+    QTimer::singleShot(0, this, SLOT(showNewDialog()));
+  } else {
+    QTimer::singleShot(0, this, SLOT(showEditDialog()));
+  }
 }
 
 
 void KstDataObject::readLock() const {
   #ifdef LOCKTRACE
-  kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::readLock() by tid=" << (int)QThread::currentThread() << endl;
-  kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::readLock() by tid=" << (int)QThread::currentThread() << ": read locking myself" << endl;
+  kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::readLock() by tid=" << (int)QThread::currentThread() << ": read locking myself" << endl;
   #endif
+
   KstObject::readLock();
-
-  QValueList<KstPrimitivePtr> inputs;
-  QValueList<KstPrimitivePtr> outputs;
-
-  QValueList<KstStringPtr> sl = _inputStrings.values();
-  for (QValueList<KstStringPtr>::Iterator i = sl.begin(); i != sl.end(); ++i) {
-    inputs += (*i).data();
-  }
-  sl = _outputStrings.values();
-  for (QValueList<KstStringPtr>::Iterator i = sl.begin(); i != sl.end(); ++i) {
-    outputs += (*i).data();
-  }
-  
-  QValueList<KstScalarPtr> sc = _inputScalars.values();
-  for (QValueList<KstScalarPtr>::Iterator i = sc.begin(); i != sc.end(); ++i) {
-    inputs += (*i).data();
-  }
-  sc = _outputScalars.values();
-  for (QValueList<KstScalarPtr>::Iterator i = sc.begin(); i != sc.end(); ++i) {
-    outputs += (*i).data();
-  }
-  
-  QValueList<KstVectorPtr> vl = _inputVectors.values();
-  for (QValueList<KstVectorPtr>::Iterator i = vl.begin(); i != vl.end(); ++i) {
-    inputs += (*i).data();
-  }
-  vl = _outputVectors.values();
-  for (QValueList<KstVectorPtr>::Iterator i = vl.begin(); i != vl.end(); ++i) {
-    outputs += (*i).data();
-  }
-  
-  QValueList<KstMatrixPtr> ml = _inputMatrices.values();
-  for (QValueList<KstMatrixPtr>::Iterator i = ml.begin(); i != ml.end(); ++i) {
-    inputs += (*i).data();
-  }
-  ml = _outputMatrices.values();
-  for (QValueList<KstMatrixPtr>::Iterator i = ml.begin(); i != ml.end(); ++i) {
-    outputs += (*i).data();
-  }
-
-  qHeapSort(inputs);
-  qHeapSort(outputs);
-
-  QValueList<KstPrimitivePtr>::ConstIterator inputIt = inputs.begin();
-  QValueList<KstPrimitivePtr>::ConstIterator outputIt = outputs.begin();
-
-  while (inputIt != inputs.end() || outputIt != outputs.end()) {
-    if (inputIt != inputs.end() && (outputIt == outputs.end() || (void*)(*inputIt) < (void*)(*outputIt))) {
-      // do input
-      #ifdef LOCKTRACE
-//      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::readLock() by tid=" << (int)QThread::currentThread() << ": read locking input " << (*inputIt)->type() << " " << (void*)(*i) << endl;
-      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::readLock() by tid=" << (int)QThread::currentThread() << ": read locking input " << (void*)(*inputIt) << endl;
-      #endif
-      (*inputIt)->readLock();
-      ++inputIt;
-    } else {
-      // do output
-      #ifdef LOCKTRACE
-//      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::readLock() by tid=" << (int)QThread::currentThread() << ": read locking output " << (*outputIt)->type() << " " << (void*)(*i) << endl;
-      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::readLock() by tid=" << (int)QThread::currentThread() << ": read locking output " << (void*)(*outputIt) << endl;
-      #endif
-      if ((*outputIt)->provider() != this) {
-        KstDebug::self()->log(i18n("(%1) KstDataObject::readLock() by tid=%2: read locking output %3 (not provider) -- this is probably an error. Please email kst@kde.org with details.").arg(this->type()).arg((int)QThread::currentThread()).arg((*outputIt)->tagName()), KstDebug::Error);
-      }
-      (*outputIt)->readLock();
-      ++outputIt;
-    }
-  }
 }
 
 
-// Note to self, this is nasty.  Some objects may lock while others don't,
-// leading to very complex deadlocks or loss of consistency.
-// Might want to guard these with another lock yet.
 void KstDataObject::writeLock() const {
   #ifdef LOCKTRACE
-  kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::writeLock() by tid=" << (int)QThread::currentThread() << endl;
-  kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::writeLock() by tid=" << (int)QThread::currentThread() << ": write locking myself" << endl;
+  kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::writeLock() by tid=" << (int)QThread::currentThread() << ": write locking myself" << endl;
+  #endif
+
+  KstObject::writeLock();
+}
+
+
+void KstDataObject::unlock() const {
+  #ifdef LOCKTRACE
+  kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking myself" << endl;
+  #endif
+
+  KstObject::unlock();
+}
+
+
+void KstDataObject::writeLockInputsAndOutputs() const {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
+  #ifdef LOCKTRACE
+  kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::writeLockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << endl;
   #endif
-  KstRWLock::writeLock();
 
   QValueList<KstPrimitivePtr> inputs;
   QValueList<KstPrimitivePtr> outputs;
@@ -336,20 +385,24 @@ void KstDataObject::writeLock() const {
   while (inputIt != inputs.end() || outputIt != outputs.end()) {
     if (inputIt != inputs.end() && (outputIt == outputs.end() || (void*)(*inputIt) < (void*)(*outputIt))) {
       // do input
-      #ifdef LOCKTRACE
-//      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::writeLock() by tid=" << (int)QThread::currentThread() << ": read locking input " << (*inputIt)->type() << " " << (void*)(*i) << endl;
-      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::writeLock() by tid=" << (int)QThread::currentThread() << ": read locking input " << (void*)(*inputIt) << endl;
-      #endif
-      (*inputIt)->readLock();
+      if (!(*inputIt)) {
+        kstdFatal() << "Input for data object " << this->tag().displayString() << " is invalid." << endl;
+      }
+#ifdef LOCKTRACE
+      kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::writeLockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": write locking input \"" << (*inputIt)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*inputIt) << ")" << endl;
+#endif
+      (*inputIt)->writeLock();
       ++inputIt;
     } else {
       // do output
-      #ifdef LOCKTRACE
-//      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::writeLock() by tid=" << (int)QThread::currentThread() << ": write locking output " << (*outputIt)->type() << " " << (void*)(*i) << endl;
-      kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::writeLock() by tid=" << (int)QThread::currentThread() << ": write locking output " << (void*)(*outputIt) << endl;
-      #endif
+      if (!(*outputIt)) {
+        kstdFatal() << "Output for data object " << this->tag().displayString() << " is invalid." << endl;
+      }
+#ifdef LOCKTRACE
+      kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::writeLockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": write locking output \"" << (*outputIt)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*outputIt) << ")" << endl;
+#endif
       if ((*outputIt)->provider() != this) {
-        KstDebug::self()->log(i18n("(%1) KstDataObject::writeLock() by tid=%2: write locking output %3 (not provider) -- this is probably an error. Please email kst@kde.org with details.").arg(this->type()).arg((int)QThread::currentThread()).arg((*outputIt)->tagName()), KstDebug::Error);
+        KstDebug::self()->log(i18n("(%1) KstDataObject::writeLockInputsAndOutputs() by tid=%2: write locking output %3 (not provider) -- this is probably an error. Please email kst@kde.org with details.").arg(this->type()).arg((int)QThread::currentThread()).arg((*outputIt)->tagName()), KstDebug::Error);
       }
       (*outputIt)->writeLock();
       ++outputIt;
@@ -358,71 +411,90 @@ void KstDataObject::writeLock() const {
 }
 
 
-void KstDataObject::unlock() const {
+void KstDataObject::unlockInputsAndOutputs() const {
   #ifdef LOCKTRACE
-  kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << endl;
+  kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << endl;
   #endif
 
   for (KstMatrixMap::ConstIterator i = _outputMatrices.begin(); i != _outputMatrices.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Output matrix for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
     #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking output matrix " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking output matrix \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
 
   for (KstMatrixMap::ConstIterator i = _inputMatrices.begin(); i != _inputMatrices.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Input matrix for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
     #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking input matrix " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking input matrix \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
 
   for (KstVectorMap::ConstIterator i = _outputVectors.begin(); i != _outputVectors.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Output vector for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
     #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking output vector " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking output vector \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
 
   for (KstVectorMap::ConstIterator i = _inputVectors.begin(); i != _inputVectors.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Input vector for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
     #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking input vector " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking input vector \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
 
   for (KstScalarMap::ConstIterator i = _outputScalars.begin(); i != _outputScalars.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Output scalar for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
     #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking output scalar " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking output scalar \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
 
   for (KstScalarMap::ConstIterator i = _inputScalars.begin(); i != _inputScalars.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Input scalar for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
     #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking input scalar " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking input scalar \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
 
   for (KstStringMap::ConstIterator i = _outputStrings.begin(); i != _outputStrings.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Output string for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
    #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking output string " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking output string \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
 
   for (KstStringMap::ConstIterator i = _inputStrings.begin(); i != _inputStrings.end(); ++i) {
+    if (!(*i)) {
+      kstdFatal() << "Input string for data object " << this->tag().displayString() << " is invalid." << endl;
+    }
     #ifdef LOCKTRACE
-    kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking input string " << (void*)(*i) << endl;
+    kstdDebug() << (void*)this << " (" << this->type() << ": " << this->tag().tagString() << ") KstDataObject::unlockInputsAndOutputs() by tid=" << (int)QThread::currentThread() << ": unlocking input string \"" << (*i)->tag().tagString() << "\" (" << (void*)((KstRWLock*)*i) << ")" << endl;
     #endif
     (*i)->unlock();
   }
-
-  #ifdef LOCKTRACE
-  kstdDebug() << (void*)this << " (" << this->type() << ") KstDataObject::unlock() by tid=" << (int)QThread::currentThread() << ": unlocking myself" << endl;
-  #endif
-  KstRWLock::unlock();
 }
 
 
@@ -591,7 +663,6 @@ void KstDataObject::replaceDependency(KstMatrixPtr oldMatrix, KstMatrixPtr newMa
 
 
 bool KstDataObject::uses(KstObjectPtr p) const {
-  
   KstVectorPtr v = kst_cast<KstVector>(p);
   if (v) {
     for (KstVectorMap::ConstIterator j = _inputVectors.begin(); j != _inputVectors.end(); ++j) {
