@@ -25,22 +25,36 @@
 #include "kstmatrix.h"
 #include "kst_export.h"
 
+#include <kservicetype.h>
+
 typedef KstSharedPtr<KstDataObject> KstDataObjectPtr;
 typedef KstObjectList<KstDataObjectPtr> KstDataObjectList;
 typedef QMap<KstDataObjectPtr, KstDataObjectPtr> KstDataObjectDataObjectMap;
+typedef QMap<QString, int> KstPluginInfoList;
 
 class KST_EXPORT KstDataObject : public KstObject {
   Q_OBJECT
+
   public:
+    enum Kind { Generic, Primitive, Fit, Filter };
+
     KstDataObject();
     KstDataObject(const QDomElement& e);
     KstDataObject(const KstDataObject& object);
     virtual ~KstDataObject();
 
+    // These static methods are not for plugins to use
+    static void cleanupForExit();
+    /** Returns a list of object plugins found on the system. */
+    static KstPluginInfoList pluginInfoList();
+    static KstDataObjectPtr plugin(const QString& name);
+    static KstDataObjectPtr createPlugin(const QString& name);
+
     virtual UpdateType update(int updateCounter = -1) = 0;
     virtual const QString& typeString() const { return _typeString; }
     virtual QString propertyString() const = 0;
     virtual const QString& type() const { return _type; }
+    virtual Kind kind() const { return Generic; }
 
     virtual int sampleCount() const { return 0; }
 
@@ -66,10 +80,9 @@ class KST_EXPORT KstDataObject : public KstObject {
     KstMatrixMap& inputMatrices() { return _inputMatrices; }
     KstMatrixMap& outputMatrices() { return _outputMatrices; }
 
+    virtual void load(const QDomElement& e);
     virtual void save(QTextStream& ts, const QString& indent = QString::null);
 
-    void showDialog();
-
     virtual bool loadInputs();
 
     virtual int getUsage() const;
@@ -94,13 +107,34 @@ class KST_EXPORT KstDataObject : public KstObject {
 
     virtual bool uses(KstObjectPtr p) const;
 
+    //These are generally only valid for plugins...
+    const QString& name() const { return _name; }
+    const QString& author() const { return _author; }
+    const QString& description() const { return _description; }
+    const QString& version() const { return _version; }
+    const QString& library() const { return _library; }
+
+    void showDialog(bool isNew = true);
+
   protected slots:
-    virtual void _showDialog() = 0;
+    virtual void showNewDialog() = 0;
+    virtual void showEditDialog() = 0;
 
   protected:
     
     double *vectorRealloced(KstVectorPtr v, double *memptr, int newSize) const;
 
+    //The plugin infrastructure will read the desktop file and set these
+    //Other objects that inherit can set the ones that apply if desired...
+    void setName(const QString& str) { _name = str; }
+    void setAuthor(const QString& str) { _author = str; }
+    void setDescription(const QString& str) { _description = str; }
+    void setVersion(const QString& str) { _version = str; }
+    void setLibrary(const QString& str) { _library = str; }
+
+    virtual void writeLockInputsAndOutputs() const;
+    virtual void unlockInputsAndOutputs() const;
+
     KstVectorMap _inputVectors;
     KstVectorMap _outputVectors;
     KstScalarMap _inputScalars;
@@ -118,6 +152,17 @@ class KST_EXPORT KstDataObject : public KstObject {
     QValueList<QPair<QString,QString> > _inputStringLoadQueue;
     QValueList<QPair<QString,QString> > _inputMatrixLoadQueue;
     KstCurveHintList *_curveHints;
+
+  private:
+    QString _name;
+    QString _author;
+    QString _description;
+    QString _version;
+    QString _library;
+
+  private:
+    static void scanPlugins();
+    static KstDataObjectPtr createPlugin(KService::Ptr);
 };
 
 
