@@ -54,8 +54,7 @@ KstEquation::KstEquation(const QString& in_tag, const QString& equation, double
   KstVectorPtr xvector;
   QString vtag = KST::suggestVectorName(QString( "(%1..%2)" ).arg( x0 ).arg( x1 ) );
 
-  xvector = new KstSVector(x0, x1, nx, vtag);
-  KST::addVectorToList( xvector );
+  xvector = new KstSVector(x0, x1, nx, KstObjectTag(vtag, QStringList(in_tag)));
 
   _doInterp = false;
   _xInVector = _inputVectors.insert(XINVECTOR, xvector);
@@ -81,7 +80,7 @@ KstEquation::KstEquation(const QDomElement &e)
 
   int ns = -1;
   double x0 = 0.0, x1 = 1.0;
-  QString xvtag;
+  KstObjectTag xvtag = KstObjectTag::invalidTag;
   bool haveVector = false;
 
   _doInterp = false;
@@ -102,7 +101,7 @@ KstEquation::KstEquation(const QDomElement &e)
       } else if (e.tagName() == "ns") {
         ns = e.text().toInt();
       } else if (e.tagName() == "xvtag") {
-        xvtag = e.text();
+        xvtag = KstObjectTag::fromString(e.text());
       } else if (e.tagName() == "xvector") {
         _inputVectorLoadQueue.append(qMakePair(XINVECTOR, e.text()));
         haveVector = true;
@@ -121,15 +120,14 @@ KstEquation::KstEquation(const QDomElement &e)
       x1 = x0 + 2;
     }
 
-    QString vtag;
-    if (xvtag.isEmpty()) {
-      vtag = KST::suggestVectorName(QString("(%1..%2)").arg(x0).arg(x1));
+    KstObjectTag vtag = KstObjectTag::invalidTag;
+    if (!xvtag.isValid()) {
+      vtag = KST::suggestUniqueVectorTag(KstObjectTag(QString("(%1..%2)").arg(x0).arg(x1), KstObjectTag::globalTagContext));
     } else {
       vtag = xvtag;
     }
 
     KstVectorPtr xvector = new KstSVector(x0, x1, ns, vtag);
-    KST::addVectorToList(xvector);
 
     _doInterp = false;
     _xInVector = _inputVectors.insert(XINVECTOR, xvector);
@@ -150,14 +148,12 @@ void KstEquation::commonConstructor(const QString& in_tag, const QString& in_equ
   _pe = 0L;
   _typeString = i18n("Equation");
   _type = "Equation";
-  KstObject::setTagName(in_tag);
+  KstObject::setTagName(KstObjectTag::fromString(in_tag));
 
-  KstVectorPtr xv = new KstVector(tagName()+"-xsv", 2, this);
-  KST::addVectorToList(xv);
+  KstVectorPtr xv = new KstVector(KstObjectTag("xsv", tag()), 2, this);
   _xOutVector = _outputVectors.insert(XOUTVECTOR, xv);
     
-  KstVectorPtr yv = new KstVector(tagName()+"-sv", 2, this);
-  KST::addVectorToList(yv);
+  KstVectorPtr yv = new KstVector(KstObjectTag("sv", tag()), 2, this);
   _yOutVector = _outputVectors.insert(YOUTVECTOR, yv);
 
   _isValid = false;
@@ -181,6 +177,8 @@ bool KstEquation::isValid() const {
 
 
 KstObject::UpdateType KstEquation::update(int update_counter) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   bool force = dirty();
   setDirty(false);
 
@@ -204,6 +202,8 @@ KstObject::UpdateType KstEquation::update(int update_counter) {
     }
   }
 
+  writeLockInputsAndOutputs();
+
   KstVectorPtr v = *_xInVector;
 
   xUpdated = KstObject::UPDATE == v->update(update_counter);
@@ -224,14 +224,39 @@ KstObject::UpdateType KstEquation::update(int update_counter) {
   }
   v->update(update_counter);
 
+  unlockInputsAndOutputs();
+
   return setLastUpdateResult(rc);
 }
 
 
+void KstEquation::reParse() {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
+  if (!_equation.isEmpty()) {
+    QMutexLocker ml(&Equation::mutex());
+    yy_scan_string(_equation.latin1());
+    ParsedEquation = 0L;
+    int rc = yyparse();
+    Equation::Node *en = static_cast<Equation::Node*>(ParsedEquation);
+    if (rc == 0 && en) {
+      if (en->takeVectorsAndScalars(VectorsUsed, ScalarsUsed)) {
+        _equation = en->text();
+        setupConnections();
+      } else {
+        kstdWarning() << "Equation [" << _equation << "] failed to find its vectors/scalars when re-parsing." << endl;
+      }
+    }
+    delete en;
+    ParsedEquation = 0L;
+  }
+}
+
+
 void KstEquation::save(QTextStream &ts, const QString& indent) {
   QString l2 = indent + "  ";
   ts << indent << "<equationobject>" << endl;
-  ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  ts << l2 << "<tag>" << QStyleSheet::escape(tag().tagString()) << "</tag>" << endl;
 
   // Reparse the equation, then write it back out in text so that we can update
   // any vectors or scalars that had name changes, but we don't get affected by
@@ -243,17 +268,20 @@ void KstEquation::save(QTextStream &ts, const QString& indent) {
     int rc = yyparse();
     Equation::Node *en = static_cast<Equation::Node*>(ParsedEquation);
     if (rc == 0 && en) {
-      if (!en->takeVectors(VectorsUsed)) {
-        KstDebug::self()->log(i18n("Equation [%1] failed to find its vectors when saving.  Resulting Kst file may have issues.").arg(_equation), KstDebug::Warning);
+      QString etext;
+      if (en->takeVectorsAndScalars(VectorsUsed, ScalarsUsed)) {
+        etext = en->text();
+      } else {
+        kstdWarning() << "Equation [" << _equation << "] failed to find its vectors/scalars when saving. The resulting Kst file may have issues." << endl;
+        etext = _equation;
       }
-      QString etext = en->text();
       ts << l2 << "<equation>" << QStyleSheet::escape(etext) << "</equation>" << endl;
     }
     delete en;
     ParsedEquation = 0L;
   }
 
-  ts << l2 << "<xvector>" << QStyleSheet::escape((*_xInVector)->tagName()) << "</xvector>" << endl;
+  ts << l2 << "<xvector>" << QStyleSheet::escape((*_xInVector)->tag().tagString()) << "</xvector>" << endl;
   if (_doInterp) {
     ts << l2 << "<interpolate/>" << endl;
   }
@@ -289,6 +317,7 @@ void KstEquation::setEquation(const QString& in_fn) {
       KstStringMap sm;
 
       if (_pe->collectObjects(VectorsUsed, ScalarsUsed, sm)) {
+        setupConnections();
         _pe->update(-1, &ctx);
       } else {
         //we have bad objects...
@@ -315,17 +344,13 @@ void KstEquation::setEquation(const QString& in_fn) {
 
 void KstEquation::setExistingXVector(KstVectorPtr in_xv, bool do_interp) {
   KstVectorPtr v = _inputVectors[XINVECTOR];
-  if (v) {
-    if (v == in_xv) {
-      return;
-    }
-    v->unlock();
+  if (v == in_xv) {
+    return;
   }
 
   setDirty();
 
   _inputVectors.erase(XINVECTOR);
-  in_xv->writeLock();
   _xInVector = _inputVectors.insert(XINVECTOR, in_xv);
 
   _ns = 2; // reset the updating
@@ -334,9 +359,15 @@ void KstEquation::setExistingXVector(KstVectorPtr in_xv, bool do_interp) {
 
 
 void KstEquation::setTagName(const QString &in_tag) {
-  KstObject::setTagName(in_tag);
-  (*_xOutVector)->setTagName(in_tag+"-xsv");
-  (*_yOutVector)->setTagName(in_tag+"-sv");
+  KstObjectTag newTag(in_tag, tag().context());  // FIXME: always the same context?
+
+  if (newTag == tag()) {
+    return;
+  }
+
+  KstObject::setTagName(newTag);
+  (*_xOutVector)->setTagName(KstObjectTag("xsv", tag()));
+  (*_yOutVector)->setTagName(KstObjectTag("sv", tag()));
 }
 
 
@@ -346,10 +377,12 @@ void KstEquation::setTagName(const QString &in_tag) {
 /*                                                                      */
 /************************************************************************/
 bool KstEquation::FillY(bool force) {
-  int v_shift, v_new;
-  int i0;
+  int v_shift=0, v_new;
+  int i0=0;
   int ns;
 
+  writeLockInputsAndOutputs();
+
   // determine value of Interp
   if (_doInterp) {
     ns = (*_xInVector)->length();
@@ -370,10 +403,12 @@ bool KstEquation::FillY(bool force) {
     KstVectorPtr yv = *_yOutVector;
     if (!xv->resize(_ns)) {
       // FIXME: handle error?
+      unlockInputsAndOutputs();
       return false;    
     }
     if (!yv->resize(_ns)) {
       // FIXME: handle error?
+      unlockInputsAndOutputs();
       return false;
     }
     yv->zero();
@@ -434,6 +469,7 @@ bool KstEquation::FillY(bool force) {
 
   if (!_pe) {
     if (_equation.isEmpty()) {
+      unlockInputsAndOutputs();
       return true;
     }
 
@@ -445,11 +481,13 @@ bool KstEquation::FillY(bool force) {
       Equation::FoldVisitor vis(&ctx, &_pe);
       KstStringMap sm;
       _pe->collectObjects(VectorsUsed, ScalarsUsed, sm);
+      setupConnections();
       ParsedEquation = 0L;
     } else {
       delete (Equation::Node*)ParsedEquation;
       ParsedEquation = 0L;
       _pe = 0L;
+      unlockInputsAndOutputs();
       return false;
     }
   }
@@ -462,8 +500,11 @@ bool KstEquation::FillY(bool force) {
 
   if (!(*_xOutVector)->resize(iv->length())) {
     // FIXME: handle error?
+    unlockInputsAndOutputs();
     return false;    
   }
+
+  unlockInputsAndOutputs();
   return true;
 }
 
@@ -473,8 +514,13 @@ QString KstEquation::propertyString() const {
 }
 
 
-void KstEquation::_showDialog() {
-  KstDialogs::self()->showEquationDialog(tagName());
+void KstEquation::showNewDialog() {
+  KstDialogs::self()->showEquationDialog();
+}
+
+
+void KstEquation::showEditDialog() {
+  KstDialogs::self()->showEquationDialog(tagName(), true);
 }
 
 
@@ -603,4 +649,16 @@ bool KstEquation::uses(KstObjectPtr p) const {
 }
 
 
+void KstEquation::setupConnections() {
+  for (KstScalarMap::iterator i = ScalarsUsed.begin(); i != ScalarsUsed.end(); ++i) {
+    disconnect(i.data(), SIGNAL(tagChanged()), this, SLOT(reParse()));
+    connect(i.data(), SIGNAL(tagChanged()), this, SLOT(reParse()));
+  } 
+  for (KstVectorMap::iterator i = VectorsUsed.begin(); i != VectorsUsed.end(); ++i) {
+    disconnect(i.data(), SIGNAL(tagChanged()), this, SLOT(reParse()));
+    connect(i.data(), SIGNAL(tagChanged()), this, SLOT(reParse()));
+  } 
+}
+
+#include "kstequation.moc"
 // vim: ts=2 sw=2 et
