@@ -31,6 +31,8 @@ namespace Equation {
 }
 
 class KST_EXPORT KstEquation : public KstDataObject {
+  Q_OBJECT
+
   public:
     KstEquation(const QString& in_tag, const QString& equation, double x0, double x1, int nx);
     KstEquation(const QString& in_tag, const QString& equation, KstVectorPtr xvector, bool do_interp);
@@ -44,6 +46,7 @@ class KST_EXPORT KstEquation : public KstDataObject {
 
     /** equations used to edit the vector */
     void setEquation(const QString &Equation);
+
     void setExistingXVector(KstVectorPtr xvector, bool do_interp);
 
     virtual QString equation() const { return _equation; }
@@ -57,7 +60,8 @@ class KST_EXPORT KstEquation : public KstDataObject {
 
     virtual void setTagName(const QString& tag);
 
-    virtual void _showDialog();
+    virtual void showNewDialog();
+    virtual void showEditDialog();
 
     virtual QString xVTag() const { return (*_xOutVector)->tagName(); }
     virtual QString yVTag() const { return (*_yOutVector)->tagName(); }
@@ -73,11 +77,15 @@ class KST_EXPORT KstEquation : public KstDataObject {
 
     virtual bool uses(KstObjectPtr p) const;
 
+  public slots:
+    void reParse();
+
   private:
     QString _equation;
 
     KstVectorMap VectorsUsed;
     KstScalarMap ScalarsUsed;
+    void setupConnections();
 
     void commonConstructor(const QString& in_tag, const QString& equation);
 
