@@ -33,7 +33,7 @@
 static const QString& THEMATRIX = KGlobal::staticQString("THEMATRIX");
 
 KstImage::KstImage(const QDomElement& e) : KstBaseCurve(e){
-  QString in_tag, in_matrixName, in_paletteName;
+  QString in_matrixName, in_paletteName;
   bool in_hasColorMap = false, in_hasContourMap = false;
   double in_zLower = 0, in_zUpper = 0;
   _autoThreshold = false;
@@ -44,7 +44,7 @@ KstImage::KstImage(const QDomElement& e) : KstBaseCurve(e){
     QDomElement e = n.toElement(); // try to convert the node to an element.
     if( !e.isNull() ) { // the node was really an element.
       if (e.tagName() == "tag") {
-        in_tag = e.text();
+        setTagName(KstObjectTag::fromString(e.text()));
       } else if (e.tagName() == "matrixtag") {
         in_matrixName = e.text();
       } else if (e.tagName() == "legend") {
@@ -75,8 +75,8 @@ KstImage::KstImage(const QDomElement& e) : KstBaseCurve(e){
 
   _inputMatrixLoadQueue.append(qMakePair(THEMATRIX, in_matrixName));
 
-  setTagName(in_tag);
   _typeString = i18n("Image");
+  _type = "Image";
   _hasColorMap = in_hasColorMap;
   _hasContourMap = in_hasContourMap;
   _zLower = in_zLower;
@@ -108,8 +108,9 @@ KstImage::KstImage(const QDomElement& e) : KstBaseCurve(e){
 KstImage::KstImage(const QString &in_tag, KstMatrixPtr in_matrix, double lowerZ, double upperZ, bool autoThreshold, KPalette* pal) : KstBaseCurve(){
 
   _inputMatrices[THEMATRIX] = in_matrix;
-  setTagName(in_tag);
+  setTagName(KstObjectTag(in_tag, KstObjectTag::globalTagContext));  // FIXME: always top-level?
   _typeString = i18n("Image");
+  _type = "Image";
   _zLower = lowerZ;
   _zUpper = upperZ;
   _autoThreshold = autoThreshold;
@@ -125,8 +126,9 @@ KstImage::KstImage(const QString &in_tag, KstMatrixPtr in_matrix, double lowerZ,
 //constructor for contour map only
 KstImage::KstImage(const QString &in_tag, KstMatrixPtr in_matrix, int numContours, const QColor& contourColor, int contourWeight) : KstBaseCurve(){
   _inputMatrices[THEMATRIX] = in_matrix;
-  setTagName(in_tag);
+  setTagName(KstObjectTag(in_tag, KstObjectTag::globalTagContext));  // FIXME: always top-level?
   _typeString = i18n("Image");
+  _type = "Image";
   _contourColor = contourColor;
   _numContourLines = numContours;
   _contourWeight = contourWeight;
@@ -150,8 +152,9 @@ KstImage::KstImage(const QString &in_tag,
                    const QColor& contourColor,
                    int contourWeight) {
   _inputMatrices[THEMATRIX] = in_matrix;
-  setTagName(in_tag);
+  setTagName(KstObjectTag(in_tag, KstObjectTag::globalTagContext));  // FIXME: always top-level?
   _typeString = i18n("Image");
+  _type = "Image";
   _contourColor = contourColor;
   _numContourLines = numContours;
   _contourWeight = contourWeight;
@@ -176,7 +179,7 @@ void KstImage::save(QTextStream &ts, const QString& indent) {
   ts << indent << "<image>" << endl;
   ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
   if (_inputMatrices.contains(THEMATRIX)) {
-    ts << l2 << "<matrixtag>" << QStyleSheet::escape(_inputMatrices[THEMATRIX]->tagName()) << "</matrixtag>" << endl;
+    ts << l2 << "<matrixtag>" << QStyleSheet::escape(_inputMatrices[THEMATRIX]->tag().tagString()) << "</matrixtag>" << endl;
   }
   ts << l2 << "<legend>" << QStyleSheet::escape(legendText()) << "</legend>" << endl;
   ts << l2 << "<hascolormap>" << _hasColorMap << "</hascolormap>" <<endl;
@@ -195,6 +198,8 @@ void KstImage::save(QTextStream &ts, const QString& indent) {
 
 
 KstObject::UpdateType KstImage::update(int update_counter) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   bool force = dirty();
   setDirty(false);
 
@@ -202,6 +207,8 @@ KstObject::UpdateType KstImage::update(int update_counter) {
     return lastUpdateResult();
   }
 
+  writeLockInputsAndOutputs();
+
   if (_inputMatrices.contains(THEMATRIX)) {
     KstMatrixPtr mp = _inputMatrices[THEMATRIX];
     bool updated = UPDATE == mp->update(update_counter);
@@ -221,16 +228,15 @@ KstObject::UpdateType KstImage::update(int update_counter) {
       _ns_minx = MinX;
       _ns_maxy = MaxY;
       _ns_miny = MinY;
-      MinPosY = MinY > 0 ? MinY : 0;
-      MinPosX = MinX > 0 ? MinX : 0;
-      
-      
+      MinPosY = MinY > 0 ? MinY : yStepSize;
+      MinPosX = MinX > 0 ? MinX : xStepSize;
+
       //recalculate the thresholds if necessary
       if (_autoThreshold) {
         _zLower = mp->minValue();
         _zUpper = mp->maxValue();
       }
-      
+
       //update the contour lines
       if (hasContourMap()) {
         double min = mp->minValue(), max = mp->maxValue();
@@ -242,16 +248,20 @@ KstObject::UpdateType KstImage::update(int update_counter) {
           }
         }
       }
+
+      unlockInputsAndOutputs();
       return setLastUpdateResult(UPDATE);
     }
   }
+
+  unlockInputsAndOutputs();
   return setLastUpdateResult(NO_CHANGE);
 }
 
 
 QString KstImage::propertyString() const {
   if (_inputMatrices.contains(THEMATRIX)) {
-    return i18n("Using matrix %1" ).arg(_inputMatrices[THEMATRIX]->tagName());
+    return i18n("Using matrix %1" ).arg(_inputMatrices[THEMATRIX]->tag().displayString());
   } else {
     return QString();
   }
@@ -287,26 +297,44 @@ void KstImage::setPalette(KPalette* pal) {
 }
 
 
-void KstImage::_showDialog() {
-  KstDialogs::self()->showImageDialog(tagName());
+void KstImage::showNewDialog() {
+  KstDialogs::self()->showImageDialog();
+}
+
+
+void KstImage::showEditDialog() {
+  KstDialogs::self()->showImageDialog(tagName(), true);
 }
 
 
 void KstImage::setUpperThreshold(double z) {
   setDirty();
-  _zLower = z;
+  _zUpper = z;
 }
 
 
 void KstImage::setLowerThreshold(double z) {
   setDirty();
-  _zUpper = z;
+  _zLower = z;
+}
+
+void KstImage::setThresholdToSpikeInsensitive(double per) {
+  if (per==0) {
+    setAutoThreshold(true);
+  } else {
+    matrix()->writeLock();
+    matrix()->calcNoSpikeRange(per);
+    matrix()->unlock();
+    setLowerThreshold(matrix()->minValueNoSpike());
+    setUpperThreshold(matrix()->maxValueNoSpike());
+    setAutoThreshold(false);
+  }
 }
 
 
 void KstImage::changeToColorOnly(const QString &in_tag, KstMatrixPtr in_matrix,
                                      double lowerZ, double upperZ, bool autoThreshold, KPalette* pal) {
-  setTagName(in_tag);
+  setTagName(KstObjectTag(in_tag, KstObjectTag::globalTagContext));  // FIXME: always top-level?
   if (_inputMatrices.contains(THEMATRIX)) {
     _inputMatrices[THEMATRIX] = in_matrix;
   }
@@ -325,7 +353,7 @@ void KstImage::changeToColorOnly(const QString &in_tag, KstMatrixPtr in_matrix,
 
 void KstImage::changeToContourOnly(const QString &in_tag, KstMatrixPtr in_matrix,
                                        int numContours, const QColor& contourColor, int contourWeight) {
-  setTagName(in_tag);
+  setTagName(KstObjectTag(in_tag, KstObjectTag::globalTagContext));  // FIXME: always top-level?
   if (_inputMatrices.contains(THEMATRIX)) {
     _inputMatrices[THEMATRIX] = in_matrix;
   }
@@ -346,7 +374,7 @@ void KstImage::changeToContourOnly(const QString &in_tag, KstMatrixPtr in_matrix
 void KstImage::changeToColorAndContour(const QString &in_tag, KstMatrixPtr in_matrix,
                                                double lowerZ, double upperZ, bool autoThreshold, KPalette* pal,
                                                int numContours, const QColor& contourColor, int contourWeight) {
-  setTagName(in_tag);
+  setTagName(KstObjectTag(in_tag, KstObjectTag::globalTagContext));  // FIXME: always top-level?
   if (_inputMatrices.contains(THEMATRIX)) {
     _inputMatrices[THEMATRIX] = in_matrix;
   }
@@ -459,7 +487,7 @@ KstDataObjectPtr KstImage::makeDuplicate(KstDataObjectDataObjectMap& duplicatedM
 
 QString KstImage::matrixTag() const {
   if (_inputMatrices.contains(THEMATRIX)) { 
-    return _inputMatrices[THEMATRIX]->tagName(); 
+    return _inputMatrices[THEMATRIX]->tag().displayString(); 
   } else {
     return QString();
   }
