@@ -67,6 +67,7 @@ KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y
   setLineStyle(0);
   setBarStyle(0);
   setPointDensity(0);
+  setInterp(InterpY);
 
   commonConstructor(in_tag, in_color);
   if (in_X) {
@@ -111,6 +112,7 @@ KstVCurve::KstVCurve(QDomElement &e)
   setLineStyle(0);
   setBarStyle(0);
   setPointDensity(0);
+  setInterp(InterpMax); // pre 1.4.0 files use old default.
 
   QDomNode n = e.firstChild();
   while (!n.isNull()) {
@@ -161,6 +163,8 @@ KstVCurve::KstVCurve(QDomElement &e)
         PointDensity = e.text().toInt();
       } else if (e.tagName() == "ignoreAutoScale") {
         _ignoreAutoScale = true;
+      } else if (e.tagName() == "interp") {
+	_interp = KstVCurve::InterpType(e.text().toInt());
       }
     }
     n = n.nextSibling();
@@ -194,7 +198,7 @@ void KstVCurve::commonConstructor(const QString &in_tag, const QColor &in_color)
   _typeString = i18n("Curve");
   _type = "Curve";
   Color = in_color;
-  setTagName(in_tag);
+  setTagName(KstObjectTag::fromString(in_tag));
   updateParsedLegendTag();
 }
 
@@ -204,6 +208,8 @@ KstVCurve::~KstVCurve() {
 
 
 KstObject::UpdateType KstVCurve::update(int update_counter) {
+  Q_ASSERT(myLockStatus() == KstRWLock::WRITELOCKED);
+
   bool force = dirty();
   setDirty(false);
 
@@ -217,6 +223,8 @@ KstObject::UpdateType KstVCurve::update(int update_counter) {
     return setLastUpdateResult(NO_CHANGE);
   }
 
+  writeLockInputsAndOutputs();
+
   bool depUpdated = force;
 
   depUpdated = UPDATE == cxV->update(update_counter) || depUpdated;
@@ -263,7 +271,24 @@ KstObject::UpdateType KstVCurve::update(int update_counter) {
     MinPosY = 0;
   }
 
-  NS = kMax(cxV->length(), cyV->length());
+  switch (interp()) {
+    case InterpMax:
+      NS = kMax(cxV->length(), cyV->length());
+      break;
+    case InterpMin:
+      NS = kMin(cxV->length(), cyV->length());
+      break;
+    case InterpX:
+      NS = cxV->length();
+      break;
+    case InterpY:
+      NS = cyV->length();
+      break;
+    default:
+      NS = kMax(cxV->length(), cyV->length());
+  }
+
+  unlockInputsAndOutputs();
 
   return setLastUpdateResult(depUpdated ? UPDATE : NO_CHANGE);
 }
@@ -385,57 +410,57 @@ void KstVCurve::getEYPoints(int i, double &x, double &y, double &eyminus, double
 }
 
 
-QString KstVCurve::xVTag() const {
+KstObjectTag KstVCurve::xVTag() const {
   KstVectorPtr xv = xVector();
   if (xv) {
-    return xv->tagName();
+    return xv->tag();
   }
-  return QString::null;
+  return KstObjectTag::invalidTag;
 }
 
 
-QString KstVCurve::yVTag() const {
+KstObjectTag KstVCurve::yVTag() const {
   KstVectorPtr yv = yVector();
   if (yv) {
-    return yv->tagName();
+    return yv->tag();
   }
-  return QString::null;
+  return KstObjectTag::invalidTag;
 }
 
 
-QString KstVCurve::xETag() const {
+KstObjectTag KstVCurve::xETag() const {
   KstVectorPtr v = xErrorVector();
   if (v) {
-    return v->tagName();
+    return v->tag();
   }
-  return "<None>";
+  return KstObjectTag::invalidTag;
 }
 
 
-QString KstVCurve::yETag() const {
+KstObjectTag KstVCurve::yETag() const {
   KstVectorPtr v = yErrorVector();
   if (v) {
-    return v->tagName();
+    return v->tag();
   }
-  return "<None>";
+  return KstObjectTag::invalidTag;
 }
 
 
-QString KstVCurve::xEMinusTag() const {
+KstObjectTag KstVCurve::xEMinusTag() const {
   KstVectorPtr v = xMinusErrorVector();
   if (v) {
-    return v->tagName();
+    return v->tag();
   }
-  return "<None>";
+  return KstObjectTag::invalidTag;
 }
 
 
-QString KstVCurve::yEMinusTag() const {
+KstObjectTag KstVCurve::yEMinusTag() const {
   KstVectorPtr v = yMinusErrorVector();
   if (v) {
-    return v->tagName();
+    return v->tag();
   }
-  return "<None>";
+  return KstObjectTag::invalidTag;
 }
 
 
@@ -463,21 +488,22 @@ void KstVCurve::save(QTextStream &ts, const QString& indent) {
   QString l2 = indent + "  ";
   ts << indent << "<curve>" << endl;
   ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
-  ts << l2 << "<xvectag>" << QStyleSheet::escape(_inputVectors[COLOR_XVECTOR]->tagName()) << "</xvectag>" << endl;
-  ts << l2 << "<yvectag>" << QStyleSheet::escape(_inputVectors[COLOR_YVECTOR]->tagName()) << "</yvectag>" << endl;
+  ts << l2 << "<xvectag>" << QStyleSheet::escape(_inputVectors[COLOR_XVECTOR]->tag().tagString()) << "</xvectag>" << endl;
+  ts << l2 << "<yvectag>" << QStyleSheet::escape(_inputVectors[COLOR_YVECTOR]->tag().tagString()) << "</yvectag>" << endl;
   ts << l2 << "<legend>" << QStyleSheet::escape(legendText()) << "</legend>" << endl;
+  ts << l2 << "<interp>" << interp() << "</interp>" << endl;
   ts << l2 << "<hasMinus/>" << endl;
   if (_inputVectors.contains(EXVECTOR)) {
-    ts << l2 << "<exVectag>" << QStyleSheet::escape(_inputVectors[EXVECTOR]->tagName()) << "</exVectag>" << endl;
+    ts << l2 << "<exVectag>" << QStyleSheet::escape(_inputVectors[EXVECTOR]->tag().tagString()) << "</exVectag>" << endl;
   }
   if (_inputVectors.contains(EYVECTOR)) {
-    ts << l2 << "<eyVectag>" << QStyleSheet::escape(_inputVectors[EYVECTOR]->tagName()) << "</eyVectag>" << endl;
+    ts << l2 << "<eyVectag>" << QStyleSheet::escape(_inputVectors[EYVECTOR]->tag().tagString()) << "</eyVectag>" << endl;
   }
   if (_inputVectors.contains(EXMINUSVECTOR)) {
-    ts << l2 << "<exMinusVectag>" << QStyleSheet::escape(_inputVectors[EXMINUSVECTOR]->tagName()) << "</exMinusVectag>" << endl;
+    ts << l2 << "<exMinusVectag>" << QStyleSheet::escape(_inputVectors[EXMINUSVECTOR]->tag().tagString()) << "</exMinusVectag>" << endl;
   }
   if (_inputVectors.contains(EYMINUSVECTOR)) {
-    ts << l2 << "<eyMinusVectag>" << QStyleSheet::escape(_inputVectors[EYMINUSVECTOR]->tagName()) << "</eyMinusVectag>" << endl;
+    ts << l2 << "<eyMinusVectag>" << QStyleSheet::escape(_inputVectors[EYMINUSVECTOR]->tag().tagString()) << "</eyMinusVectag>" << endl;
   }
   ts << l2 << "<color>" << Color.name() << "</color>" << endl;
   if (HasLines) {
@@ -583,12 +609,17 @@ KstCurveType KstVCurve::curveType() const {
 
 
 QString KstVCurve::propertyString() const {
-  return i18n("%1 vs %2").arg(yVTag()).arg(xVTag());
+  return i18n("%1 vs %2").arg(yVTag().displayString()).arg(xVTag().displayString());
 }
 
 
-void KstVCurve::_showDialog() {
-  KstDialogs::self()->showCurveDialog(tagName());
+void KstVCurve::showNewDialog() {
+  KstDialogs::self()->showCurveDialog();
+}
+
+
+void KstVCurve::showEditDialog() {
+  KstDialogs::self()->showCurveDialog(tagName(), true);
 }
 
 
@@ -1618,7 +1649,7 @@ void KstVCurve::yRange(double xFrom, double xTo, double* yMin, double* yMax) {
 
 KstDataObjectPtr KstVCurve::providerDataObject() const {
   KST::vectorList.lock().readLock();
-  KstVectorPtr vp = *KST::vectorList.findTag(yVTag());
+  KstVectorPtr vp = *KST::vectorList.findTag(yVTag().tag());  // FIXME: should use full tag
   KST::vectorList.lock().unlock();
   KstDataObjectPtr provider = 0L;
   if (vp) {
@@ -1709,4 +1740,14 @@ void KstVCurve::paintLegendSymbol(KstPainter *p, const QRect& bound) {
 }
 
 
+KstVCurve::InterpType KstVCurve::interp() const {
+  return (_interp);
+}
+
+void KstVCurve::setInterp(KstVCurve::InterpType itype) {
+  _interp = itype;
+  setDirty();
+}
+
+
 // vim: ts=2 sw=2 et
