@@ -71,10 +71,13 @@ Chunk::~Chunk() {
   delete next;
   delete up;
   delete down;
+  delete group;
+  group = 0L;
   if (prev) {
     switch (vOffset) {
       case None:
         prev->next = 0L;
+        // Note: kind of does the wrong thing if we're a group...  no issue though
         break;
       case Up:
         prev->up = 0L;
@@ -524,20 +527,16 @@ static Chunk *parseInternal(Chunk *ctail, const QString& txt, uint& start, uint
       case 0x5e:   // ^
         dir = Chunk::Up;
       case 0x5f:   // _
-        if (ctail->text.isEmpty() && !ctail->group) {
-          setNormalChar(c, &ctail);
-        } else {
-          if (ctail->vOffset != Chunk::None) {
-            if (ctail->vOffset != dir) {
-              ctail = new Chunk(ctail->prev, dir, false, true);
-            } else if (ctail->group) {
-              ctail = new Chunk(ctail, dir, false, true);
-            } else {
-              return 0L; // parse error - x^y^z etc
-            }
-          } else {
+        if (ctail->vOffset != Chunk::None) {
+          if (ctail->vOffset != dir) {
+            ctail = new Chunk(ctail->prev, dir, false, true);
+          } else if (ctail->group) {
             ctail = new Chunk(ctail, dir, false, true);
+          } else {
+            return 0L; // parse error - x^y^z etc
           }
+        } else {
+          ctail = new Chunk(ctail, dir, false, true);
         }
         break;
       case 0x7b:   // {
