@@ -318,12 +318,12 @@ int main(int argc, char **argv) {
   test("257|-1", 0.0, -1);
 
   // Scalars
-  new KstScalar("test1", 0L, 1.0, true);
-  new KstScalar("test2", 0L, 0.0, true);
-  new KstScalar("test3", 0L, -1.0, true);
-  new KstScalar("test4", 0L, NOPOINT, true);
-  new KstScalar("test5", 0L, INF, true);
-  new KstScalar("test6", 0L, -INF, true);
+  new KstScalar(KstObjectTag("test1"), 0L, 1.0, true);
+  new KstScalar(KstObjectTag("test2"), 0L, 0.0, true);
+  new KstScalar(KstObjectTag("test3"), 0L, -1.0, true);
+  new KstScalar(KstObjectTag("test4"), 0L, NOPOINT, true);
+  new KstScalar(KstObjectTag("test5"), 0L, INF, true);
+  new KstScalar(KstObjectTag("test6"), 0L, -INF, true);
 
   test("[test1]", 0.0, 1.0);
   test("[test4]", 0.0, NOPOINT);
@@ -335,12 +335,12 @@ int main(int argc, char **argv) {
   test("[=10+10]", 0.0, 20.0);
   
   // Vectors
-  KstVector::generateVector(0, 1.0, 10, "1");
-  KstVector::generateVector(0, 1.0, 10, "V1");
-  KstVector::generateVector(1.0, 2.0, 10, "V2");
-  KstVector::generateVector(0, 1.0, 2, "V3");
-  KstVector::generateVector(-1.0, 1.0, 1000, "V4");
-  KstVector::generateVector(-1.0, 1.0, 1000, "V5-%+-_!");
+  KstVector::generateVector(0, 1.0, 10, KstObjectTag("1"));
+  KstVector::generateVector(0, 1.0, 10, KstObjectTag("V1"));
+  KstVector::generateVector(1.0, 2.0, 10, KstObjectTag("V2"));
+  KstVector::generateVector(0, 1.0, 2, KstObjectTag("V3"));
+  KstVector::generateVector(-1.0, 1.0, 1000, KstObjectTag("V4"));
+  KstVector::generateVector(-1.0, 1.0, 1000, KstObjectTag("V5-%+-_!"));
   test("[V2] - [V1]", 0.0, 1.0);
   test("[V2[9]]", 0.0, 2.0);
   test("[V2[5+4]]", 0.0, 2.0);
@@ -352,7 +352,7 @@ int main(int argc, char **argv) {
   test("2*plugin(bin, [V4], 12)", 1.0, -1.9779779779779778);
   test("4*plugin(bin, [V4], x)", 5.0, -3.9839839839839839);
   test("-3*plugin(bin, x, 12)", 2.0, NOPOINT);
-  xVector = KstVector::generateVector(0, 100, 2000, "XVector");
+  xVector = KstVector::generateVector(0, 100, 2000, KstObjectTag("XVector"));
   test("-3*plugin(bin, x, 12)", 2.0, -0.8254127063531767);
   test("-3*plugin(bin, y, 12)", 2.0, NOPOINT);
 
