@@ -35,7 +35,7 @@ void testAssert(bool result, const QString& text = "Unknown") {
   } } while(0)
 
 void doTests() {
-  KstVectorPtr vp = KstVector::generateVector(0, 10, 100, "V1");
+  KstVectorPtr vp = KstVector::generateVector(0, 10, 100, KstObjectTag("V1"));
   KstHistogramPtr h1 = new KstHistogram("H1", vp, 0, 10, 10, KST_HS_NUMBER);
   KST::dataObjectList.append(h1.data());
   doTest(h1->propertyString() == "Histogram: V1");
